<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Controllers
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: ContentController.php 1554 2010-10-18 13:54:22Z ms $
 */
class Sicore_ContentController extends Sicore_Controllers_Simploo
{
    const ID_PREFIX   = 'si_tree_cobjects_';  
    const MODULE      = 'Sicore'; 
       
    public function init()
    {
        $this->_helper->contextSwitch()->addActionContext('save',         'json');
        $this->_helper->contextSwitch()->addActionContext('move',         'json');
        $this->_helper->contextSwitch()->addActionContext('show',         'json');
        $this->_helper->contextSwitch()->addActionContext('hide',         'json'); 
        $this->_helper->contextSwitch()->addActionContext('edit',         'json'); 
        $this->_helper->contextSwitch()->addActionContext('save',         'json'); 
        $this->_helper->contextSwitch()->addActionContext('editpanel',    'json');
        $this->_helper->contextSwitch()->addActionContext('elements',     'json'); 
        $this->_helper->contextSwitch()->addActionContext('editorconfig', 'json');  
        $this->_helper->contextSwitch()->addActionContext('choosertree', 'json');
        $this->_helper->contextSwitch()->addActionContext('choosercontent', 'json');
        parent::init();
    }
    
    public function saveAction()
    {
        $data = array();
        $data['saved'] = false;
        
        if($this->getRequest()->isPost()){
            $formData = $this->getRequest()->getPost();
            
            $mNodes = new Sicore_Models_Server_Db_Nodes();
            $node = $mNodes->setDraft(true)
                           ->doCheckParents(false) 
                           ->doCheckIsDeleted(false)
                           ->doCheckIsActive(false)
                           ->doCheckDate(false)
                           ->get(intval($formData['uid']), $formData['language'], true);
            if(isset($formData['si_content_property_changed'])){
                foreach($formData['si_content_property_changed'] as $key => $value){
                    $node->$key->set(trim(base64_decode($value)));                   
                }
                if($mNodes->update($node)){
                    $data['saved'] = true;
                }
            }
        }
        
        echo Zend_Json::encode($data);
        die();
    }
    
    public function moveAction()
    {
        $data = array(); 
        $sorting = $this->getRequest()->getParam('sorting');    
        $page    = intval($this->getRequest()->getParam('page'));
        if($sorting && $page){
            $sorting = explode(',', $sorting);
            $idlist = array();
            foreach($sorting as $id){
                $idlist[] = str_replace('si_content_', '', $id);
                
            }
            
            $mNodes = new Sicore_Models_Server_Db_Nodes();
            $nodes  = $mNodes->getByParentUid($page, array('Sicore_Models_Node_Content'), array(), true);
        
            $minSorting = -1;
            
            foreach($nodes as $key=>$node){
                if(array_search($node->uid->value(), $idlist) === false){
                    continue;
                }     
                $sorting    = $nodes[$key]->getTree()->sorting->value(); 
                $minSorting = $minSorting > -1 && $sorting > $minSorting ? $minSorting : $sorting; 
            }
            foreach($idlist as $position => $uid){
                foreach($nodes as $key=>$node){
                    if($nodes[$key]->uid->value() != $uid) continue;
                    $nodes[$key]->getTree()->sorting->set($position + $minSorting);   
                    break;
                }       
            }  
            $mNodes->updateContentSorting($nodes, $page);
            $cache = Sicore_Models_Registry::getCache();
            
            $cache->clean(
                Zend_Cache::CLEANING_MODE_MATCHING_TAG,
                array('content', (string)($page))
            );
        }
        $data['status'] = true;
        
        if($this->getRequest()->getparam('getversions')){
            //$mVersions = new Sicore_Models_Server_Db_Version();
            //$this->view->versions = $mVersions->getByPage($page);
            $data['versions']     = count($this->view->versions) ? $this->view->render('menu/versions.phtml') : ''; 
        }
        
        echo Zend_Json::encode($data);
        die();
    }
    
    public function movedownAction()
    {
        $mNodes = new Sicore_Models_Server_Db_Nodes(); 
        
        $uid  = intval($this->getRequest()->getParam('uid'));
        try{
            $node = $mNodes->getByUid($uid);
        }catch(Exception $e){
            echo Zend_Json::encode(false);
            die();
        }
        
        $return = $mNodes->movedown($node);
        
        $cache = Sicore_Models_Registry::getCache();
        $cache->clean(
            Zend_Cache::CLEANING_MODE_MATCHING_TAG,
            array('content', (string)($node->getTree()->parent->value()))
        );
        
        echo Zend_Json::encode($return);
        die();
    }
    
    public function moveupAction()
    {
        $mNodes = new Sicore_Models_Server_Db_Nodes(); 
        
        $uid  = intval($this->getRequest()->getParam('uid'));
        try{
            $node = $mNodes->getByUid($uid);
        }catch(Exception $e){
            echo Zend_Json::encode(false);
            die();
        }
        
        $return = $mNodes->moveup($node);
        
        $cache = Sicore_Models_Registry::getCache();
        $cache->clean(
            Zend_Cache::CLEANING_MODE_MATCHING_TAG,
            array('content', (string)($node->getTree()->parent->value()))
        );
        
        echo Zend_Json::encode($return);
        die();
    }
    
    public function showAction()
    {
        $data = array();
        $data['saved'] = false;
        
        $config_fe = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND);
        $uid  = intval($this->getRequest()->getParam('uid'));
        $page = intval($this->getRequest()->getParam('page'));
        
        if(!($lang = $this->getRequest()->getParam('lang'))){
            $lang = $config_fe->languages->{0};       
        }
        
        $mNodes = new Sicore_Models_Server_Db_Nodes();
        $node   = $mNodes->doCheckParents(false)
                         ->get($uid, $lang, true);       

         if($node->isHidden->value()){
            $node->isHidden->set(false);
            $mNodes->update($node);
            $data['status'] = true;
        }else{
            $data['status'] = false;
        }
        if($this->getRequest()->getparam('getversions') && $data['status']){
            //$mVersions = new Sicore_Models_Server_Db_Version();
            //$this->view->versions = $mVersions->getByPage($page);
            $data['versions']     = count($this->view->versions) ? $this->view->render('menu/versions.phtml') : ''; 
        }
        
        echo Zend_Json::encode($data);
        die();
    }
    
    public function hideAction()
    {
        $data = array();
        $data['saved'] = false;
        
        $config_fe = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND);
        $uid = intval($this->getRequest()->getParam('uid'));
        $page = intval($this->getRequest()->getParam('page'));
        
        if(!($lang = $this->getRequest()->getParam('lang'))){
            $lang = $config_fe->languages->{0};       
        }
        
        $mNodes = new Sicore_Models_Server_Db_Nodes();
        $node   = $mNodes->doCheckParents(false)
                         ->get($uid, $lang, true);
        
        if(!$node->isHidden->value()){
            $node->isHidden->set(true);
            $mNodes->update($node);
            $data['status'] = true;
        }else{
            $data['status'] = false;
        }
        if($this->getRequest()->getParam('getversions') && $data['status']){
            $data['versions']     = count($this->view->versions) ? $this->view->render('menu/versions.phtml') : ''; 
        }
        
        echo Zend_Json::encode($data);
        die();
    }
    
    public function choosertreeAction()
    {
        $mSettings = new Sicore_Models_Server_Db_Settings();
        $settings  = $mSettings->getByParent('contentelements');
        
        $data      = array();
        $icon      = null;
        foreach($settings as $setting){
            $keys   = explode('_', $setting->uid);
            $icon   = empty($setting->action) ? 'icon_container.gif' : 'icon_settings.gif'; 
            
            $data[] = array('attributes' => array('id'              => self::ID_PREFIX.$setting->uid,
                                                  'uid'             => $setting->uid, 
                                                  'parent'          => isset($keys[1]) ? $keys[1] : '',
                                                  'title'           => isset($setting->title) ? $this->view->translate($setting->title) : $this->view->translate(strtoupper('settings_'.$setting->uid)),
                                                  'metadata'        => array("draggable" => isset($setting->draggable) ? (bool)$setting->draggable : false), 
                                                  'action'          => $setting->action,
                                                  'object'          => isset($setting->object) ? $setting->object : '', 
                                                  'controller'      => $setting->controller,  
                                                  'module'          => $setting->module ? $setting->module : self::MODULE,  
                                                ),         
                              'state'      => 'opened',
                              'data'       => array('title' => isset($setting->title) ? $this->view->translate($setting->title) : $this->view->translate(strtoupper('settings_'.$setting->uid)), 
                                                    'icon'  => $icon
                                                    ) 
                              );  
        }
        
        if($this->_helper->contextSwitch()->getCurrentContext() == 'json'){
            echo Zend_Json::encode($data);
            die();
        } 
    }
    
    public function choosercontentAction()
    {
        $name = $this->getRequest()->getParam('uid');
        $path = Sicore_Models_Helper_FSO::joinPathAbs('application/Sicore/Models/Node/Content'); 
        
        $status = false;
        
        if(strpos($name, '_') === false){
            $configContent = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_CONTENT); 
            if(isset($configContent->$name)){
                $element = new stdClass();
                $element->uid        = $name; 
                $element->title      = $configContent->$name->title;   
                $element->object     = 'Sicore_Models_Node_Content_Interface';  
                $status = true; 
            } 
        }else{
            $names = explode('_', $name);
            $name  = $names[count($names) - 1];
            if(file_exists($path.$name.'.php')){
                $element = new stdClass();
                $element->uid        = 'Sicore_Models_Node_Content_'.$name; 
                $element->title      = strtoupper('Sicore_Models_Node_Content_'.$name);
                $element->object     = 'Sicore_Models_Node_Content_Interface';  
                $status = true;   
            }
        }        
        $this->view->cobject = $element; 

        $content = $this->view->render('chooser/cobjects.phtml'); 
        
        $data    = array('status'  => $status,
                         'content' => $content
                        );
                        
        die(Zend_Json::encode($data));
    }
    
    public function editAction()
    {
        $data      = array();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $uid    = intval($this->getRequest()->getParam('uid'));
          
        $mNodes = new Sicore_Models_Server_Db_Nodes();
        $node   = $mNodes->setDraft(true)
                         ->doCheckParents(false) 
                         ->doCheckIsDeleted(false)
                         ->doCheckIsActive(false)
                         ->doCheckDate(false)
                         ->getByUid($uid);
        $form   = new Sicore_Models_Gui_Form_Backend($node, $this->user);
                              
        if($this->getRequest()->isPost()){
            $formData = $this->getRequest()->getPost();               
            $data     = array();
            if (!$form->isValid($formData)) {
                $errors = $form->getBackendErrors();   
                $data['saved']  = false;
                $data['errors']    = $errors['errors'];
                $data['fields']    = $errors['fields'];
                $data['subforms']  = $errors['subformIndexs'];    
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT'); 
            }else{
                $elements = $form->getElementsToSave();
                $node->init($elements);
 
                if($mNodes->update($node)){
                    $data['saved'] = true;
                    $data['uid']   = $node->uid->value(); 
                    $data['mode']  = $this->getRequest()->getParam('mode'); 
                    $data['msg_title'] = $translate->_('GENERAL_FORM_SAVED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_SAVED_TEXT');
                }else{  
                    $data['saved']  = false;
                    $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_UNKOWN'); 
                } 
            }
        }else{
            
            $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);
            $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path; 
            $form->setAction($path.'sicore/content/edit');
            
            $data = array();
            $data['labelSave']      = $this->view->translate('GENERAL_SAVE');
            $data['labelCancel']    = $this->view->translate('GENERAL_CANCEL');   
            $data['action']         = $path.'sicore/content/save';
            $data['title']          = $this->view->translate('GENERAL_EDIT_CONTENT');
            $data['content']        = $form->render();
            $data['scripts']        = $this->view->jQuery()->getOnLoadActions();
            $data['scripts'][]      = 'var oldvals1 = $(\'#si_edit_main\').trackChanges({excludeClass: \'si_notrack\', changeListName: "si_edit_main_changed", events: "change blur keypress keydown click"});';
        
        }
        echo Zend_Json::encode($data);
        die();
    }
    
    
    
    public function editpanelAction()
    {
        $data['content'] = $this->view->render('content/editpanel.phtml');
        echo Zend_Json::encode($data);
        die();    
    }
    
    public function elementsAction()
    {
        $pageUid    = intval($this->getRequest()->getParam('page'));
        $contentUid = intval($this->getRequest()->getParam('content')); 
        $lang       = $this->getRequest()->getParam('lang', 'de');
        
        $mNode    = new Sicore_Models_Server_Db_Nodes();
        
        $page = $mNode->get($pageUid, $lang, true);
        $elements = array();   
        foreach($page->contentelements->toArray() as $contentelement){
            try{
                if(!class_exists($contentelement)){
                    $configContent = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_CONTENT);
                    $element = new Sicore_Models_Node_Content(null, $contentelement);
                }else{
                    $element = new $contentelement();
                }
                if(!($element instanceof Sicore_Models_Node_Content)) throw new Exception('\''.$contentelement.'\' keine Instanz von Sicore_Models_Node_Content'); 
                $elements[] = $element;
            }catch(Exception $e){
                Sicore_Models_Logger::exception($e); 
            }
        }
        
        $this->view->contentelements = $elements; 
        $data['content'] = $this->view->render('content/elements.phtml');
       
        echo Zend_Json::encode($data);
        die();    
    }
     
     public function tableeditAction()
     {
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);                   
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path; 
        $form   = new Sicore_Models_Gui_Form_Tableedit($this->getRequest()->getParam('cols'));
        
        $form->setAction($path.'sicore/media/tableedit');
        $data = array();
        $data['labelInsert']   = $this->view->translate('GENERAL_SAVE');
        $data['labelCancel']   = $this->view->translate('GENERAL_CANCEL');
        $data['title']      = $this->view->translate('GENERAL_TABLEEDIT');  
        $data['content']    = $form->render();     
        $data['scripts']    = $this->view->jQuery()->getOnLoadActions(); 
        echo Zend_Json::encode($data);
        die();   
     }
     
     public function editorconfigAction()
     {
        $templatePath = Sicore_Models_Helper_FSO::getLayoutPathAbs();
        $config_editor = new Simploo_Config_Ini(APPLICATION_PATH . '/config/system/editor.ini.php',     APPLICATION_PATH . '/config/custom/editor.ini.php',  $templatePath."editor.ini.php", null, array('allowModifications' => true)); 
        echo Zend_Json::encode($config_editor->toArray());
        die();
     }
}