<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Controllers
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @license    http://www.simploo.de
 * @version    $Id: SettingsController.php 1554 2010-10-18 13:54:22Z ms $
 */
class Sicore_SettingsController extends Sicore_Controllers_Simploo
{
    const ID_PREFIX   = 'si_tree_settings_';  
    const MODULE      = 'Sicore'; 
    
    public function init()
    {
        $this->_helper->contextSwitch()->addActionContext('tree', 'json');
        $this->_helper->contextSwitch()->addActionContext('load', 'json');
        $this->_helper->contextSwitch()->addActionContext('generaldata', 'json');  
        $this->_helper->contextSwitch()->addActionContext('generallicence', 'json'); 
        parent::init();
    }
    
    public function treeAction()
    {
        $mPages = new Sicore_Models_Gui_Menu_Settings();
        $mPages->appendFilesToView();
        $mPages->render(); 
    }
    
    public function loadAction()
    {
        
        $parent    = $this->getRequest()->getParam('uid');
        $isDraft   = (bool)$this->getRequest()->getParam('draft',  false); 
        
        $mSettings = new Sicore_Models_Server_Db_Settings();
        $settings  = $mSettings->getByParent($parent);
          
        $editable  = in_array('Sicore_Models_Gui_Menu_Settings', $this->user->gui_menu_enabled->toArray()) || $this->user->isadmin->value();
        $data      = array();
        $icon      = null;
        foreach($settings as $setting){
            $keys   = explode('_', $setting->uid);
            $icon   = empty($setting->action) ? 'icon_container.gif' : 'icon_settings.gif'; 
            
            $data[] = array('attributes' => array('id'              => self::ID_PREFIX.$setting->uid,
                                                  'uid'             => $setting->uid, 
                                                  'parent'          => isset($keys[1]) ? $keys[1] : '',
                                                  'title'           => isset($setting->title) ? $this->view->translate($setting->title) : $this->view->translate(strtoupper('settings_'.$setting->uid)),
                                                  'metadata'        => array("draggable" => isset($setting->draggable) ? (bool)$setting->draggable : false), 
                                                  'action'          => $setting->action,
                                                  'object'          => isset($setting->object) ? $setting->object : '', 
                                                  'controller'      => $setting->controller,  
                                                  'module'          => $setting->module ? $setting->module : self::MODULE,  
                                                  'isDraft'         => intval($isDraft),  
                                                  'canEdit'         => intval($editable && !empty($setting->action)  && $isDraft),
                                                  'canShow'         => intval(!empty($setting->action)),
                                                ),         
                              'state'      => 'closed',
                              'data'       => array('title' => isset($setting->title) ? $this->view->translate($setting->title) : $this->view->translate(strtoupper('settings_'.$setting->uid)), 
                                                    'icon'  => $icon
                                                    ) 
                              );  
        }
        
        if($this->_helper->contextSwitch()->getCurrentContext() == 'json'){
            echo Zend_Json::encode($data);
            die();
        } 
    }
    
    public function generaldataAction()
    {
        $editmode = $this->getRequest()->getParam('editmode');
        $data = array();
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path; 
        $form = new Sicore_Models_Gui_Form_Settings_Generaldata($this->config_base, $this->user, $editmode);
        
        $form->setAction($path.'sicore/settings/generaldatasave');
        
        if($editmode){        
            $data['button1']['label']   = $this->view->translate('GENERAL_SAVE');
            $data['button1']['onclick'] = "$('#si_edit_main').submit();"; 
            $data['button1']['bold']    = true;
            $data['button2']['label']   = $this->view->translate('GENERAL_CANCEL');
            $data['button2']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 0, 0)';     
        }else{
            $data['button1']['label']   = $this->view->translate('GENERAL_EDIT');
            $data['button1']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 1, 0)'; 
        }
        $data['title']      = $this->view->translate('SETTINGS_GENERAL');
        $data['content']    = $form->render();
        $data['scripts']    = $this->view->jQuery()->getOnLoadActions();
        $data['scripts'][]  = 'var oldvals1 = $(\'#si_edit_main\').trackChanges({excludeClass: \'si_notrack\', changeListName: "si_edit_main_changed", events: "change blur keypress keydown click"});';   
        
        echo Zend_Json::encode($data);
        die();    
    }
    
    public function generaldatasaveAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        if($this->getRequest()->isPost()){
            $formData= $this->getRequest()->getPost(); 
            $form = new Sicore_Models_Gui_Form_Settings_Generaldata($this->config_base, $this->user, false);
            if(!$form->isValid($formData)){
                $errors = $form->getBackendErrors();
                $data['saved']     = false;
                $data['errors']    = $errors['errors'];
                $data['fields']    = $errors['fields'];
                $data['subforms']  = $errors['subformIndexs'];
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT'); 
            }else{
                if(file_exists(APPLICATION_PATH . '/config/custom/base.ini.php')){
                    $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
                    $data   = $config->toArray();
                }else{
                    $data   = array();
                }
                $elements = $form->getElementsToSave();
                $data['app']['maintenance']['status']   = (bool)$elements['maintenance']; 
                $data['customer']['name']               = $elements['name'];
                $data['customer']['email']              = $elements['email'];   
                
                try{
                    $writer = new Simploo_Config_Writer_Ini();
                    $writer->write(APPLICATION_PATH . '/config/custom/base.ini.php', new Zend_Config($data, true));
                    $data['saved']     = true; 
                    $data['msg_title'] = $translate->_('GENERAL_FORM_SAVED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_SAVED_TEXT');
                }catch(Zend_Config_Exception $e){
                    Sicore_Models_Logger::exception($e);
                    $data['saved']     = false;
                    $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_CONFIGWRITE');
                }
                
            } 
        }
        
        echo Zend_Json::encode($data);
        die();    
    }
    
    public function generallicenceAction()
    {
        $editmode = $this->getRequest()->getParam('editmode');
        $data = array();
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path; 
        $form = new Sicore_Models_Gui_Form_Settings_Generallicence($this->config_base, $this->user, $editmode);
        
        $form->setAction($path.'sicore/settings/generallicencesave');
        
        if($editmode){        
            $data['button1']['label']   = $this->view->translate('GENERAL_ACTIVATE');
            $data['button1']['onclick'] = "$('#si_edit_main').submit();";
            $data['button1']['bold']    = true;
            $data['button2']['label']   = $this->view->translate('GENERAL_CANCEL');
            $data['button2']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 0, 0)';       
        }else{
            $data['button1']['label']   = $this->view->translate('GENERAL_ACTIVATE');
            $data['button1']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 1, 0)'; 
        }
        
        $data['title']      = $this->view->translate('SETTINGS_GENERAL');
        $data['content']    = $form->render();
        $data['scripts']    = $this->view->jQuery()->getOnLoadActions();
        $data['scripts'][]  = 'var oldvals1 = $(\'#si_edit_main\').trackChanges({excludeClass: \'si_notrack\', changeListName: "si_edit_main_changed", events: "change blur keypress keydown click"});';
        
        if($this->_helper->contextSwitch()->getCurrentContext() == 'json'){
            echo Zend_Json::encode($data);
            die();
        }     
    }
    
    public function generallicencesaveAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        if($this->getRequest()->isPost()){
            $formData= $this->getRequest()->getPost(); 
                     
            $form = new Sicore_Models_Gui_Form_Settings_Generallicence($this->config_base, null, $this->user, false);
            if(!$form->isValid($formData)){
                $errors = $form->getBackendErrors();
                $data['saved']     = false;
                $data['errors']    = $errors['errors'];
                $data['fields']    = $errors['fields'];
                $data['subforms']  = $errors['subformIndexs'];
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT'); 
            }else{
                $elements = $form->getElementsToSave(); 
                if(($ret = Sicore_Models_Core::ac($elements['activationcode'])) > 0){
                    if(file_exists(APPLICATION_PATH . '/config/custom/base.ini.php')){
                        $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
                        $data   = $config->toArray();
                    }else{
                        $data   = array();
                    }
                    $elements = $form->getElementsToSave();
                    $codes    = Sicore_Models_Core::ae($elements['activationcode']); 
                    $data['app']['number']      = $codes[0];  
                    $data['app']['serial']      = $codes[1];
                    $data['customer']['secret'] = $codes[2];   
                    try{
                        $writer = new Simploo_Config_Writer_Ini();
                        $writer->write(APPLICATION_PATH . '/config/custom/base.ini.php', new Zend_Config($data, true));
                        $data['saved']     = true; 
                        $data['scripts'][] = "$('#si_deved').hide();";
                        $data['msg_title'] = $translate->_('GENERAL_ACTIVATION_DONE1');
                        $data['msg_text']  = $translate->_('GENERAL_ACTIVATION_DONE2');
                    }catch(Zend_Config_Exception $e){
                        Sicore_Models_Logger::exception($e);
                        $data['saved']     = false;
                        $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                        $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_CONFIGWRITE');
                    }  
                }else{
                    $data['saved']     = false; 
                    $data['msg_title'] = $translate->_('GENERAL_ACTIVATION_FAILED');
                    $data['msg_text']  = $translate->_('GENERAL_ACTIVATION_FAILED'.$ret);
                }  
            } 
        }
        
        echo Zend_Json::encode($data);
        die();    
    }
    
    public function generalsystemAction()
    {
        $translate = Sicore_Models_Registry::getTranslate();  

        ob_start(); 
        phpinfo();
        $content = ob_get_contents();
        ob_end_clean();
        
        $data['title']   = $translate->_('SETTINGS_GENERAL_SYSTEM');
        $data['content'] = $content;

        echo Zend_Json::encode($data);
        die(); 
    }
    
    public function loggingoptionsAction()
    {
        $editmode = $this->getRequest()->getParam('editmode');
        $data = array();
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path; 
        $form = new Sicore_Models_Gui_Form_Settings_Loggingoptions($this->config_base, $this->user, $editmode);
        
        $form->setAction($path.'sicore/settings/loggingoptionssave');
        
        if($editmode){        
            $data['button1']['label']   = $this->view->translate('GENERAL_SAVE');
            $data['button1']['onclick'] = "$('#si_edit_main').submit();";
            $data['button1']['bold']    = true;
            $data['button2']['label']   = $this->view->translate('GENERAL_CANCEL');
            $data['button2']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 0, 0)';      
        }else{
            $data['button1']['label']   = $this->view->translate('GENERAL_EDIT');
            $data['button1']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 1, 0)'; 
        }
        $data['title']      = $this->view->translate('SETTINGS_LOGGING');
        $data['content']    = $form->render();
        $data['scripts']    = $this->view->jQuery()->getOnLoadActions();
        $data['scripts'][]  = 'var oldvals1 = $(\'#si_edit_main\').trackChanges({excludeClass: \'si_notrack\', changeListName: "si_edit_main_changed", events: "change blur keypress keydown click"});';   
        
        echo Zend_Json::encode($data);
        die();    
    }
    
    public function loggingoptionssaveAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        if($this->getRequest()->isPost()){
            $formData= $this->getRequest()->getPost(); 
            $form = new Sicore_Models_Gui_Form_Settings_Loggingoptions($this->config_base, $this->user, false);
            if(!$form->isValid($formData)){
                $errors = $form->getBackendErrors();
                $data['saved']     = false;
                $data['errors']    = $errors['errors'];
                $data['fields']    = $errors['fields'];
                $data['subforms']  = $errors['subformIndexs'];
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT'); 
            }else{
                if(file_exists(APPLICATION_PATH . '/config/custom/base.ini.php')){
                    $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
                    $data   = $config->toArray();
                }else{
                    $data   = array();
                }
                $elements = $form->getElementsToSave();
                $data['logging']['info']        = (bool)$elements['information'];  
                $data['logging']['error']       = (bool)$elements['error'];
                $data['logging']['exception']   = (bool)$elements['exception'];   
                $data['logging']['feout']       = (bool)$elements['feout']; 
                try{
                    $writer = new Simploo_Config_Writer_Ini();
                    $writer->write(APPLICATION_PATH . '/config/custom/base.ini.php', new Zend_Config($data, true));
                    $data['saved']     = true; 
                    $data['msg_title'] = $translate->_('GENERAL_FORM_SAVED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_SAVED_TEXT');
                }catch(Zend_Config_Exception $e){
                    Sicore_Models_Logger::exception($e);
                    $data['saved']     = false;
                    $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_CONFIGWRITE');
                }
                
            } 
        }
        
        echo Zend_Json::encode($data);
        die();    
    }
    
    public function loggingshowAction(){
        $data = array();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $logFile   = $this->getRequest()->getParam('log');
        switch($logFile){
            case 'info':
                $this->view->fileContent = file_get_contents(APPLICATION_PATH.'/data/logs/info.txt');
                $data['content'] = $this->view->render('settings/logging.phtml');
                break;
            case 'error':
                $this->view->fileContent = file_get_contents(APPLICATION_PATH.'/data/logs/error.txt');  
                $data['content'] = $this->view->render('settings/logging.phtml');
                break;
            case 'exception':
                $this->view->fileContent = file_get_contents(APPLICATION_PATH.'/data/logs/exception.txt');  
                $data['content'] = $this->view->render('settings/logging.phtml');
                break;
            default:
                $data['content'] = $translate->_('SHOW_LOGGING');  
                break;
        }
        
        $data['title']  = $translate->_('SETTINGS_LOGGING');
        if(file_exists(APPLICATION_PATH.'/data/logs/info.txt')){
            $data['button1']['label']   = $this->view->translate('SHOW_LOGGING_INFORMATION');
            $data['button1']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 0, 0, {log : \'info\'})';
        }
        if(file_exists(APPLICATION_PATH.'/data/logs/error.txt')){ 
            $data['button2']['label']   = $this->view->translate('SHOW_LOGGING_ERROR');
            $data['button2']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 0, 0, {log : \'error\'})';
        }
        if(file_exists(APPLICATION_PATH.'/data/logs/exception.txt')){ 
            $data['button3']['label']   = $this->view->translate('SHOW_LOGGING_EXCEPTION');
            $data['button3']['onclick'] = 'si_menu_settings_edit($.tree_reference(\'si_menu_content_settings_tree\').selected, $.tree_reference(\'si_menu_content_settings_tree\'), 0, 0, {log : \'exception\'})';
        }
        echo Zend_Json::encode($data);
        die();     
    }
    
    public function alertAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        $data['message'] = $translate->_('GENERAL_SE_ERROR');
        $data['title']   = $translate->_('GENERAL_ERROR'); 
        Sicore_Models_Logger::error($translate->_('GENERAL_SE_ERROR'));
        echo Zend_Json::encode($data);
        die(); 
    }
    
}