<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Database.php 1557 2010-10-21 16:50:11Z ms $
 */
class Sicore_Models_Database implements Iterator
{
    
    /**
     * BESCHREIBUNG
     *
     * @var TYPE
     */
    protected $_properties = array();
    
    /**
     * BESCHREIBUNG
     *
     * @var TYPE
     */
    protected $_subforms = array();
        
    /**
     * interner Speicher für alle Felder aus der Datenbankabfrage
     *
     * @var Array
     */
    protected $_rawData    = array();
    
    
    public function init($rawData, $suffix = '')
    {
        
        if(!is_array($rawData) && !is_object($rawData)) return;
        
        $this->_rawData  = $rawData;
        foreach($rawData as $key => $value){
            if(!empty($suffix)){
                if(substr($key, 0, strlen($suffix)) !=  $suffix) continue;
                $key = substr($key, strlen($suffix));
            }
            if(($property = $this->getPropertyByDb($key))){
                $property->set($value);
            }            
        }
    }
    
    public function getRawData()
    {
        return $this->_rawData;
    }
    
    protected function _loadConfig(Zend_Config $config)
    {
        if(isset($config->subforms)){
            foreach($config->subforms as $name => $subform){
                
                $admin     = (isset($subform->admin) && $subform->admin);
                $fields    = str_replace(' ', '', $subform->fields);
                if($subform->addfields instanceof Zend_Config){
                    $addfields = $subform->addfields->toArray();
                }else{
                    $addfields = array();
                }
                
                if(isset($this->_subforms[$name])){
                    $this->_subforms[$name]['fields'] = array_merge($this->_subforms[$name], explode(',', $fields), $addfields); 
                    $this->_subforms[$name]['admin']  = $admin;
                }else{
                    $this->_subforms[$name]['fields'] = array_merge(explode(',', $fields), $addfields);
                    $this->_subforms[$name]['admin']  = $admin; 
                }
                
            }
        }
        if(isset($config->properties)){   
            foreach($config->properties as $property => $options){
                $alias = $options->options && $options->options->alias ? $options->options->alias : null;

                $object = new $options->type($property, $this, $alias);
                if(isset($options->form)){
                    $object->setFormConfig($options->form);
                }
                 if(isset($options->options)){
                    $object->setConfig($options->options);
                }
                $key  = !empty($options->options->alias) ? $options->options->alias : $property; 
                $this->_properties[$key] = $object;
            } 
        }   
    }
       
    protected function getPropertyByDb($dbFieldName)
    {
        foreach($this->_properties as $property){
            if($property->field() == $dbFieldName) return $property;
        }
        
        return null;
    }
    
    public function onUpdate()
    {
    
    }
    
    public function onCreate()
    {
    
    }
    
    public function onDelete()
    {
    
    }
    
    public function getSubforms()
    {
        return $this->_subforms;
    }
    
   
    public function __get($key)
    {
        if(isset($this->_properties[$key])) return $this->_properties[$key];
    }
    
    public function rewind()
    {
        reset($this->_properties);
    }
    
    public function valid()
    {
        return current($this->_properties);
    }
    
    public function next()
    {
        next($this->_properties);    
    }
    
    public function key()
    {
        return key($this->_properties);
    }
    
    public function current()
    {
        return current($this->_properties); 
    }
      
}

