<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: FSO.php 1207 2010-02-27 19:03:54Z ms $
 */
class Sicore_Models_FSO extends Sicore_Models_Database
{
    
    /**
     * Konfiguration
     *
     * @var String
     */
    protected $_config      = null;
    
    /**
     * Übergeordnete Ordner / Rootline
     *
     * @var Array
     */
    protected $_parents      = null;
    
    /**
     * Speicherort der Dateien
     *
     * @var String
     */
    static protected $_path   = null; 
    
    /**
     * Konstruktor
     *
     * @param Array Datenbankfelder des Datensatzes
     */
    public function __construct($rawData = null, $suffix = null){
        $this->_parents = array();
        
        $this->_config = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FILESYSTEM);
        self::$_path   = $this->_config->options->path;   
    } 
    
    /**
     * Setzen des Speicherortes der Dateien
     * 
     * @param String $path Absoluter Dateipfad
     * @return String
     */   
    static function setPath($path)
    {
        self::$_path = $path;    
    }
    
    
    /**
     * Rückgabe des Speicherortes der Dateien
     * 
     * @return String
     */
    static function getPath()
    {
        return self::$_path;
    }
        
    /**
     * Rückgabe des übergeordneten Ordners aus einer Ebene
     *
     * @param  Integer Nummer der Ebene, aus der übergeordnete Ordner zurückgegeben werden soll (0 = Root-Ebene)  
     * @return Sicore_Models_FSO
     */
    public function getParent($level)
    {
        return isset($this->_parents[$level]) ? $this->_parents[$level] : null;
    }
    
    /**
     * Rückgabe aller übergeordneter Ordner 
     *
     * @return Array
     */
    public function getAllParents()
    {
        return $this->_parents;
    }
    
    /**
     * Hinzufügen eines übergeordneten Ordners
     *
     * @param Sicore_Models_FSO_Folder $node 
     * @return Sicore_Models_FSO Fluent Interface
     */
    public function addParent(Sicore_Models_FSO_Folder $folder)
    {      
        $this->_parents[$folder->level->value()] = $folder;
        
        return $this;
    }
       
}

