<?php
 /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage FSO
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Image.php 1221 2010-03-03 20:46:55Z ms $
 */
class Sicore_Models_FSO_File_Image extends Sicore_Models_FSO_File
{
    public function getRealPath($mode = null)
    {
        $dir = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value());
        if($mode){
            $path = $dir.$mode.'_'.$this->name->value();
            if(file_exists($path)){
                return $path;
            }  
        }
        return $dir.$this->name->value();    
    } 
    
    public function getIcon($mode = 'image')
    {
        $configBase = Sicore_Models_Registry::getConfig();
        $prefix     = isset($configBase->gfx->backend->icon) ? $configBase->gfx->backend->icon.'_' : 'icon_';   
        switch($mode){
            case 'width':
            case 'height':
                $dir    = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value());
                $image  = $dir.$prefix.$this->name->value();
                $size = getimagesize($image);  
            case 'width':
                return $size[0];
            case 'height':   
                return $size[1];
            default: 
                $dir = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, self::$_path, $this->folders_uid->value(), $this->uid->value());
                return $dir.$prefix.$this->name->value();     
        }  
    }
    
    public function get($size, $mode = 'image')
    {
        $configBase = Sicore_Models_Registry::getConfig();
        $this->_checkImage($size);  
        switch($mode){
            case 'width':
            case 'height':
                $dir    = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value());
                $image  = $dir.$size.'_'.$this->name->value();
                $size = getimagesize($image);  
            case 'width':
                return $size[0];
            case 'height':   
                return $size[1];
            default:
                $dir = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, self::$_path, $this->folders_uid->value(), $this->uid->value());
                return $size ? $dir.$size.'_'.$this->name->value() : $dir.$this->name->value();     
        }     
    }
    
    public function getThumbnail($mode = 'image')
    {
        $configBase = Sicore_Models_Registry::getConfig(); 
        $prefix     = isset($configBase->gfx->backend->thumb) ? $configBase->gfx->backend->thumb.'_' : 'thumb_';
        switch($mode){
            case 'width':
            case 'height':
                $dir    = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value());
                $image  = $dir.$prefix.$this->name->value();
                $size = getimagesize($image);  
            case 'width':
                return $size[0];
            case 'height':   
                return $size[1];
            default: 
                $dir = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, self::$_path, $this->folders_uid->value(), $this->uid->value());
                return $dir.$prefix.$this->name->value();     
        } 
    } 
    
    public function getPreview($mode = 'image')
    {
        $configBase = Sicore_Models_Registry::getConfig(); 
        $prefix     = isset($configBase->gfx->backend->thumb) ? $configBase->gfx->backend->preview.'_' : 'preview_'; 
        switch($mode){
            case 'height':
                $dir    = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value());
                $image  = $dir.$prefix.$this->name->value();
                $size = getimagesize($image); 
                return $size[1]; 
            case 'width':
                $dir    = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value());
                $image  = $dir.$prefix.$this->name->value();
                $size = getimagesize($image); 
                return $size[0]; 
            default: 
                $dir = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, self::$_path, $this->folders_uid->value(), $this->uid->value());
                return $dir.$prefix.$this->name->value();     
        }    
    }
        
    public function onUpdate()
    {
    
        $dir   = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value()); 
        
        $image = $this->name->value();
        $mImage = new Sicore_Models_Image();
        $mImage->generate($dir.$image);
    }
    
    protected function _checkImage($size)
    {
        if(empty($size)) return false;
        
        $configBase = Sicore_Models_Registry::getConfig();
        $absDir = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->folders_uid->value(), $this->uid->value());
        if(file_exists($absDir.$size.$this->name->value())) return true;
        
        $mImage = new Sicore_Models_Image();
        $mImage->generateFromConfig($absDir.$this->name->value(), $size);
        
        return true;
    }   
}

