<?php
 /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage FSO
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Folder.php 1207 2010-02-27 19:03:54Z ms $
 */
class Sicore_Models_FSO_Folder extends Sicore_Models_FSO
{
    
    /**
     * Übergeordnete Ordner / Rootline
     *
     * @var Array
     */
    protected $_parents    = null;
    
       
    /**
     * Konstruktor
     *
     * @param Array Datenbankfelder des Datensatzes
     */
    public function __construct($rawData = null, $suffix = null){
        
        parent::__construct($rawData, $suffix);
        $this->_loadConfig($this->_config->Sicore_Models_FSO_Folder);
        
        $this->init($rawData);
          
    } 
        
    /**
     * Rückgabe des übergeordneten Ordners aus einer Ebene
     *
     * @param  Integer Nummer der Ebene, aus der übergeordnete Ordner zurückgegeben werden soll (0 = Root-Ebene)  
     * @return Sicore_Models_Node
     */
    public function getParent($level)
    {
        return isset($this->_parents[$level]) ? $this->_parents[$level] : null;
    }
    
    /**
     * Rückgabe aller übergeordneter Ordner 
     *
     * @return Array
     */
    public function getAllParents()
    {
        return $this->_parents;
    }
    
    /**
     * Hinzufügen eines übergeordneten Ordners
     *
     * @param Sicore_Models_FSO_Folder $node 
     * @return Sicore_Models_FSO_Folder Fluent Interface
     */
    public function addParent(Sicore_Models_FSO_Folder $folder)
    {      
        $this->_parents[$folder->level->value()] = $folder;
        
        return $this;
    }
    
    public function onCreate()
    {
        $dir = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->uid->value());
        Sicore_Models_Helper_FSO::mkdir($dir);
    }
    
    public function onDelete()
    {
        $dir = Sicore_Models_Helper_FSO::joinPathAbs(self::$_path, $this->uid->value());
        Sicore_Models_Helper_FSO::deleteDir($dir);
    }
    
    public function getThumbnail($mode = 'folder')
    {
        $configBase    = Sicore_Models_Registry::getConfig(); 
        $configBackend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_BACKEND);
        switch($mode){
            case 'width':
            case 'height':
                return 75;
            default: 
                $dir = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, 'templates/backend/', $configBackend->layout, '_img/files');
                return $dir.'folder_thumb.gif';     
        } 
    }
       
}

