<?php
  /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Gui
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Template.php 1544 2010-09-22 16:15:54Z ms $
 */
class Sicore_Models_Gui_Form_Design_Template extends Sicore_Models_Gui_Form
{
    protected $_user       = null;
    protected $_template   = null;
    protected $_isStandard = null; 
    
    const IDPREFIX = 'siform_'; 
    
    public function __construct(Sicore_Models_Template_Local $template, $user, $isStandard = false)
    {        
        $this->_template   = $template;
        $this->_user       = $user;  
        $this->_isStandard = $isStandard; 
        
        parent::__construct();
    }
    
    public function init()
    {
        
        $this->setAttrib('id',     'si_edit_main'); 
        $locale    = Sicore_Models_Registry::getLocale();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $this->setDecorators(array(
            'FormElements',
            array('TabContainer', array(
                'id'          => 'tabContainer'
            )),
            'Form',
        ));
        
        //INTERN
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('SICORE_MODELS_NODE_PAGE_TEMPLATE'))))
        ));
        
                
        $element = new Zend_Form_Element_Checkbox('standard');
        $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('DESIGN_TEMPLATE_STANDARD'));
        $element->setChecked($this->_isStandard);   
        $element->setAttrib('id', self::IDPREFIX.'standard');  
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'standard');
        
        $element = new Zend_Form_Element_Text('folder');
        $element->setLabel($translate->_('DESIGN_TEMPLATE_FOLDER'));
        $element->setValue($this->_template->getFolder()); 
        $element->setAttrib('id', self::IDPREFIX.'name');
        $element->setAttrib('readonly', 'readonly'); 
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'folder');
        
        $element = new Zend_Form_Element_Text('name');
        $element->setAttrib('disabled', 'disabled');
        $element->setLabel($translate->_('DESIGN_TEMPLATE_NAME'));
        $element->setValue($this->_template->getName()); 
        $element->setAttrib('id', self::IDPREFIX.'name');  
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $subForm->addElement($element, 'name');
                
        $element = new Zend_Form_Element_Text('author');
        $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('DESIGN_TEMPLATE_COPYRIGHT'));
        $element->setValue($this->_template->getAuthor()); 
        $element->setAttrib('id', self::IDPREFIX.'author');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $subForm->addElement($element, 'author');
        
        if($this->_template->getDescription()){
            $element = new Zend_Form_Element_Textarea('description');
            $element->setAttrib('disabled', 'disabled'); 
            $element->setLabel($translate->_('DESIGN_TEMPLATE_DESCRIPTION'));
            $element->setValue($this->_template->getDescription()); 
            $element->setAttrib('id', self::IDPREFIX.'description');
            $element->removeDecorator('HtmlTag');
            $element->addDecorator('Label'); 
            $element->addDecorator('Description');  
            $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
            $subForm->addElement($element, 'description');
        }
        $subForm->addElement('hash', 'si_hash_', array('salt' => 'unique')); 
        $this->addSubForm($subForm, 'general');
        
        
    }   
}