<?php
 /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Gui
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Generaldata.php 1544 2010-09-22 16:15:54Z ms $
 */
class Sicore_Models_Gui_Form_Settings_Generaldata extends Sicore_Models_Gui_Form
{
    protected $_user     = null;
    protected $_config   = null; 
    protected $_editmode = true;
    
    const IDPREFIX = 'siform_'; 
    
    public function __construct(Zend_Config $config, $user, $editmode = true)
    {        
        $this->_user     = $config;
        $this->_config   = $config; 
        $this->_editmode = $editmode; 
        
        parent::__construct();
    }
    
    public function init()
    {
        
        $this->setAttrib('id',     'si_edit_main'); 
        $locale    = Sicore_Models_Registry::getLocale();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $this->setDecorators(array(
            'FormElements',
            array('TabContainer', array(
                'id'          => 'tabContainer'
            )),
            'Form',
        ));
        
        //INTERN
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('SETTINGS_GENERAL_DATA'))))
        ));
        
        $element = new Zend_Form_Element_Checkbox('maintenance');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('GENERAL_MAINTENANCE'));
        $element->setValue($this->_config->app->maintenance->status); 
        $element->setAttrib('id', self::IDPREFIX.'uid'); 
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'maintenance');
        
        $element = new Zend_Form_Element_Text('release');
        $element->setAttrib('disabled', 'disabled');
        $element->setLabel($translate->_('GENERAL_APP_RELEASE'));
        $element->setValue($this->_config->app->release); 
        $element->setAttrib('id', self::IDPREFIX.'release');  
        $element->setAttrib('class', 'si_small');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $subForm->addElement($element, 'release');
                
        $element = new Zend_Form_Element_Text('name');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled');  
        $element->setLabel($translate->_('GENERAL_CUSTOMER_NAME'));
        $element->setValue($this->_config->customer->name); 
        $element->setAttrib('id', self::IDPREFIX.'name');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $element->addFilter(new Zend_Filter_Alnum(true));   
        $subForm->addElement($element, 'name');
        
        $element = new Zend_Form_Element_Text('email');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('GENERAL_CUSTOMER_MAIL'));
        $element->setValue($this->_config->customer->email); 
        $element->setAttrib('id', self::IDPREFIX.'email');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $element->addValidator(new Zend_Validate_EmailAddress());   
        $subForm->addElement($element, 'email');
        
        $subForm->addElement('hash', 'si_hash_', array('salt' => 'unique')); 
        
        $this->addSubForm($subForm, 'general');
        
        
    }   
}