<?php
     /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Gui
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Generallicence.php 1544 2010-09-22 16:15:54Z ms $
 */
class Sicore_Models_Gui_Form_Settings_Generallicence extends Sicore_Models_Gui_Form
{
    protected $_user     = null;
    protected $_config   = null; 
    protected $_editmode = true;
    
    const IDPREFIX = 'siform_'; 
    
    public function __construct(Zend_Config $config, $user, $editmode = true)
    {        
        $this->_user     = $config;
        $this->_config   = $config; 
        $this->_editmode = $editmode; 
        
        parent::__construct();
    }
    
    public function init()
    {
        
        $this->setAttrib('id',     'si_edit_main'); 
        $locale    = Sicore_Models_Registry::getLocale();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $this->setDecorators(array(
            'FormElements',
            array('TabContainer', array(
                'id'          => 'tabContainer'
            )),
            'Form',
        ));
        
        //INTERN
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('SETTINGS_GENERAL_LICENCE'))))
        ));
                 
        $element = new Zend_Form_Element_Text('release');
        $element->setAttrib('disabled', 'disabled');
        $element->setLabel($translate->_('GENERAL_APP_RELEASE'));
        $element->setValue($this->_config->app->release); 
        $element->setAttrib('id', self::IDPREFIX.'release');  
        $element->setAttrib('class', 'si_small');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $subForm->addElement($element, 'release');
        
        $element = new Zend_Form_Element_Text('number');
        $element->setAttrib('disabled', 'disabled');
        $element->setLabel($translate->_('GENERAL_APP_NUMBER'));
        $element->setValue($this->_config->app->number); 
        $element->setAttrib('id', self::IDPREFIX.'number');  
        $element->setAttrib('class', 'si_small');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'number');
        
        $element = new Zend_Form_Element_Text('serial');
        $element->setAttrib('disabled', 'disabled');  
        $element->setLabel($translate->_('GENERAL_CUSTOMER_SERIAL'));
        $element->setValue($this->_config->app->serial); 
        $element->setAttrib('id', self::IDPREFIX.'serial');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));   
        $subForm->addElement($element, 'serial');
        
        $element = new Zend_Form_Element_Text('version');
        $element->setAttrib('disabled', 'disabled');  
        $element->setLabel($translate->_('GENERAL_CUSTOMER_VERSION'));
        $element->setValue($translate->_('GENERAL_E'.Sicore_Models_Core::le($this->_config->app->serial))); 
        $element->setAttrib('id', self::IDPREFIX.'version');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'version');
        
        if($this->_editmode){
            $element = new Zend_Form_Element_Textarea('activationcode'); 
            $element->setLabel($translate->_('GENERAL_ACTIVATION_CODE'));
            $element->setAttrib('id', self::IDPREFIX.'activationcode');
            $element->removeDecorator('HtmlTag');
            $element->addDecorator('Label'); 
            $element->addDecorator('Description');  
            $element->setRequired(true); 
            $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
            $element->addFilter(new Zend_Filter_Alnum()); 
            $subForm->addElement($element, 'activationcode');
        }
        
        $subForm->addElement('hash', 'si_hash_', array('salt' => 'unique'));       
        $this->addSubForm($subForm, 'general');
        
        
    }   
}