<?php
  /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Gui
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Updateoptions.php 1544 2010-09-22 16:15:54Z ms $
 */
class Sicore_Models_Gui_Form_Settings_Updateoptions extends Sicore_Models_Gui_Form
{
    protected $_user     = null;
    protected $_config   = null; 
    protected $_editmode = true;
    
    const IDPREFIX = 'siform_'; 
    
    public function __construct(Zend_Config $config, $user, $editmode = true)
    {        
        $this->_user     = $config;
        $this->_config   = $config; 
        $this->_editmode = $editmode; 
        
        parent::__construct();
    }
    
    public function init()
    {
        
        $this->setAttrib('id',     'si_edit_main'); 
        $locale    = Sicore_Models_Registry::getLocale();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $this->setDecorators(array(
            'FormElements',
            array('TabContainer', array(
                'id'          => 'tabContainer'
            )),
            'Form',
        ));
        
        //INTERN
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('SETTINGS_UPDATES_OPTIONS'))))
        ));
         
        $element = new Zend_Form_Element_Checkbox('ftpenable');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('GENERAL_FTP_USE'));
        $element->setChecked($this->_config->ftp->enable);   
        $element->setAttrib('id', self::IDPREFIX.'ftpenable'); 
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'ftpenable');
        
        $element = new Zend_Form_Element_Text('ftpserver');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled');
        $element->setLabel($translate->_('GENERAL_FTP_HOST'));
        $element->setValue($this->_config->ftp->server); 
        $element->setAttrib('id', self::IDPREFIX.'ftpserver');  
        $element->setAttrib('class', 'si_small');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $element->addFilter(new Zend_Filter_StripTags()); 
        $element->addFilter(new Simploo_Filter_StripSlashes());
        $element->addFilter(new Simploo_Filter_StripQuotes()); 
        $subForm->addElement($element, 'ftpserver');
        
        $element = new Zend_Form_Element_Text('ftpuser');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled');
        $element->setLabel($translate->_('GENERAL_FTP_USER'));
        $element->setValue($this->_config->ftp->user); 
        $element->setAttrib('id', self::IDPREFIX.'ftpuser');  
        $element->setAttrib('class', 'si_small');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->addFilter(new Zend_Filter_StripTags()); 
        $element->addFilter(new Simploo_Filter_StripSlashes());
        $element->addFilter(new Simploo_Filter_StripQuotes()); 
        $subForm->addElement($element, 'ftpuser');
        
        if($this->_editmode){
            $element = new Zend_Form_Element_Password('ftppass');
            $element->setLabel($translate->_('GENERAL_FTP_PASS'));
            $element->setAttrib('id', self::IDPREFIX.'ftppass');
            $element->removeDecorator('HtmlTag');
            $element->setAttrib('class', 'si_small'); 
            $element->addDecorator('Label'); 
            $element->addDecorator('Description');  
            $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
            $element->addFilter(new Zend_Filter_StripTags()); 
            $element->addFilter(new Simploo_Filter_StripSlashes());
            $element->addFilter(new Simploo_Filter_StripQuotes());   
            $subForm->addElement($element, 'ftppass');
        }else{
            $element = new Zend_Form_Element_Hidden('ftppass');
            $subForm->addElement($element, 'ftppass'); 
        }
     
        $this->addSubForm($subForm, 'general');
        
        
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('SETTINGS_UPDATES_OPTIONS2'))))
        ));
        $element = new Zend_Form_Element_Checkbox('ftppasv');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('GENERAL_FTP_PASV'));
        $element->setChecked($this->_config->ftp->passv);  
        $element->setAttrib('id', self::IDPREFIX.'ftppasv'); 
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'ftppasv');
        
        $element = new Zend_Form_Element_Checkbox('ftpssl');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('GENERAL_FTP_SSL'));
        $element->setChecked($this->_config->ftp->ssl);  
        $element->setAttrib('id', self::IDPREFIX.'ftpssl'); 
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));  
        $subForm->addElement($element, 'ftpssl');
        
        $element = new Zend_Form_Element_Text('ftpremote');
        if(!$this->_editmode) $element->setAttrib('disabled', 'disabled'); 
        $element->setLabel($translate->_('GENERAL_FTP_REMOTE'));
        $element->setValue($this->_config->ftp->remotedir); 
        $element->setAttrib('id', self::IDPREFIX.'ftpremote'); 
        $element->setAttrib('class', 'si_small'); 
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label'); 
        $element->addDecorator('Description');  
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':')); 
        $element->addFilter(new Zend_Filter_StripTags()); 
        $element->addFilter(new Simploo_Filter_StripSlashes());
        $element->addFilter(new Simploo_Filter_StripQuotes());  
        $subForm->addElement($element, 'ftpremote');
        
        $subForm->addElement('hash', 'si_hash_', array('salt' => 'unique'));
        
        $this->addSubForm($subForm, 'extended'); 
    }   
}