<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Helper
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: FSO.php 1430 2010-06-16 13:19:58Z ms $
 */
class Sicore_Models_Helper_FSO{

    static function getMime($filename)
    {
        $extget = explode(".", $filename);
        $extension = $extget[count($extget)-1];
                   
        $mime = array(
                'doc'    => 'application/msword',
                'bin'    => 'application/octet-stream',
                'exe'    => 'application/octet-stream',
                'rar'    => 'application/x-rar-compressed',
                'so'     => 'application/octet-stream',
                'pdf'    => 'application/pdf',
                'swf'    => 'application/x-shockwave-flash',
                'tar'    => 'application/x-tar',
                'zip'    => 'application/zip',
                'mpga'   => 'audio/mpeg',
                'mp2'    => 'audio/mpeg',
                'mp3'    => 'audio/mpeg',
                'aif'    => 'audio/x-aiff',
                'aiff'   => 'audio/x-aiff',
                'aifc'   => 'audio/x-aiff',
                'm3u'    => 'audio/x-mpegurl',
                'ram'    => 'audio/x-pn-realaudio',
                'rm'     => 'audio/x-pn-realaudio',
                'rpm'    => 'audio/x-pn-realaudio-plugin',
                'ra'     => 'audio/x-realaudio',
                'wav'    => 'audio/x-wav',
                'bmp'    => 'image/bmp',
                'gif'    => 'image/gif',
                'jpeg'   => 'image/jpeg',
                'jpg'    => 'image/jpeg',
                'jpe'    => 'image/jpeg',
                'png'    => 'image/png',
                'tiff'   => 'image/tiff',
                'tif'    => 'image/tiff',
                'mpeg'   => 'video/mpeg',
                'mpg'    => 'video/mpeg',
                'mpe'    => 'video/mpeg',
                'qt'     => 'video/quicktime',
                'mov'    => 'video/quicktime',
                'avi'    => 'video/x-msvideo'
            );
        if(isset($mime[$extension])){
            return $mime[$extension];
        }else{
            return 'application/octet-stream';
        }
    }
    
    static function joinPathAbs()
    {
        $path = '';
        
        $args = func_get_args();
        foreach($args as $arg){
            $arg   = str_replace('\\', '/', $arg);
            $arg   = substr($arg, 0, 1) != '/' ? "/$arg" : $arg;
            $arg   = substr($arg, strlen($arg) - 1) == '/' ? substr($arg, 0, strlen($arg) - 1) : $arg;
            $path .= $arg;   
        }
        
        return APPLICATION_PATH.$path.'/';
    } 
    
    static function joinPath()
    {
        $path = '';
        
        $args = func_get_args();
        foreach($args as $arg){
            $arg   = str_replace('\\', '/', $arg);
            $arg   = substr($arg, 0, 1) != '/' ? "/$arg" : $arg;
            $arg   = substr($arg, strlen($arg) - 1) == '/' ? substr($arg, 0, strlen($arg) - 1) : $arg;
            if($arg != '/' && $arg != '/'){
                $path .= $arg;
            }   
        }

        return $path.'/';
    } 
    
    static function getLayoutPathAbs($page = null)
    {
        $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND);
        
        if($page instanceof Sicore_Models_Node_Page && $this->view->page->template->value()){
            $template = $this->view->page->template->value();
        }elseif($config_frontend->layout){
            $template = $config_frontend->layout;   
        }else{
            $template = 'sibase';
        }

        return self::joinPathAbs('templates/frontend', $template);

    }   
    
    static function mkdir($dir)
    {
        $config_fs = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FILESYSTEM); 
        if(is_dir($dir)){
            @chmod($dir, 0775);
            return true;
        }   
        return mkdir($dir, 0775);  
    }
    
    static function deleteFile($file)
    {
        if (!file_exists($file)) return true; 
        if (!is_dir($file)) return unlink($file);
        
        return false;
    }
    
    static function deleteDir($dir)
    {
        if (!file_exists($dir)) return true;
        if (!is_dir($dir)) return unlink($dir);
        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') continue;
            if (!self::deleteDir($dir.'/'.$item)) return false;
        }
        return rmdir($dir);
    }
    
    static function deleteDirContent($dir)
    {
        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') continue;
            if (!self::deleteDir($dir.'/'.$item)) return false;
        }
        return true;
    }
    
    static function readfile($filename) {
      $buffer = '';
      $handle = fopen($filename, 'rb');
      
      if ($handle === false) {
        return false;
      }
      
      while (!feof($handle)) {
        $buffer = fread($handle, 1024);
        print $buffer;
        ob_flush();
        flush();
      }
      return fclose($handle);
    } 
    
    static function cleanFilename($name)
    {
        $config_fe = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND);
        $name = mb_strtolower($name, $config_fe->charset);
        $from = array('ü', 'ä', 'ö', 'ß', ' ');
        $to   = array('ue', 'ae', 'oe', 'ss', '_');
        $name = str_replace($from, $to, $name);
        $name = preg_replace("/[^a-z0-9_\.\-]/" , "" , $name);
         
        return $name;
    }
    
    static function installUpdate($dir, $simulate = false, $toDir = '', $ftpObject = null)
    {
        $iterator = new DirectoryIterator($dir);
        foreach ($iterator as $fileinfo) {
            if(substr($fileinfo->getFilename(), 0, 1) == '.' || $fileinfo->getFilename() == '..') continue;
            if(is_dir($fileinfo->getPathname())){
                if(!is_dir(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename())){
                   $perms = self::getFilePerm(APPLICATION_PATH.'/'.$toDir); 
                    if($simulate){   
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, 755, $ftpObject);
                        if(!is_writeable(APPLICATION_PATH.'/'.$toDir)){
                            Sicore_Models_Logger::error("UPDATE: DIR NOT WRITABLE: ".APPLICATION_PATH.'/'.$toDir); 
                            self::setFilePerm($perms, $perms, $ftpObject);
                            return false; 
                        }
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, $perms, $ftpObject); 
                    }else{
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, 755, $ftpObject);  
                        mkdir(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename(), 0755);
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, $perms, $ftpObject); 
                        if(!self::installUpdate($fileinfo->getPathname(), $simulate, $toDir.$fileinfo->getFilename().'/', $ftpObject)) return false;
                    }
                }else{
                    if(!self::installUpdate($fileinfo->getPathname(), $simulate, $toDir.$fileinfo->getFilename().'/', $ftpObject)) return false;  
                }       
            }else{
                if(file_exists(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename())){
                    $perms    = self::getFilePerm(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename());
                    $permsDir = self::getFilePerm(APPLICATION_PATH.'/'.$toDir);
                    if($simulate){
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, 755, $ftpObject);  
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename(), 755, $ftpObject);
                        if(!is_writeable(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename()) || !is_writeable(APPLICATION_PATH.'/'.$toDir)){ 
                            Sicore_Models_Logger::error("UPDATE: FILE NOT WRITABLE: ".APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename()); 
                            self::setFilePerm(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename(), $perms, $ftpObject); 
                            return false;
                        }
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, $permsDir, $ftpObject);  
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename(), $perms, $ftpObject); 
                    }else{
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, 755, $ftpObject);
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename(), 755, $ftpObject);
                        Sicore_Models_Logger::info(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename() . " = " . is_writeable(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename())); 
                        unlink(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename());
                        rename($fileinfo->getPathname(), APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename());
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename(), $perms, $ftpObject); 
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, $permsDir, $ftpObject); 
                    }    
                }else{
                    $perms = self::getFilePerm(APPLICATION_PATH.'/'.$toDir);
                    if($simulate){
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, 755, $ftpObject);
                        if(!is_writeable(APPLICATION_PATH.'/'.$toDir)){    
                            Sicore_Models_Logger::error("UPDATE: DIR NOT WRITABLE: ".APPLICATION_PATH.'/'.$toDir); 
                            self::setFilePerm(APPLICATION_PATH.'/'.$toDir, $perms, $ftpObject);
                            return false; 
                        }
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, $perms, $ftpObject); 
                    }else{
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, 755, $ftpObject);  
                        rename($fileinfo->getPathname(), APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename());
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir.$fileinfo->getFilename(), 755, $ftpObject); 
                        self::setFilePerm(APPLICATION_PATH.'/'.$toDir, $perms, $ftpObject); 
                    }
                }
            }        
        }
        return true;
    }
    
    static function getFilePerm($file)
    {
        return substr(decoct( fileperms('.') ), 2);
    }
    
    static function setFilePerm($file, $perms, $ftpObject = null)
    { 
        if(@chmod($file, octdec($perms))){
            return true;
        }
        if($ftpObject){
            $file  = str_replace(APPLICATION_PATH, '', $file);
            if($ftpObject->chmod($file, $perms)){
                return true;
            }
        }
        
        return false;
    }
    
    static function filesize($file, $precision = 2){
        $bytes  = @filesize($file);
        
        $units = array('B', 'KB', 'MB', 'GB', 'TB');
      
        $bytes = max($bytes, 0);
        $pow   = floor(($bytes ? log($bytes) : 0) / log(1024));
        $pow   = min($pow, count($units) - 1);
      
        $bytes /= pow(1024, $pow);
        
        return round($bytes, $precision) . ' ' . $units[$pow];
    }
    
    
    
}