<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Image
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Gdlib.php 1551 2010-10-07 16:37:27Z ms $
 */
if(!defined('IMG_FILTER_GRAYSCALE')){
    define('IMG_FILTER_GRAYSCALE', 2);
}
class Sicore_Models_Image_Gdlib implements Sicore_Models_Image_Interface
{
    static protected $_info  = array();
    static protected $_types = array('png'  => IMAGETYPE_PNG, 
                                     'gif'  => IMAGETYPE_GIF,
                                     'jpeg' => IMAGETYPE_JPEG
                                     );
    static protected $_gifWrite = false;
    
    public function generate($imagePath)
    {
        $baseConfig = Sicore_Models_Registry::getConfig();  
        
        $splitSices = array();
        foreach($baseConfig->gfx->sizes as $key => $size){
            $this->generateFromConfig($imagePath, $key);
        }    
    }
    
    public function generateFromConfig($imagePath, $key)
    {
        $baseConfig = Sicore_Models_Registry::getConfig();
        $size =  $baseConfig->gfx->sizes->$key;
        if(!$size) return false;
        
        $imageName = basename($imagePath);
        $imageDir  = dirname($imagePath);
        
        $split      = explode(',', $size); 
        if(isset($split[1])){
            $filter   = null;
            $cropmode = Sicore_Models_Image_Interface::CONVERT_CROP_CENTER; 
            if(isset($split[1])){
                switch(strtoupper($split[1])){
                    case 'GREYSCALE':
                        $filter = IMG_FILTER_GRAYSCALE;
                        break;
                    case 'TOP':
                        $cropmode = Sicore_Models_Image_Interface::CONVERT_CROP_TOP;        
                        break; 
                    case 'BOTTOM':
                        $cropmode = Sicore_Models_Image_Interface::CONVERT_CROP_BOTTOM;        
                        break; 
                }
            }
        }else{
            $filter   = null; 
            $cropmode = Sicore_Models_Image_Interface::CONVERT_CROP_CENTER;  
        }
        $splitSices = explode('x', $split[0]);
        $width      = intval($splitSices[0]); 
        if(count($splitSices) > 1){
            $height = intval($splitSices[1]);
            if(empty($width)){
                $mode = Sicore_Models_Image_Interface::CONVERT_HEIGHT;  
            }elseif(empty($height)){
                $mode = Sicore_Models_Image_Interface::CONVERT_WIDTH;  
            }else{
                $mode = $cropmode;
            }   
        }else{
            $mode   = Sicore_Models_Image_Interface::CONVERT_MAX;
            $height = 0;   
        } 
        $this->convert($imagePath, $imageDir.'/'.$key.'_'.$imageName, $width, $height, $mode, $filter);
        return true;    
    }
    
    public function convertFromConfig($imageFrom, $imageTo, $configSize)
    {
        $split      = explode(',', $configSize); 
        
        $filter   = null;
        $cropmode = Sicore_Models_Image_Interface::CONVERT_CROP_CENTER; 
        if(isset($split[1])){
            switch(strtoupper($split[1])){
                case 'GREYSCALE':
                    $filter = IMG_FILTER_GRAYSCALE;
                    break;
                case 'TOP':
                    $cropmode = Sicore_Models_Image_Interface::CONVERT_CROP_TOP;        
                    break; 
                case 'BOTTOM':
                    $cropmode = Sicore_Models_Image_Interface::CONVERT_CROP_BOTTOM;        
                    break; 
            }
        }
        
        $splitSices = explode('x', $split[0]);
        $width      = intval($splitSices[0]); 
        if(count($splitSices) > 1){
            $height = intval($splitSices[1]);
            if(empty($width)){
                $mode = Sicore_Models_Image_Interface::CONVERT_HEIGHT;  
            }elseif(empty($height)){
                $mode = Sicore_Models_Image_Interface::CONVERT_WIDTH;  
            }else{
                $mode = $cropmode;
            }   
        }else{
            $mode   = Sicore_Models_Image_Interface::CONVERT_MAX;
            $height = intval($splitSices[0]);   
        }
                 
        $this->convert($imageFrom, $imageTo, $width, $height, $mode, $filter);
        return true;  
    }
        
    public function convert($imageFrom, $imageTo, $width_dest, $height_dest = 0, $mode = Sicore_Models_Image_Interface::CONVERT_MAX, $filter = null, $options = array(), $doNotEnlarge = true)
    {
        
        $width_dest  = intval($width_dest);
        $height_dest = intval($height_dest);  
        if(!$width_dest && !$height_dest){
            return false;
        }
          
        try{
            list($width_source, $height_source, $image_type) = getimagesize($imageFrom);
            switch($image_type){
                case IMAGETYPE_PNG:
                    $type = 'png';         
                    break;
                case IMAGETYPE_GIF:
                    $type = 'gif';              
                    break;
                case IMAGETYPE_JPEG:
                    $type = 'jpeg';       
                    break;
                default:
                    throw new Sicore_Models_Image_Exception_Notsupported();
                    return false;
            }
            $createFunc = "imagecreatefrom$type";
            
            $image_source = $createFunc($imageFrom);
             
            switch($mode){
                case Sicore_Models_Image_Interface::CONVERT_CROP_CENTER: 
                case Sicore_Models_Image_Interface::CONVERT_CROP_TOP: 
                case Sicore_Models_Image_Interface::CONVERT_CROP_BOTTOM: 
                case Sicore_Models_Image_Interface::CONVERT_CROP_LEFT: 
                case Sicore_Models_Image_Interface::CONVERT_CROP_RIGHT: 
                   if(!$width_dest || !$height_dest){
                        return false;
                    }
                case Sicore_Models_Image_Interface::CONVERT_CROP_CENTER: 
                case Sicore_Models_Image_Interface::CONVERT_CROP_BOTTOM:
                case Sicore_Models_Image_Interface::CONVERT_CROP_TOP:     
                    
                    if($width_dest > $height_dest){
                        if($height_source/$width_source >=  $height_dest/$width_dest){
                            $width_resized  = $width_dest; 
                            $height_resized = round($width_dest/$width_source * $height_source);  
                        }else{
                            $height_resized  = $height_dest; 
                            $width_resized   = round($height_dest/$height_source * $width_source);  
                        }
                    }else{
                        if($width_source/$height_source >=  $width_dest/$height_dest){
                            $width_resized  = round($height_dest/$height_source * $width_source);
                            $height_resized = $height_dest;    
                        }else{
                            $height_resized = round($height_dest/$width_source * $height_source);
                            $width_resized  = $height_dest;
                        }
                    }
                    $image_resized = imagecreatetruecolor($width_resized, $height_resized);
                    $bgcolor = imageColorAllocate($image_resized, 255, 255, 255);
                    imagefilledrectangle($image_resized, 0, 0, $width_resized, $height_resized, $bgcolor); 
                    if($image_type == IMAGETYPE_GIF){
                       if(($transparency = imageColorTransparent($image_source)) >= 0){
                           list($r, $g, $b) = array_Values(imageColorsForIndex($image_source, $transparency));
                           $transparency = ImageColorAllocate($image_resized, $r, $g, $b);
                           imagefill($image_resized, 0, 0, $transparency);
                           imageColorTransparent($image_resized, $transparency);
                       }
                   }else{
                       imagealphablending($image_resized, false);
                       imagesavealpha($image_resized, true); 
                   } 
                    imagecopyresampled($image_resized, $image_source, 0, 0, 0, 0, $width_resized, $height_resized, $width_source, $height_source);
                   
                    $width_dest = $width_resized <  $width_dest ? $width_resized : $width_dest;
                    $height_dest = $height_resized <  $height_dest ? $height_resized : $height_dest; 
                    
                    if($mode == Sicore_Models_Image_Interface::CONVERT_CROP_TOP){
                        $src_y = 0;
                    }elseif($mode == Sicore_Models_Image_Interface::CONVERT_CROP_BOTTOM){
                        $src_y = $height_resized - $height_dest;
                    }else{
                        $src_y = round(($height_resized - $height_dest)/2);
                    }
                    $src_x = round(($width_resized - $width_dest)/2);
                    
                    
                    $image_new = imagecreatetruecolor($width_dest, $height_dest);   
                    if($image_type == IMAGETYPE_GIF){
                       if(($transparency = imageColorTransparent($image_resized)) >= 0){ 
                           list($r, $g, $b) = array_Values(imageColorsForIndex($image_resized, $transparency));
                           $transparency = ImageColorAllocate($image_new, $r, $g, $b);
                           imagefill($image_new, 0, 0, $transparency);
                           imageColorTransparent($image_new, $transparency);
                       }
                   }else{
                       imagealphablending($image_new, false);
                       imagesavealpha($image_new, true); 
                   } 
                    imagefilledrectangle($image_new, 0, 0, $width_dest, $height_dest, $bgcolor);
                    imagecopyresampled($image_new, $image_resized, 0, 0, $src_x, $src_y, $width_dest, $height_dest, $width_dest, $height_dest);
                   imagedestroy($image_source); 
                   imagedestroy($image_resized); 
                    break; 
                case Sicore_Models_Image_Interface::CONVERT_EXPLICIT:
                   $width_new  = $width_dest ? $width_dest : $width_source;
                   $height_new = $height_dest ? $height_dest : $height_source;
                   $image_new = imagecreatetruecolor($width_new, $height_new); 
                   if($image_type == IMAGETYPE_GIF){
                       if(($transparency = imageColorTransparent($image_source)) >= 0){ 
                           list($r, $g, $b) = array_Values(imageColorsForIndex($image_source, $transparency));
                           $transparency = ImageColorAllocate($image_new, $r, $g, $b);
                           imagefill($image_new, 0, 0, $transparency);
                           imageColorTransparent($image_new, $transparency);
                       }
                   }else{
                       imagealphablending($image_new, false);
                       imagesavealpha($image_new, true); 
                   }  
                   imagecopyresampled($image_new, $image_source, 0, 0, 0, 0, $width_new, $height_new, $width_source, $height_source);
                   imagedestroy($image_source); 
                   break; 
                
                case Sicore_Models_Image_Interface::CONVERT_MAX:
                default:      
                    if($width_source >= $height_source){
                        if($width_source > $width_dest){
                            $width_new  = $width_dest; 
                            $height_new = round($width_dest/$width_source * $height_source);     
                        }else{
                            $height_new = $height_source;
                            $width_new  = $width_source;
                        } 
                    }else{
                        if($height_source > $width_dest){
                            $height_new = $width_dest;
                            $width_new  = round($width_dest/$height_source * $width_source);
                        }else{
                            $height_new = $height_source;
                            $width_new  = $width_source;
                        }   
                    }
                    $image_new = imagecreatetruecolor($width_new, $height_new);
                    if($image_type == IMAGETYPE_GIF){
                        $transparency = imageColorTransparent($image_source);
                        if(($transparency = imageColorTransparent($image_source)) >= 0){ 
                            list($r, $g, $b) = array_Values(imageColorsForIndex($image_source, $transparency));
                            $transparency = ImageColorAllocate($image_new, $r, $g, $b);
                            imagefill($image_new, 0, 0, $transparency);
                            imageColorTransparent($image_new, $transparency);
                        }
                    }else{
                        imagealphablending($image_new, false);
                        imagesavealpha($image_new, true); 
                    }
                     
                    imagecopyresampled($image_new, $image_source, 0, 0, 0, 0, $width_new, $height_new, $width_source, $height_source);
                    imagedestroy($image_source); 
                    break;  
                case Sicore_Models_Image_Interface::CONVERT_WIDTH:
                    $width_new  = $width_dest;
                    $height_new = round($width_dest/$width_source * $height_source);
                    $image_new = imagecreatetruecolor($width_new, $height_new);
                    if($image_type == IMAGETYPE_GIF){
                       $transparency = imageColorTransparent($image_source);
                       if(($transparency = imageColorTransparent($image_source)) >= 0){ 
                           list($r, $g, $b) = array_Values(imageColorsForIndex($image_source, $transparency));
                           $transparency = ImageColorAllocate($image_new, $r, $g, $b);
                           imagefill($image_new, 0, 0, $transparency);
                           imageColorTransparent($image_new, $transparency);
                       }
                   }else{
                       imagealphablending($image_new, false);
                       imagesavealpha($image_new, true); 
                   }  
                    imagecopyresampled($image_new, $image_source, 0, 0, 0, 0, $width_new, $height_new, $width_source, $height_source);
                    imagedestroy($image_source); 
                    break;
                case Sicore_Models_Image_Interface::CONVERT_HEIGHT:
                    $height_new  = $height_dest;
                    $width_new = round($height_dest/$height_source * $width_source);
                    $image_new = imagecreatetruecolor($width_new, $height_new);
                    if($image_type == IMAGETYPE_GIF){
                       $transparency = imageColorTransparent($image_source);
                       if(($transparency = imageColorTransparent($image_source)) >= 0){ 
                           list($r, $g, $b) = array_Values(imageColorsForIndex($image_source, $transparency));
                           $transparency = ImageColorAllocate($image_new, $r, $g, $b);
                           imagefill($image_new, 0, 0, $transparency);
                           imageColorTransparent($image_new, $transparency);
                       }
                   }else{
                       imagealphablending($image_new, false);
                       imagesavealpha($image_new, true); 
                   } 
                    imagecopyresampled($image_new, $image_source, 0, 0, 0, 0, $width_new, $height_new, $width_source, $height_source);
                    imagedestroy($image_source); 
                    break;
            }   
            
            if($filter){
                if(function_exists('imagefilter')){
                    imagefilter($image_new, $filter); 
                }else{
                    $this->_imagefilter($image_new, $filter); 
                }
                
            }
                        
            $createFunc = "image$type"; 
            if($type == 'jpeg'){
                $createFunc($image_new, $imageTo, 100);
            }else{
                $createFunc($image_new, $imageTo);
            }
            imagedestroy($image_new); 
            @chmod($imageTo, 0775);   
            return true;
        }catch(Sicore_Models_Image_Exception_Notsupported $e){
            Sicore_Models_Logger::exception("Sicore_Models_Image_Exception_Notsupported: ".$e);
            return false;
        }catch(Exception $e){
            Sicore_Models_Logger::exception("IMAGE_EXCEPTION: ".$e); 
            return false;
        }
    }
    
    protected function _imagefilter($source, $var, $arg1 = null, $arg2 = null, $arg3 = null){

        $max_y = imagesy($source);
        $max_x = imagesx($source);
        
        switch ($var){ 
            case IMG_FILTER_GRAYSCALE:
                $y = 0;
                while($y<$max_y) {
                    $x = 0;
                    while($x<$max_x){
                        $rgb = imagecolorat($source,$x,$y);
                        $a = $rgb >> 24;
                        $r = ((($rgb >> 16) & 0xFF)*0.299)+((($rgb >> 8) & 0xFF)*0.587)+(($rgb & 0xFF)*0.114);
                        $new_pxl = imagecolorallocatealpha($source, $r, $r, $r, $a);
                        if ($new_pxl == false){
                            $new_pxl = imagecolorclosestalpha($source, $r, $r, $r, $a);
                        }
                        imagesetpixel($source,$x,$y,$new_pxl);
                        ++$x;
                    }
                    ++$y;
                }
                return true;
                break;
        }
           
        return false;
        
    }
    
    protected function setTransparency($new_image, $image_source)
    {
       
            $transparencyIndex = imagecolortransparent($image_source);
            $transparencyColor = array('red' => 255, 'green' => 255, 'blue' => 255);
            
            if ($transparencyIndex >= 0) {
                $transparencyColor    = imagecolorsforindex($image_source, $transparencyIndex);   
            }
           
            $transparencyIndex    = imagecolorallocate($new_image, $transparencyColor['red'], $transparencyColor['green'], $transparencyColor['blue']);
            imagefill($new_image, 0, 0, $transparencyIndex);
            imagecolortransparent($new_image, $transparencyIndex);
       
    } 
    
    public function isAvailable()
    {
         
        if(!function_exists('gd_info')){
            return false;
        }
        self::$_info = gd_info();
        
        self::$_types = array();
        if(self::$_info['PNG Support']) self::$_types['png']      = IMAGETYPE_PNG; 
        if(self::$_info['GIF Read Support']) self::$_types['gif'] = IMAGETYPE_GIF;
        if(@self::$_info['JPG Support'] || @self::$_info['JPEG Support']) self::$_types['jpeg']    = IMAGETYPE_JPEG;
        
        if(self::$_info['GIF Create Support']) self::$_gifWrite    = true; 
        
        return true;
    } 
    
    public function getSupportedTypes()
    {
        return self::$_types;
    }  
    
    public function isSupported($image)
    {
        if(!($info = getimagesize($image))){
            return false;
        }
        list($width, $height, $type, $attr) = $info;
        return (bool)in_array($type, self::$_types);
    }
    
    public function getMime($image)
    {
        if(!($info = @getimagesize($image))){
            return false;
        }

        return $info['mime'];
    }    
}

