<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Node.php 1540 2010-09-21 15:51:20Z ms $
 */
class Sicore_Models_Node extends Sicore_Models_Database
{
    
    /**
     * Knoten im Baum
     *
     * @var Sicore_Models_Node_Tree
     */
    protected $_tree    = null;
    
    /**
     * Urlalias
     *
     * @var Sicore_Models_Node_Urlalias
     */
    protected $_urlalias = null;
       
    /**
     * Übergeordnete Elternknoten / Rootline
     *
     * @var Array
     */
    protected $_parents    = null;
    
    
    /**
     * Direkte untergeordnete Knoten
     * wird für die Darstellung der Menüs benötigt und auch nur dort gefüllt
     *
     * @var Array
     */
    protected $_childs     = null; 
        
    /**
     * Knoten ist in der Navigation ausgewählt
     *
     * @var Boolean
     */
    protected $_isSelected = false;
    
    /**
     * Knoten ist in der aktuellen Rootline enthalten
     *
     * @var Boolean
     */
    protected $_isInRootline = false;
   
    /**
     * Im Entwurfsmodus, die höchste Version der direkten Unterknoten
     *
     * @var Integer
     */
    protected $_subnodes_version   = 0;  
   
    /**
     * Konstruktor
     *
     * @param Array Datenbankfelder des Datensatzes
     */
    public function __construct($rawData = null, $suffix = null){
        $this->_parents = array();
        $this->_childs  = array();
        
        $config = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_NODES);
        $this->_loadConfig($config->Sicore_Models_Node);
                
        $this->_tree     = new Sicore_Models_Node_Tree($rawData, 'tree_');
        $this->_urlalias = new Sicore_Models_Node_Urlalias($rawData, 'urlalias_');
        
        $this->_subnodes_version = isset($rawData->subnodes_version) ? intval($rawData->subnodes_version) : 0;
        
        $this->init($rawData);
        foreach($this->_properties as $key => $property){
            $this->_properties[$key]->doCheckPublish($this);
            if($this->uid instanceof Sicore_Models_Node_Property){
                $this->_properties[$key]->setNodeUid($this->uid->value());
            }
        }  
    } 
        
    public function getTree()
    {
        return $this->_tree;
    }
    
    public function setTree(Sicore_Models_Node_Tree $tree)
    {
        $this->_tree = $tree;
        return $this;
    }
    
    public function getUrlalias()
    {
        return $this->_urlalias;
    }
    
    public function getSubnodesVersion()
    {
        return $this->_subnodes_version;
    }
    
    public function setUrlalias(Sicore_Models_Node_Urlalias $urlalias)
    {
        $this->_urlalias = $urlalias;
        return $this;
    }
    
    /**
     * Überprüft, ob Start- und/oder Enddatum eingetragen ist und ob der Knoten zum aktuellen Zeitpunkt angezeigt wird
     *
     * @param Zend_Date $date zu überprüfendes Datum
     * @return Integer -1 = kein Datum eingetragen / 0 = Datum eingetragen, Knoten wird nicht angezeigt / 1 = Datum eingetragen, Knoten wird angezeigt
     */
    public function checkDate(Zend_Date $date){
        
        
        if($this->enddate->value() instanceof Zend_Date && $this->startdate->value() instanceof Zend_Date){
            return $this->startdate->value()->isEarlier($date) && $this->enddate->value()->isLater($date) ? 1 : 0;
        }elseif($this->enddate->value() instanceof Zend_Date){
            return $this->enddate->value()->isLater($date) ? 1 : 0;
        }elseif($this->startdate->value() instanceof Zend_Date){
            return $this->startdate->value()->isEarlier($date)  ? 1 : 0;
        } 
        
        return -1;
    }
    
 
    /**
     * Rückgabe oder Setzen, ob der Knoten aktiv ist
     * Bei Parameterübergabe wird der Wert gesetzt und das Knoten-Objekt zurückgegeben (Fluent Interface)
     *
     * @param Boolean
     * @return Boolean/Sicore_Models_Node
     */
    public function isActive($isActive = null)
    {
        if($isActive != null){
            $this->isActive->set($isActive);
            return $this;
        }
        
        return $this->isActive->value();
    }
      
    
    
    /**
     * Rückgabe oder Setzen, ob der Knoten versteckt ist
     * Bei Parameterübergabe wird der Wert gesetzt und das Knoten-Objekt zurückgegeben (Fluent Interface)
     *
     * @param Boolean
     * @return Boolean/Sicore_Models_Node
     */
    public function isHidden($isHidden = null)
    {
        if($isHidden != null){
            $this->isHidden->set($isHidden);
            return $this;
        }
        
        return $this->isHidden->value(); 
    }
    
    /**
     * Rückgabe oder Setzen, ob der Knoten in der Navigation ausgewählt ist
     * Bei Parameterübergabe wird der Wert gesetzt und das Knoten-Objekt zurückgegeben (Fluent Interface)
     *
     * @param Boolean
     * @return Boolean/Sicore_Models_Node 
     */
    public function isSelected($isSelected = null)
    {
        if($isSelected != null){
            $this->_isSelected = $isSelected;
            return $this;
        }
        
        return $this->_isSelected;
    }
    
    /**
     * Rückgabe oder Setzen, ob der Knoten in der aktuellen Rootline enthalten ist ist
     * Bei Parameterübergabe wird der Wert gesetzt und das Knoten-Objekt zurückgegeben (Fluent Interface)
     *
     * @param Boolean
     * @return Boolean/Sicore_Models_Node 
     */
    public function isInRootline($isInRootline = null)
    {
        if($isInRootline != null){
            $this->_isInRootline = $isInRootline;
            return $this;
        }
        
        return $this->_isInRootline;
    }
    
    /**
     * Rückgabe des Links 
     *
     * @return String
     */
    public function getLink()
    {
        $config = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_BASE);
        return '/'.$config->website->path.'/'.$this->permalink->value();
    }
    
    /**
     * Rückgabe aller direkt untergeordneten Knoten 
     *
     * @return Array
     */
    public function getChilds()  
    {
        return $this->_childs;
    }
    
    /**
     * Abfrage ob untergeordnete Knoten gespeicher sind
     *
     * @return Boolean
     */
    public function haveChilds()
    {
        return count($this->_childs) > 0;
    }
    
    /**
     * Rückgabe des Elternknotens aus einer Ebene
     *
     * @param  Integer Nummer der Ebene, aus der der Elternknoten zurückgegeben werden soll (0 = Root-Ebene)  
     * @return Sicore_Models_Node
     */
    public function getParent($level = null)
    {
        if($level == null){
           $level = $this->getTree()->level->value() - 1; 
        }
        $level = intval($level);
        return isset($this->_parents[$level]) ? $this->_parents[$level] : null;
    }
    
    /**
     * Rückgabe aller Elternknoten
     *
     * @return Array
     */
    public function getAllParents($asc = true)
    {
        if($asc){
            ksort($this->_parents);
        }else{
            krsort($this->_parents);  
        }
        return $this->_parents;
    }
    
    /**
     * Hinzufügen eines Elternknotens
     * Es werden automatisch alle betroffenen Felder (aktiv, gelöscht,..) nach dem Vererbungsprinzip gesetzt
     *
     * @param Sicore_Models_Node $node 
     * @param Boolean $mergeParentData 
     * @return Sicore_models_Node Fluent Interface
     */
    public function addParent(Sicore_Models_Node_Page $node, $mergeParentData = true)
    {
        if($mergeParentData){
            $this->meta->compare($node->meta->value());

            if($this->isActive()){
                $this->isActive($node->isActive());
            }
            
            if(!$this->isHidden() && $this->getTree()->isPage->value()){
                $this->isHidden($node->isHidden());
            }
            if(!$this->access->value()){
                $this->access->set($node->access->value());
            }
            
            if($this->startdate->value() instanceof Zend_Date){
                if($node->startdate->value() instanceof Zend_Date && $this->startdate->value().isEarlier($node->startdate->value())){
                    $this->startdate->set($node->startdate->value());
                }    
            }elseif($node->startdate->value() instanceof Zend_Date){
                $this->startdate->set($node->startdate->value()); 
            }
            if($this->enddate->value() instanceof Zend_Date){
                if($node->enddate->value() instanceof Zend_Date && $this->getEnddate().isLater($node->enddate->value())){
                    $this->enddate->set($node->enddate->value());          
                }    
            }elseif($node->enddate->value() instanceof Zend_Date){
                $this->enddate->set($node->enddate->value());
            }
            if(!$this->caneditpage->value()){
                $this->caneditpage->set($node->caneditpage->value());
            } 
            if(!$this->caneditcontent->value()){
                $this->caneditcontent->set($node->caneditcontent->value());
            } 
            if(!$this->canpublish->value()){
                $this->canpublish->set($node->canpublish->value());
            } 
            if(!$this->candelete->value()){
                $this->candelete->set($node->candelete->value());
            }
            if(!$this->canaddsubpages->value()){
                $this->canaddsubpages->set($node->canaddsubpages->value());
            }
            if(!$this->caneditsubpages->value()){
                $this->caneditsubpages->set($node->caneditsubpages->value());
            }
            if(!$this->caneditsubcontent->value()){    
                $this->caneditsubcontent->set($node->caneditsubcontent->value());
                if(!$this->caneditcontent->value()) $this->caneditcontent->set($node->caneditsubcontent->value()); 
            }
            if(!$this->canpublishsubpages->value()){
                $this->canpublishsubpages->set($node->canpublishsubpages->value());
            }
            if(!$this->canundosubpages->value()){
                $this->canundosubpages->set($node->canundosubpages->value());
            }
            if(!$this->candeletesubpages->value()){
                $this->candeletesubpages->set($node->candeletesubpages->value());
            }
            foreach($this as $property){ 
                $field = $property->getConfig('alias') ? $property->getConfig('alias') : $property->field();
                if($property->getConfig('rootline')){  
                    if(!$this->$field->value()){
                        $canPublish = ($node->publishfield->value() == $node->$field->field());
                        $deleteSubfields = (bool)$node->publishdeletesubfields->get($node->$field->field());
                        $this->$field->set($node->$field->value(), $node->uid->value(), $canPublish, $deleteSubfields);
                    }
                }
            }
        }
        
        $this->_parents[$node->getTree()->level->value()] = $node;
        
        return $this;
    }
    
    /**
     * Speichert die direkt untergeordneten Knoten
     *
     * @param Array
     * @return Sicore_models_Node
     */
    public function setChilds(array $childNodes)
    {
        $this->_childs = $childNodes;
        return $this;
    }
    
    public function getLabel($doClean = true)
    {
        $label = '';
        
        if(!isset($this->_properties[$this->_labelKey])){
            $label =  isset($this->_properties['title']) 
                      ? $this->_properties['title']->value() 
                      : $this->_properties['uid']->value();
        }else{
            $label = $this->_properties[$this->_labelKey]->value(); 
        }
        if($doClean){
            $label = strip_tags($label);
        }
        
        return $label;
    }
    
    public function hasSingleView()
    {
        return false;
    }
       
}

