<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Node
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Content.php 1556 2010-10-21 12:40:22Z ms $
 */
class Sicore_Models_Node_Content extends Sicore_Models_Node{
    
    protected $_key       = null;
    protected $_icon      = null; 
    protected $_iconFE    = false; 
    protected $_class     = null;
    protected $_title     = null;
    protected $_detaillink= null;
    protected $_configKey = null;
    protected $_labelKey  = null;
    protected $_hasDetailView  = false;
    protected $_hasSearchSingleView = false;
    
    public function __construct($rawData = null, $configKey = null)
    {
        if($configKey){
            $this->setConfig($configKey);
        }
        
        parent::__construct($rawData);        
    }
    
    public function setConfig($configKey)
    {
        $configContent = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_CONTENT);    
        $this->_loadConfig($configContent->$configKey);
        
        $this->_key                 = $configContent->$configKey->key      ? $configContent->$configKey->key      : null; 
        $this->_icon                = $configContent->$configKey->icon     ? $configContent->$configKey->icon     : null;
        $this->_iconFE              = $configContent->$configKey->iconFE   ? $configContent->$configKey->iconFE   : false;
        $this->_class               = $configContent->$configKey->class    ? $configContent->$configKey->class    : null;
        $this->_title               = $configContent->$configKey->title    ? $configContent->$configKey->title    : null;
        $this->_labelKey            = $configContent->$configKey->labelKey ? $configContent->$configKey->labelKey : null;
        $this->_hasDetailView       = ($configContent->$configKey->detail && $configContent->$configKey->detail->button); 
        $this->_hasSearchSingleView = ($configContent->$configKey->detail && $configContent->$configKey->detail->search); 
        $this->_optionsTitle        = $configContent->$configKey->options && $configContent->$configKey->options->button ? $configContent->$configKey->options->title : '';
        
        if($configContent->$configKey->detail && $configContent->$configKey->detail->button ){
            if($configContent->$configKey->detail->title){
                $this->_detaillink = $configContent->$configKey->detail->title;
            }else{
                $translate = Sicore_Models_Registry::getTranslate(); 
                $this->_detaillink = $translate->_('GENERAL_DETAILLINK');
            }
        }else{
            $this->_detaillink = '';
        }
        $this->_configKey  = $configKey;
    }
    
    public function getOptionstitle()
    {
        return $this->_optionsTitle;
    }
    
    public function getDetaillink()
    {
        if(($this->disablelistview instanceof Sicore_Models_Node_Property) && $this->disablelistview->value()){
            return '';
        }
        return $this->_detaillink; 
    }
    
    
    public function __toString()
    {
        return (get_class($this));
    }
    
    public function getConfig()
    {
        return 'default';
    }
    
    public function getConfigKey()
    {
        return $this->_configKey != get_class($this) ? $this->_configKey : '';
    }
    
    public function getKey()
    {
        return $this->_key;
    } 
    
    public function getIcon()
    {
        return $this->_icon;  
    }
    
    public function isFrontendIcon()
    {
        return (bool)$this->_iconFE;  
    }
    
    public function getClass()
    {
        return $this->_class;  
    }
    
    public function getTitle()
    {
        return $this->_title ? $this->_title : strtoupper(get_class($this));  
    }
    
    public function getLabel($doClean = true, $shorten = 0, $shortenSuff = '...')
    {
        $label = '';
        if(!isset($this->_properties[$this->_labelKey])){
            $label =  isset($this->_properties['title']) 
                      ? $this->_properties['title']->value() 
                      : $this->_properties['uid']->value();
        }else{
            $label = $this->_properties[$this->_labelKey]->value(); 
        }
        if($doClean){
            $label = strip_tags($label);
            $label = trim(str_replace(array("  ", "\n"), "", $label));
        }
        
        if($shorten = intval($shorten)){
            $label = strlen($label) > $shorten 
                     ? substr($label, 0, $shorten) 
                     : $label;
        }
        return empty($label) ? $this->_properties['uid']->value() : $label;
    }
    
    public function hasSingleView()
    {
        return $this->_hasDetailView;
    }
    
    public function hasSearchSingleView()
    {
        return $this->_hasSearchSingleView;   
    }
}
?>
