<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Node
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: MultiInput.php 1509 2010-08-19 16:13:41Z ms $
 */
class Sicore_Models_Node_Property_MultiInput extends Sicore_Models_Node_Property
                                       implements Iterator, Sicore_Models_Node_Property_Database_Interface{
    
    const LINEBREAK = "\r\n";
        
    public function set($value, $rootUid = null, $canPublish = false, $deleteSubfields = false)
    {
        if(empty($value)){
            return;
        } 
        
        $this->_value = self::extractFromRawData($value);
        $this->_rootUid = $rootUid;    
        $this->_canPublish = (bool)$canPublish;   
        $this->_deleteSubfields = (bool)$deleteSubfields; 
    }
    
    public function get($key)
    {
        if(!isset($this->_value[$key])){
            return;
        }
        
        return $this->_value[$key];
    }
    
    public function __get($key)
    {
        $this->get($key);
    }
    
    public function add($key, $value)
    {
        $value = str_replace('=', '', $value);
        $value = str_replace(self::LINEBREAK, '', $value);
        
        $this->_value[$key] = $value;
    } 
    
    public function getRawData(){ 
        if(!count($this->_value)){
            return '';
        }
        
        $rawArray = array();
        foreach($this->_value as $key=>$value){
            $rawArray[] = "$key=$value";
        }
        return implode(self::LINEBREAK, $rawArray);
    } 
    
    public function compare($rawData = ''){
        if(!($meta = self::extractFromRawData($rawData))){
            return;
        }
        foreach($meta as $key => $value){
            if(isset($this->_value[$key])){
                continue;
            }
            $this->_value[$key] = $value; 
        }  
    } 
    
    public function __toString()
    {
        return $this->getRawData();
    }
    
    static public function extractFromRawData($rawData)
    {
        $data = array();
        if(empty($rawData) || is_array($rawData)){
            return $rawData;
        }
        
        $rows = explode(self::LINEBREAK, $rawData);
        
        foreach($rows as $row){
            $strpos = strpos($row, '=');
            if($strpos === false){
                continue;
            }
            $key   = trim(substr($row, 0, $strpos));
            $value = trim(substr($row, $strpos + 1));
            $data[$key] = $value; 
        }
        
        return $data; 
    } 
    
    public function rewind()
    {
        reset($this->_value);
    }
    
    public function valid()
    {
        return current($this->_value);
    }
    
    public function next()
    {
        next($this->_value);    
    }
    
    public function key()
    {
        return key($this->_value);
    }
    
    public function current()
    {
        return current($this->_value); 
    }
    
    public function toDatabase(Array $data){
        
        $data[$this->field()] = stripslashes(strip_tags($this->getRawData()));
        return $data;
    }
}
?>
