<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Registry.php 1510 2010-08-20 13:45:38Z ms $
 */
class Sicore_Models_Registry{
    
    const CONFIG_BASE       = 'base';
    const CONFIG_FRONTEND   = 'frontend';
    const CONFIG_BACKEND    = 'backend';
    const CONFIG_NODES      = 'nodes';
    const CONFIG_CONTENT    = 'content';
    const CONFIG_FILESYSTEM = 'filesystem';
    const CONFIG_USERS      = 'users'; 
    const DB_DEFAULT        = 'simploo';
    
    private static $_data     = null;
    private static $_instance = null;
    
    public function __construct(Array $config, Zend_Db_Adapter_Abstract $db, Zend_Acl $acl, Array $plugins = array())
    {
        if(self::$_instance instanceof Sicore_Models_Registry){
            return;
        }
        
        self::$_data     = new stdClass();
        
        foreach($config as $key => $data){
            switch($key){
                case self::CONFIG_BASE:
                case self::CONFIG_FRONTEND:
                case self::CONFIG_BACKEND:
                case self::CONFIG_NODES: 
                case self::CONFIG_FILESYSTEM:
                case self::CONFIG_CONTENT: 
                case self::CONFIG_USERS:  
                    $name = "config_$key";
                    self::$_data->$name = $data;
                    break;
            }
        }
        
        foreach($plugins as $plugin => $data){
            $name = "config_plugin_$plugin";
            self::$_data->$name = $data;
        }
        
        self::$_data->db_adapter = $db;
        self::$_data->acl        = $acl;
        self::$_data->page       = null;
        self::$_data->content    = null;
        self::$_data->user       = null;
 
        self::$_instance = $this; 
    }
        
    public static function getConfig($key = self::CONFIG_BASE)
    {
        switch($key){
            case self::CONFIG_BASE:
            case self::CONFIG_FRONTEND:
            case self::CONFIG_BACKEND:
            case self::CONFIG_NODES:
            case self::CONFIG_FILESYSTEM: 
            case self::CONFIG_CONTENT:
            case self::CONFIG_USERS:  
                break;
            default:
                throw new Sicore_Models_Registry_Exception("Keine Konfiguration gefunden: $key");  
        }
        
        $name = "config_$key";
        return self::$_data->$name; 
    }     
    
    public static function getPluginConfig($key)
    {
        return self::_getKey("config_plugin_$key", 'Keine Plugin-Konfiguration gefunden');   
    }
    
    public static function getDbAdapter($key = null)
    {
        $name = $key ? "db_adapter_$key" : "db_adapter"; 
        return self::_getKey($name, 'Keine DB-Adapter gefunden'); 
    }
    
    public static function getAcl()
    {
        return self::_getKey("acl");
    }
    
    public static function getPage()
    {
        return self::_getKey("page"); 
    }
    
    public static function getLocale()
    {
        return self::_getKey("locale"); 
    }
    
     public static function getLogger($mode)
    {
        if(!isset(self::$_data->logger[$mode])) return null;
        
        return self::$_data->logger[$mode]; 
    }
    
    public static function setCache(Zend_Cache_Core $cache)
    {
        return Zend_Registry::set('Zend_Cache', $cache);
    }
    
    public static function getCache()
    {
        return Zend_Registry::get('Zend_Cache');
    }
    
    public static function setTranslate(Zend_Translate $translate)
    {
        return Zend_Registry::set('Zend_Translate', $translate);
    }
    
    public static function getTranslate()
    {
        return Zend_Registry::get('Zend_Translate');
    }
    
    public static function getContent()
    {
        return self::_getKey("content"); 
    }
    
    public static function getUser()
    {
        return Zend_Auth::getInstance()->getIdentity();
    }
    
    public static function get($key)
    {
        return self::_getKey("custom_$key", 'Keine Eintrag gefunden'); 
    }
    
    public static function setAuthResult(Zend_Auth_Result $authResult)
    {
        $name = "auth_result";  
        self::$_data->$name = $authResult;   
    }
    
    public static function setDbAdapter($key, Zend_Db_Adapter_Abstract $adapter)
    {
        $name = $key ? "db_adapter_$key" : 'db_adapter';
        self::$_data->$name = $adapter; 
    }
     
    public static function setPage(Sicore_Models_Node_Page $page)
    {
        self::$_data->page = $page; 
    }
    
    public static function setLocale(Zend_Locale $locale)
    {
        self::$_data->locale = $locale; 
    }
    
     public static function setLogger(Zend_Log $logger, $mode)
    {
        self::$_data->logger[$mode] = $logger; 
    }
    
    public static function setContent($content)
    {
        self::$_data->content = $content; 
    }
        
    public static function set($key, $value)
    {
        switch($key){
            case self::CONFIG_BASE:
            case self::CONFIG_FRONTEND:
            case self::CONFIG_BACKEND:
            case self::CONFIG_NODES: 
            case self::CONFIG_FILESYSTEM:
            case self::CONFIG_CONTENT: 
            case self::CONFIG_USERS:  
                $name = "config_$key";
                self::$_data->$name = $data;
                break;
            default:
                $name = "custom_$key";
                self::$_data->$name = $value;
        }    
    }
    
    public static function getTableFields($table, $fieldPrefix = '', $aliasPrefix = '', $excludeFields = array())
    {
        
        $config_base = self::getConfig();
        
        if(!(isset($config_base->database->tables->$table) && 
             isset($config_base->database->tables->$table->fields) &&
             isset($config_base->database->tables->system->$table) && 
             isset($config_base->database->tables->system->$table->fields)
            )){
            return;
        }
        
        $fieldsSytem       = str_replace(' ', '', $config_base->database->tables->system->$table->fields); 
        $fieldsSytemArray  = explode(',', $fieldsSytem);  
        
        $fieldsCustom      = str_replace(' ', '', $config_base->database->tables->$table->fields);
        $fieldsCustomArray = explode(',', $fieldsCustom);
        
        $fieldsArray       = array_merge($fieldsSytemArray, $fieldsCustomArray);
        $keys              = array_keys($fieldsArray, '');
        foreach($keys as $key){
            unset($fieldsArray[$key]);
        }
        
        $returnFields = array();
        foreach($fieldsArray as $field){
            if(in_array($field, $excludeFields)) continue;
            $fieldName = $fieldPrefix ? $fieldPrefix.'.'.trim($field) : trim($field);
            $fieldName = $aliasPrefix ? $fieldName.' AS '.$aliasPrefix.trim($field) : $fieldName;
            $returnFields[trim($field)] = $fieldName;
        }
        
        return implode(',', array_diff_key($returnFields, $excludeFields));
    }
    
    
    private static function _getKey($key, $errorMessage = '')
    {

        if(!property_exists(self::$_data, $key) ){
            $errorMessage = $errorMessage ? "$errorMessage [$key]" :$key;
            throw new Sicore_Models_Registry_Exception($errorMessage); 
        }
        
        return self::$_data->$key; 
    }
}

