<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Files.php 1553 2010-10-13 09:52:56Z ms $
 */
class Sicore_Models_Server_Db_Files extends Sicore_Models_Server_Db
{
    
    protected $_doLoadParents = true;
    
    /**
     * Elternordner laden
     *
     * @param Boolean $load Standard = true
     * @return $this
     */
    public function doLoadParents($load = true)
    {
        $this->_doLoadParents = (bool)$load;
        return $this;
    }
        
    /**
     * Einzelnen Ordner laden
     *
     * @param Integer $uid Ordner-ID 
     * @return Sicore_Models_FSO_File
     * 
     */
    public function get($uid)
    {
        //Node laden
        $si_files_fields     = Sicore_Models_Registry::getTableFields('si_files'); 
        $si_folders_fields    = Sicore_Models_Registry::getTableFields('si_folders'); 
        
        $sql = "SELECT $si_files_fields FROM si_files WHERE uid = ?";
        
        if(!($data = $this->_con->fetchRow($sql, array(intval($uid))))){     
            return null;
        }
        
        $className = $data->class;
        if(!class_exists($className)){
            Sicore_Models_logger::error("Klasse '$className' existiert nicht.");
            return null;
        }
        $file      = new $className($data);
        $parents   = array();
        
        if(!$this->_doLoadParents){
            return $file;
        }
        
        $sql = "SELECT $si_folders_fields
                FROM si_folders  
                INNER JOIN si_folders_folders 
                   ON uid = folders_uid_parent  
                WHERE folders_uid_child = ?";
      
        if(!($data = $this->_con->fetchAll($sql, array(intval($file->folders_uid->value()))))){
            return $file;
        } 
          
        foreach($data as $folderData){
            
            $file->addParent(new Sicore_Models_FSO_Folder($folderData));     
        } 
                        
        return $file;  
    }
    
     
    /**
     * Übergeordneten Ordner laden
     *
     * @param Sicore_Models_FSO_File $folder 
     * @return Sicore_Models_FSO_File
     */
    public function getParentFolder(Sicore_Models_FSO_File $file)
    {
        $mFolder = new Sicore_Models_FSO_Folder();
        return $mFolder->get($file->folders_uid->value());
    }
    
    /**
     * Alle Unterordner laden
     *
     * @param Integer $uid übergeordnete Ordner-Id 
     * @return array
     */
    public function getByFolder($uid, $includeClasses = array(), $limit = 0)
    {        
        //Ordner laden
        $si_files_fields  = Sicore_Models_Registry::getTableFields('si_files');  
        
        $sql = "SELECT $si_files_fields 
                FROM si_files 
                WHERE folders_uid = ".intval($uid)."
                ORDER BY name";
        $sql.= $limit ? " LIMIT ".intval($limit) : ""; 
        
        if(!($rawFiles = $this->_con->fetchAll($sql))){
            return array();
        }
        
        $files = array(); 
        foreach($rawFiles as $rawFile){
            $className = $rawFile->class;
            if(!class_exists($className)){
                Sicore_Models_logger::error("Klasse '$className' existiert nicht.");
                continue;
            }
            $file = new $className($rawFile);
            if(count($includeClasses)){
                foreach($includeClasses as $class){
                    if($file instanceof $class){
                        $files[] = $file; 
                        break;
                    }
                }
            }else{
                $files[] = $file;
            }       
        }
        
        return $files;     
    }
    
    /**
     * Alle Unterordner laden
     *
     * @param String $name Bildname
     * @param Integer $folder Ordner-Id 
     * @return array
     */
    public function getByName($name, $folder)
    {        
        //Ordner laden
        $si_files_fields     = Sicore_Models_Registry::getTableFields('si_files');  
        
        $sql = "SELECT $si_files_fields 
                FROM si_files 
                WHERE name = ".$this->_con->quote($name)." AND folders_uid = ".intval($folder)."
                LIMIT 1";
        
        if(!($rawFile = $this->_con->fetchRow($sql))){
            return false;
        }

        $className = $rawFile->class;
        if(!class_exists($className)){
            Sicore_Models_logger::error("Klasse '$className' existiert nicht.");
            return false;
        } 
        $file = new $className($rawFile); 

        return $file;     
    }
    
    
    public function getByRand($folder)
    {        
        //Ordner laden
        $si_files_fields     = Sicore_Models_Registry::getTableFields('si_files');  
        
        $sql = "SELECT $si_files_fields 
                FROM si_files 
                WHERE folders_uid = ".intval($folder)."
                LIMIT 1 
                ORDER BY RAND()";
        
        if(!($rawFile = $this->_con->fetchRow($sql))){
            return false;
        }

        $className = $rawFile->class;
        if(!class_exists($className)){
            Sicore_Models_logger::error("Klasse '$className' existiert nicht.");
            return false;
        } 
        $file = new $className($rawFile); 

        return $file;     
    }
    
    
        
    /**
     * Ändert einen Knoten ab
     *
     * @param Sicore_Models_FSO_File $folder 
     */
    public function update(Sicore_Models_FSO_File $file)
    {
        
        $data = array();
        foreach($file as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            $data = $property->toDatabase($data);
        }
         
        $this->_con->update('si_files', $data, "uid = ".$file->uid->value());        
         
        return true;
    }
    
    /**
     * Erstellt einen Ordner
     *
     * @param Sicore_Models_FSO_File $folderDest Übergeordneter Ordner
     * @param String $name Name des neuen Ordner  
     */
    public function create(Sicore_Models_FSO_File $file) 
    {       
        $data = array();        
        foreach($file as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            $data = $property->toDatabase($data);
        }
        
        $this->_con->insert('si_files', $data);        
        $newId = $this->_con->lastInsertId(); 
        $file->uid->set($newId);
        $file->onCreate();  
         
        return $newId;    
    }
    
    /**
     * Entfernt einen Knoten
     *
     * @param Sicore_Models_Node $node
     * @return Boolean Knoten konnte gelöscht werden (ja/nein)  
     */
    public function remove(Sicore_Models_FSO_File $file) 
    {
        $file->onDelete();
        $this->_con->delete('si_files', 'uid = '.$file->uid->value());       
    }
    
    public function move(Sicore_Models_FSO_File $fileSource, Sicore_Models_FSO_Folder $folderDest, $type)
    {
        $oldFolder = $fileSource->folders_uid->value();
        switch($type){
            case 'after':
            case 'before':
                $newFolder = $folderDest->uid_parent->value();
                break;
            case 'inside':
                $newFolder = $folderDest->uid->value(); 
                break;
            default:
                return false;
        }
        
        $fileSource->folders_uid->set($newFolder); 
        $data = array();
        foreach($fileSource as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            $data = $property->toDatabase($data);
        }
        
        try{
            $this->_con->update('si_files', $data, 'uid = '.$fileSource->uid->value()); 
            $fileSource->onMove($oldFolder, $newFolder);
        }catch(Exception $e){
            Sicore_Models_Logger::exception($e); 
        }   
    }  
    
    public function copy(Sicore_Models_FSO_File $folderSource, Sicore_Models_FSO_File $folderDest, $type)
    {
        $folderNew = clone $folderSource;
        
        $folderNew->uid->set(null);
        if($type == 'after' || $type == 'before'){
            $folderNew->uid_parent->set($folderDest->uid_parent->value());  
            $folderNew->level->set($folderDest->level->value());
        }else{
            $folderNew->uid_parent->set($folderDest->uid->value());  
            $folderNew->level->set($folderDest->level->value() + 1); 
        }
        
        $data = array();
        foreach($folderNew as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            $data = $property->toDatabase($data);
        }
       
        //TODO: Folders_Folders anpassen
         
        $this->_con->insert('si_folders', $data); 
    } 
    
    public function getChilds(Sicore_Models_FSO_File $folder)
    {
        $sql = "SELECT uid 
                FROM si_folders 
                INNER JOIN si_folders_folders
                   ON uid = folders_uid_child
                WHERE folders_uid_parent = ".intval($folder->uid->value());
        
        if(!($rawFolders = $this->_con->fetchAll($sql))){
            return array();
        }
        
        $folders = array();
        foreach($rawFolders as $rawFolder){
            $folders[] = new Sicore_Models_FSO_File($rawFolder);       
        }
        
        return $folders;
    }    
    
    
}
?>
