<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Folders.php 1225 2010-03-06 20:11:33Z ms $
 */
class Sicore_Models_Server_Db_Folders extends Sicore_Models_Server_Db
{
    
    protected $_doLoadParents = true;
    
    /**
     * Elternordner laden
     *
     * @param Boolean $load Standard = true
     * @return $this
     */
    public function doLoadParents($load = true)
    {
        $this->_doLoadParents = (bool)$load;
        return $this;
    }
    
    public function getRootFolder()
    {
        $si_folders_fields = Sicore_Models_Registry::getTableFields('si_folders');  
        
        $sql = "SELECT $si_folders_fields FROM si_folders WHERE level = 0 ORDER BY uid LIMIT 1";
        
        if(!($data = $this->_con->fetchRow($sql, array(intval($uid))))){     
            return null;
        }
        
        return new Sicore_Models_FSO_Folder($data); 
    }
        
    /**
     * Einzelnen Ordner laden
     *
     * @param Integer $uid Ordner-ID 
     * @return Sicore_Models_FSO_Folder
     * 
     */
    public function get($uid)
    {
        //Node laden
        $si_folders_fields     = Sicore_Models_Registry::getTableFields('si_folders'); 
        
        $sql = "SELECT $si_folders_fields FROM si_folders WHERE uid = ?";
        
        if(!($data = $this->_con->fetchRow($sql, array(intval($uid))))){     
            return null;
        }
        
        $folder    = new Sicore_Models_FSO_Folder($data);
        $parents   = array();
        
        if(!$this->_doLoadParents){
            return $folder;
        }
        
        $sql = "SELECT $si_folders_fields
                FROM si_folders  
                INNER JOIN si_folders_folders 
                   ON uid = folders_uid_parent  
                WHERE folders_uid_child = ?";
        
        if(!($data = $this->_con->fetchAll($sql, array(intval($folder->uid->value()))))){
            return $folder;
        } 
          
        foreach($data as $folderData){
            
            $folder->addParent(new Sicore_Models_FSO_Folder($folderData));     
        } 
                        
        return $folder;  
    }
    
     
    /**
     * Übergeordneten Ordner laden
     *
     * @param Sicore_Models_FSO_Folder $folder 
     * @return Sicore_Models_FSO_Folder
     */
    public function getParentFolder(Sicore_Models_FSO_Folder $folder)
    {
        return $this->get($folder->uid_parent->value());
    }
    
    /**
     * Alle Unterordner laden
     *
     * @param Integer $uid übergeordnete Ordner-Id 
     * @return array
     */
    public function getByParentUid($uid)
    {        
        $sqlUidparent = (intval($uid) == 0) ? '(uid_parent IS NULL OR uid_parent = 0)' : " uid_parent = ".intval($uid); 
        
        //Ordner laden
        $si_folders_fields     = Sicore_Models_Registry::getTableFields('si_folders'); 
        
        $sql = "SELECT $si_folders_fields 
                FROM si_folders 
                WHERE $sqlUidparent 
                ORDER BY name";
        
        if(!($rawFolders = $this->_con->fetchAll($sql))){
            return array();
        }
        
        $folders = array(); 
        foreach($rawFolders as $rawFolder){ 
            $folders[] = new Sicore_Models_FSO_Folder($rawFolder);    
        }
        return $folders;     
    }
        
    /**
     * Ändert einen Knoten ab
     *
     * @param Sicore_Models_FSO_Folder $folder 
     */
    public function update(Sicore_Models_FSO_Folder $folder)
    {
        $data = array();
        foreach($folder as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            if($property->field() == 'uid_parent' && !$property->value()) continue;
            $data = $property->toDatabase($data);
        }
        
        $this->_con->update('si_folders', $data, "uid = ".$folder->uid->value());        
         
        return true;
    }
    
    /**
     * Erstellt einen Ordner
     *
     * @param Sicore_Models_FSO_Folder $folderDest Übergeordneter Ordner
     * @param String $name Name des neuen Ordner  
     */
    public function create(Sicore_Models_FSO_Folder $folderDest, $name, $mode) 
    {
        $folderNew = clone $folderDest;
        
        if($mode == 'inside'){
            $parent = $folderDest->uid->value();
            $level  = $folderDest->level->value() + 1; 
        }else{
            $parent = $folderDest->uid_parent->value();
            $level  = $folderDest->level->value();
        }
        
        
        $folderNew->uid->set(null);
        $folderNew->level->set($level);
        $folderNew->name->set($name);
        $folderNew->uid_parent->set($parent); 
        $data = array();
        foreach($folderNew as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            $data = $property->toDatabase($data);
        }
        
        $this->_con->insert('si_folders', $data);        
        $newId = $this->_con->lastInsertId(); 
        
        //Folders_Folders ergänzen
        if($mode == 'inside'){
            $sql = "INSERT INTO si_folders_folders  
                      (folders_uid_child, folders_uid_parent)
                    VALUES
                      (".intval($newId).", ".intval($parent).")
                    ";  
            $this->_con->exec($sql);
        }
        $rootline = $folderDest->getAllParents();
        foreach($rootline as $level => $folder){
            $sql = "INSERT INTO si_folders_folders  
                      (folders_uid_child, folders_uid_parent)
                    VALUES
                      (".intval($newId).", ".intval($folder->uid->value()).")
                    ";  
            $this->_con->exec($sql);
        }
        
        $folderNew->uid->set($newId);
        $folderNew->onCreate();  
         
        return $newId;    
    }
    
    /**
     * Entfernt einen Knoten
     *
     * @param Sicore_Models_Node $node
     * @return Boolean Knoten konnte gelöscht werden (ja/nein)  
     */
    public function remove(Sicore_Models_FSO_Folder $folder) 
    {
        $childs = $this->getChilds($folder);
        foreach($childs as $child){
            $child->onDelete();
        }
        $this->_con->delete('si_folders', 'uid = '.$folder->uid->value());
        $folder->onDelete();     
    }
    
    public function move(Sicore_Models_FSO_Folder $folderSource, Sicore_Models_FSO_Folder $folderDest, $type)
    {
        $this->_con->beginTransaction();
        try{
            $oldLevel  = $folderSource->level->value();
            $levelDiff = $folderDest->level->value() - $folderSource->level->value(); 
            switch($type){
                case 'after':
                case 'before':
                    $folderDestUid = $folderDest->uid_parent->value();  
                    $folderSource->level->set($folderDest->level->value());           
                    break;
                case 'inside':
                    $folderDestUid = $folderDest->uid->value();
                    $folderSource->level->set($folderDest->level->value() + 1);
                    $levelDiff++; 
                    break;
                default:
                    return false; 
            }
            $folderSource->uid_parent->set($folderDestUid);
             
            $data = array();
            foreach($folderSource as $property){
                if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
                $data = $property->toDatabase($data);
            }
            
            $sql = "DELETE FROM si_folders_folders 
                    WHERE folders_uid_child = ".intval($folderSource->uid->value());
            $this->_con->exec($sql);
            $sql = "INSERT INTO si_folders_folders
                       (folders_uid_parent, folders_uid_child)
                    VALUES
                       (".intval($folderDestUid).", ".intval($folderSource->uid->value()).")";
            $this->_con->exec($sql);
            $sql = "INSERT INTO si_folders_folders
                       (folders_uid_parent, folders_uid_child)
                    SELECT folders_uid_parent, ".intval($folderSource->uid->value())." 
                    FROM si_folders_folders 
                    WHERE folders_uid_child = ".intval($folderDestUid);
            $this->_con->exec($sql);
            $sql = "SELECT DISTINCT folders_uid_child 
                    FROM si_folders_folders 
                    WHERE folders_uid_parent = ".intval($folderSource->uid->value()); 
            $childs = $this->_con->fetchCol($sql);
            if($childs){  
                $sql = "DELETE si_folders_folders AS ff 
                        FROM si_folders_folders AS ff 
                        INNER JOIN si_folders AS f ON ff.folders_uid_parent=f.uid WHERE ff.folders_uid_child IN(".implode(',', $childs).") AND f.level < ".intval($oldLevel); 
                $this->_con->exec($sql);
                foreach($childs as $child){
                    $sql = "INSERT INTO si_folders_folders
                               (folders_uid_parent, folders_uid_child)
                            SELECT folders_uid_parent, ".intval($child)." 
                            FROM si_folders_folders 
                            WHERE folders_uid_child = ".intval($folderDestUid);
                    $this->_con->exec($sql); 
                    $sql = "INSERT INTO si_folders_folders
                               (folders_uid_parent, folders_uid_child)
                            VALUES
                               (".intval($folderDestUid).", ".intval($child).")";
                    $this->_con->exec($sql);
                } 
                $sql = "UPDATE si_folders 
                        SET level = level + ".$levelDiff."
                        WHERE uid IN (".implode(',', $childs).")"; 
                $this->_con->exec($sql);          
            } 
            $this->_con->update('si_folders', $data, 'uid = '.$folderSource->uid->value()); 
            $this->_con->commit();    
        }catch(Exception $e){
            Sicore_Models_Logger::exception($e); 
            $this->_con->rollback();
            return false;
        }
        
        return true;
    }  
    
    
    public function getChilds(Sicore_Models_FSO_Folder $folder)
    {
        $sql = "SELECT uid 
                FROM si_folders 
                INNER JOIN si_folders_folders
                   ON uid = folders_uid_child
                WHERE folders_uid_parent = ".intval($folder->uid->value());
        
        if(!($rawFolders = $this->_con->fetchAll($sql))){
            return array();
        }
        
        $folders = array();
        foreach($rawFolders as $rawFolder){
            $folders[] = new Sicore_Models_FSO_Folder($rawFolder);       
        }
        
        return $folders;
    }    
    
    
}
?>
