<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Navigation.php 1518 2010-08-26 14:30:10Z ms $
 */
class Sicore_Models_Server_Db_Navigation extends Sicore_Models_Server_Db
{
    
    
    protected $_language     = 'de';
    protected $_level        = 0;
    protected $_depth        = -1; 
    protected $_doExpandAll  = false;
    protected $_uidArray     = null; 
    protected $_taggedPages  = null;
    protected $_excludeTags = array();
    
    /**
     * Entwurfsansicht bei der Navigation ja/nein
     *
     * @var Boolean
     */
    protected $_isPreviewView  = false; 
    
    
    public function __construct()
    {
        parent::__construct();
        $this->reset();
    }

    public function reset()
    {
        $this->_level       = 0;
        $this->_depth       = -1;
        $this->_doExpandAll = false;
        $this->_excludeTags = array(); 
        $this->_taggedPages = null; 
        $this->_uidArray    = null; 
    }
    
    /**
     * Entwurfsansicht bei der Navigation an-/auschalten
     *
     * @param Boolean $check Standard = true
     * @return Sicore_Models_Server_Db_Nodes Fluent Interface
     */
    public function setPreviewView($check = false)
    {
        $this->_isPreviewView = (bool)$check;
        return $this;
    }
    
    public function get($uidArray = array())
    {
        $this->_uidArray = $uidArray;
        return $this;
    }
    
    public function getUidArray(Sicore_Models_Node_Page $currentPage)
    {
        if(!$this->_checkPageAccess($currentPage)){
            return array();
        }
        
        $model_nodes = new Sicore_Models_Server_Db_Nodes();     
        $reflect     = new ReflectionClass($model_nodes);
        $props       = $reflect->getProperties();
        
        $pages = array();
        foreach($this->_uidArray as $uid){
            try{
                $cache   = Sicore_Models_Registry::getCache();  
                $key     = 'menu_page_'.($model_nodes->getDraft() ? 'draft' : 'live').'_'.md5(serialize($props).$uid);
                if(!$page = $cache->load($key)) { 
                    $page =  $model_nodes->get($uid, $this->_language, $this->_isPreviewView);
                    $cache->save($page, $key, array('menu', 'menu_page', (string)$uid, $model_nodes->getDraft() ? 'draft' : 'live'));
                }
                
                if(!($page instanceof Sicore_Models_Node_Page) || !$this->_checkPageAccess($page)){
                    continue;
                }
                $page->isInRootline($this->_checkIfIsInRootline($currentPage, $uid)); 
                $page->isSelected($uid === $currentPage->uid->value());
                $pages[] = $page;  
            }catch(Sifront_Models_Exception_Pagenotfound $e){  }
              
        }
        
        return $pages;
    }
    
    public function getByTag($tag)
    {
        $mNodes      = new Sicore_Models_Server_Db_Nodes();     
        $reflect     = new ReflectionClass($mNodes);
        $props       = $reflect->getProperties();
        
        $cache       = Sicore_Models_Registry::getCache(); 
        $key         = 'menu_tags_'.($mNodes->getDraft() ? 'draft' : 'live').'_'.md5(serialize($props).$tag);  
        $this->_taggedPages   = $cache->load($key);
        if(!is_array($this->_taggedPages)) { 
            $this->_taggedPages  = $mNodes->getByTag($tag, $this->_language, $this->_isPreviewView);
            $cache->save($this->_taggedPages, $key, array('menu', (string)$tag, $mNodes->getDraft() ? 'draft' : 'live'));
        }
        return $this;
    }
    
    public function getMenu(Sicore_Models_Node_Page $currentPage, $level = null)
    {
        
       if(is_array($this->_taggedPages)){
            $pages = array();
            foreach($this->_taggedPages as $taggedPage){
                if(!$this->_checkPageAccess($taggedPage)) continue;
                $pages[] = $taggedPage;
            }
            return  $pages;
        } 
        
        if(is_array($this->_uidArray)){
            return $this->getUidArray($currentPage);
        }
        
        if(!$this->_checkPageAccess($currentPage)){
            return array();
        }
                
        if($this->_doExpandAll){
            return $this->getSitemap($currentPage);
        }
        
        if($level == null){
            $level = $this->_level;   
        }
        
        if($level > $this->_level + $this->_depth && $this->_depth >= 0){
            return array();
        }
        
        $checkParents = ($currentPage->getTree()->level->value() > 0 && $level > 0);
        
        $model_nodes = new Sicore_Models_Server_Db_Nodes(); 
        $reflect     = new ReflectionClass($model_nodes);
        
        $pages         = array();
        $pages_visible = array();
         
        if($currentPage->getParent($level) instanceof Sicore_Models_Node_Page){
            
            $cache     = Sicore_Models_Registry::getCache();  
            $uid2load  = $currentPage->getParent($level)->getTree()->parent->value();
            
            $model_nodes->doCheckIsHidden(true)
                        ->setLanguage($this->_language)
                        ->setDraft($this->_isPreviewView)
                        ->doCheckParents($checkParents);
                        
            $props   = $reflect->getProperties(); 
            $key     = 'menu_'.($model_nodes->getDraft() ? 'draft' : 'live').'_'.md5(serialize($props).$uid2load.serialize(array('Sicore_Models_Node_Page'))); 
            $pages   = $cache->load($key);
            if(!is_array($pages)) {  
                $pages = $model_nodes->getByParentUid($uid2load, array('Sicore_Models_Node_Page'));
                $cache->save($pages, $key, array('menu', (string)$uid2load, $model_nodes->getDraft() ? 'draft' : 'live')); 
            }  
              
            foreach($pages as $sorting => $page){
                if(!$this->_checkPageAccess($page)){
                    continue; 
                }
                if(count($this->_excludeTags)){
                    $pageTags = $page->tags->toArray();
                    $tagFound = false;
                    foreach($this->_excludeTags as $tag){ 
                        if(in_array($tag, $pageTags)) $tagFound = true;
                    }
                    if($tagFound) continue;
                }
                if($pages[$sorting]->uid->value() == $currentPage->getParent($level)->uid->value()){
                    $pages[$sorting]->setChilds($this->getMenu($currentPage, $level+1)); 
                }
                $pages[$sorting]->isSelected($pages[$sorting]->uid->value() == $currentPage->uid->value());
                $pages[$sorting]->isInRootline($this->_checkIfIsInRootline($currentPage, $pages[$sorting]->uid->value()));
                
                $pages_visible[] = $page;
            }   
        }elseif($currentPage->getTree()->level->value() == $level){
            
            $cache     = Sicore_Models_Registry::getCache();  
            $uid2load  = $currentPage->getTree()->parent->value();
            
            $model_nodes->doCheckIsHidden(true)
                        ->setLanguage($this->_language) 
                        ->setDraft($this->_isPreviewView) 
                        ->doCheckParents($checkParents);
                        
            $props   = $reflect->getProperties(); 
            $key     = 'menu_'.($model_nodes->getDraft() ? 'draft' : 'live').'_'.md5(serialize($props).$uid2load.serialize(array('Sicore_Models_Node_Page'))); 
            
            $pages   = $cache->load($key);
            if(!is_array($pages)) { 
                $pages = $model_nodes->getByParentUid($uid2load, array('Sicore_Models_Node_Page'));
                $cache->save($pages, $key, array('menu', (string)$uid2load, $model_nodes->getDraft() ? 'draft' : 'live')); 
            } 
            
            foreach($pages as $sorting => $page){ 
                if(!$this->_checkPageAccess($page)){
                    continue;
                }
                if(count($this->_excludeTags)){
                    $pageTags = $page->tags->toArray();
                    $tagFound = false;
                    foreach($this->_excludeTags as $tag){ 
                        if(in_array($tag, $pageTags)) $tagFound = true;
                    }
                    if($tagFound) continue;
                }
                if($pages[$sorting]->uid->value() == $currentPage->uid->value() && ($level + 1 <= $this->_level + $this->_depth || $this->_depth < 0)){
                    $cache     = Sicore_Models_Registry::getCache();  
                    $uid2load  = $currentPage->uid->value();
                    
                    $model_nodes->doCheckIsHidden(true) 
                                ->setLanguage($this->_language)
                                ->setDraft($this->_isPreviewView) 
                                ->doCheckParents($checkParents); 
        
                    $props   = $reflect->getProperties();
                    $key     = 'menu_'.($model_nodes->getDraft() ? 'draft' : 'live').'_'.md5(serialize($props).$uid2load.serialize(array('Sicore_Models_Node_Page')));
                    
                    $subpages   = $cache->load($key);
                    if(!is_array($subpages)) { 
                        $subpages = $model_nodes->getByParentUid($uid2load,  array('Sicore_Models_Node_Page'));
                        $cache->save($subpages, $key, array('menu', (string)$uid2load, $model_nodes->getDraft() ? 'draft' : 'live')); 
                    } 
                    foreach($subpages as $key => $subpage){
                        if(!$this->_checkPageAccess($subpage)){
                            unset($subpages[$key]);;
                        }
                    }
                    $pages[$sorting]->setChilds($subpages); 
                }
                $pages[$sorting]->isSelected($pages[$sorting]->uid->value() == $currentPage->uid->value());
                $pages[$sorting]->isInRootline($this->_checkIfIsInRootline($currentPage, $pages[$sorting]->uid->value()));
                $pages_visible[] = $page; 
            }   
        }elseif($currentPage->getTree()->level->value() + 1 == $level){  
            $cache     = Sicore_Models_Registry::getCache();  
            $uid2load  = $currentPage->uid->value();
            
            $model_nodes->doCheckIsHidden(true)
                        ->setLanguage($this->_language) 
                        ->setDraft($this->_isPreviewView) 
                        ->doCheckParents($checkParents);
        
            $props   = $reflect->getProperties();
            $key     = 'menu_'.($model_nodes->getDraft() ? 'draft' : 'live').'_'.md5(serialize($props).$uid2load.serialize(array('Sicore_Models_Node_Page')));
            
            $pages   = $cache->load($key);
            if(!is_array($pages)) { 
                $pages = $model_nodes->getByParentUid($uid2load,  array('Sicore_Models_Node_Page'));
                $cache->save($pages, $key, array('menu', (string)$uid2load, $model_nodes->getDraft() ? 'draft' : 'live')); 
            } 
            foreach($pages as $sorting => $page){
                if(!$this->_checkPageAccess($page)){
                    continue;
                }
                 if(count($this->_excludeTags)){
                    $pageTags = $page->tags->toArray();
                    $tagFound = false;
                    foreach($this->_excludeTags as $tag){ 
                        if(in_array($tag, $pageTags)) $tagFound = true;
                    }
                    if($tagFound) continue;
                }
                $pages[$sorting]->isSelected($pages[$sorting]->uid->value() == $currentPage->uid->value());
                $pages[$sorting]->isInRootline($this->_checkIfIsInRootline($currentPage, $pages[$sorting]->uid->value()));
                $pages_visible[] = $page;
            }     
        }
        
        return $pages_visible; 
    }
        
    public function getSitemap(Sicore_Models_Node_Page $currentPage, $parent = null, $level = null)
    {
        if(!$this->_checkPageAccess($currentPage)){
            return array();
        }
        
        $level =  $level  == null ? $this->_level : $level;
          
        if(!($currentPage->getParent($level) instanceof Sicore_Models_Node_Page) && $parent == null){
            if($currentPage->getTree()->level->value() < $this->_level){
                $parent = $currentPage->uid->value();    
            }else{
                $parent = $currentPage->getTree()->parent->value();
            }
        }else{
            $parent = $parent == null ? $currentPage->getParent($level)->getTree()->parent->value() : $parent;
        }
        
        $model_nodes = new Sicore_Models_Server_Db_Nodes();

        $pages         = array(); 
        $pages_visible = array();
        
        $checkParents = ($currentPage->getTree()->level->value() > 0); 
        
        $cache     = Sicore_Models_Registry::getCache();  
        
        $model_nodes->doCheckIsHidden(true)
                     ->setLanguage($this->_language) 
                     ->setDraft($this->_isPreviewView)  
                     ->doCheckParents($checkParents); 
        
        $reflect  = new ReflectionClass($model_nodes);
        $props    = $reflect->getProperties();
        $key      = 'menu_'.($model_nodes->getDraft() ? 'draft' : 'live').'_'.md5(serialize($props).$parent.serialize(array('Sicore_Models_Node_Page')));
        
        $pages   = $cache->load($key);
        if(!is_array($pages)) {                    
            $pages = $model_nodes->getByParentUid($parent, array('Sicore_Models_Node_Page')); 
            $cache->save($pages, $key, array('menu', $model_nodes->getDraft() ? 'draft' : 'live'));  
        }
        
        foreach($pages as $page){
            if(!$this->_checkPageAccess($page)){
                continue;
            }
            if($page->getTree()->level->value() + 1 > $this->_level + $this->_depth && $this->_depth >= 0){
                break;
            } 
            if(count($this->_excludeTags)){
                $pageTags = $page->tags->toArray();
                $tagFound = false;
                foreach($this->_excludeTags as $tag){ 
                    if(in_array($tag, $pageTags)) $tagFound = true;
                }
                if($tagFound) continue;
            }
            $page->setChilds($this->getSitemap($currentPage, $page->uid->value())); 
            $page->isSelected($page->uid->value() == $currentPage->uid->value());
            $page->isInRootline($this->_checkIfIsInRootline($currentPage, $page->uid->value()));
            
            $pages_visible[] = $page;    
        } 
        
        return $pages_visible;
    }
    
    public function excludeTags($tags)
    {
        if(is_array($tags)){
            $this->_excludeTags   = $tags;
        }else{
            $this->_excludeTags[] = $tags;    
        }
        return $this;
    }
    
    public function setLevel($level)
    {
        $this->_level = intval($level);
        return $this;
    }
    
    public function setDepth($depth)
    {
        $this->_depth = intval($depth);
        return $this; 
    }
    
    public function doExpandAll($expand)
    {
        $this->_doExpandAll = (boolean)$expand;
        return $this; 
    }
    

    
    public function setLanguage($language)
    {
        $this->_language = $language;
        return $this; 
    }
        
    protected function _checkIfIsInRootline(Sicore_Models_Node_Page $currentPage, $uid)
    {
        
        foreach($currentPage->getAllParents() as $parent)
        {
            if($parent->uid->value() == $uid) return true;
        }
        
        return false;
    }
    
    protected function _checkPageAccess(Sicore_Models_Node_Page $currentPage)
    {
        if(!($accessList = $currentPage->access->toArray())){
            return true;
        }
        
        if(!Zend_Auth::getInstance()->hasIdentity()){
            return false;    
        }
        $user = Zend_Auth::getInstance()->getIdentity();
        if($user->isadmin->value()){
            return true;  
        } 
        foreach($accessList as $role){
            if(Sicore_Models_Registry::getAcl()->hasRole($role)){
                return true;
            }
        }
        
        return false;
    }    
}
?>
