<?php
 /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Urlalias.php 1504 2010-08-17 16:35:17Z ms $
 */
class Sicore_Models_Server_Db_Urlalias extends Sicore_Models_Server_Db 
                                       implements Sicore_Models_Server_Db_Urlalias_Interface
{
    public function getNode($urlalias, $isDraft = false)
    {
        $urlalias = self::decode($urlalias);
        if(empty($urlalias)){
            return false; 
        }
        
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes', 'n'); 
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');
        
        $sql = "SELECT $si_nodes_fields, $si_urlalias_fields      
                FROM si_nodes AS n
                LEFT JOIN si_urlalias AS url
                   ON n.uid = url.nodes_uid  
                WHERE n.permalink = ".$this->_con->quote($urlalias)." OR url.alias = ".$this->_con->quote($urlalias);
        if($isDraft){
            $subQuery2 = Sicore_Models_Server_Db_Nodes::getQueryStringVersionNodes(); 
            $subQuery3 = Sicore_Models_Server_Db_Nodes::getQueryStringVersionUrlalias();  
            $sql .= " AND n.version = ($subQuery2)
                      AND url.version = ($subQuery3)";
        }else{
            $sql .= " AND n.version = 0 AND url.version <= 0 ORDER BY url.version DESC LIMIT 1"; 
        } 
          
        if(!($node = $this->_con->fetchRow($sql))){
            throw new Sifront_Models_Exception_Pagenotfound();
        }
       
        if(!$isDraft){
            if($node->permalink != $urlalias){
                if(!empty($node->permalink)){
                    throw new Sifront_Models_Exception_Pagemoved($node->permalink);
                }elseif($node->urlalias_version < 0){
                    $this->_con->update('si_urlalias', array('lastvisit' => new Zend_Db_Expr('NOW()')), 'uid = '.intval($node->urlalias_uid)." AND version = -1");            
                    $sql = "SELECT url.alias 
                            FROM si_nodes AS n  
                            INNER JOIN si_urlalias AS url
                               ON n.uid = url.nodes_uid
                            WHERE url.nodes_uid = ".$node->uid." 
                                  AND n.version = 0 
                                  AND url.version = 0 
                            LIMIT 1  
                           ";
                     if(!($node2 = $this->_con->fetchRow($sql))){
                        return $node;  
                     }     
                    throw new Sifront_Models_Exception_Pagemoved($node2->alias);
                }            
            }  
       }      
       
       return $node;
    }
    
    public function check($rootAlias, $name, $uid = null, $withSlash = false)
    {
        
        $name = self::decode(trim($name));
         
        if(substr($rootAlias, strlen($rootAlias)-1) != '/'){
            $rootAlias = $rootAlias.'/';
        }
        if(substr($rootAlias, 0, 1) == '/'){
            $rootAlias = substr($rootAlias, 1);
        }
        $name      = self::clear($name, $withSlash);
        $orgAlias  = $rootAlias.$name;
        $alias     = $orgAlias;       
        
        if(intval($uid)){
            $sql = "SELECT uid 
                    FROM si_urlalias  
                    WHERE alias = ? AND NOT nodes_uid = ?   
                    LIMIT 1
                    UNION
                    SELECT uid
                    FROM si_nodes 
                    WHERE permalink = ? AND NOT uid = ?    
                    LIMIT 1";
            $level  = 0;      
            while($this->_con->fetchRow($sql, array($alias, intval($uid), $alias, intval($uid)))){
                $level++;
                $alias  = $orgAlias."_".$level;
            }
            
            return $alias != $name || !$rootAlias ? $alias : false; 
        }else{
            $sql = "SELECT uid 
                    FROM si_urlalias  
                    WHERE alias = ?    
                    LIMIT 1
                    UNION
                    SELECT uid
                    FROM si_nodes 
                    WHERE permalink = ?    
                    LIMIT 1";
              $level  = 0;
               while($this->_con->fetchRow($sql, array($alias,  $alias))){
                    $level++;
                    $alias  = $orgAlias."_".$level;
                }  
                
              return $alias;           
        }    
        
    }
    
    public function clear($name, $withSlash = false)
    {
        if($withSlash){
            $name = preg_replace("/[^a-zA-Z0-9_ \/\-üäöÜÖÄß]/" , "" , $name);
            $name = preg_replace("/\/{2,}/" , "/" , $name);   
        }else{
            $name = preg_replace("/[^a-zA-Z0-9_ üäöÜÖÄß]/" , "" , $name);   
        }
        
        $name = str_replace(array(' ', '--', 'ü', 'Ü', 'ä', 'Ä', 'ö', 'Ö', 'ß'),
                            array('-', '-', 'ue', 'ue', 'ae', 'ae', 'oe', 'oe', 'ss'),
                            $name
                           );
        
        $config_fe = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND); 
        return mb_strtolower($name, $config_fe->charset);
    }
    
    public static function encode($name)
    {
        
        $from   = array('ä', 'ü', 'ö', 'Ä', 'Ü', 'Ö', 'ß');
        $to     = array('%C3%A4', '%C3%BC', '%C3%B6', '%C3%84', '%C3%9C', '%C3%96', '%C3%9F');
        
        return str_replace($from, $to, $name);
    }
    
    public static function decode($name)
    {
        
        $from = array('%C3%A4', '%C3%BC', '%C3%B6', '%C3%84', '%C3%9C', '%C3%96', '%C3%9F'); 
        $to   = array('ä', 'ü', 'ö', 'Ä', 'Ü', 'Ö', 'ß');
        
        return str_replace($from, $to, $name);
    }
}

