<?php
  /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Users.php 1542 2010-09-22 13:18:43Z ms $
 */
class Sicore_Models_Server_Db_Users extends Sicore_Models_Server_Db
{
    
    const ALL    = 0;
    const GROUPS = 1;
    const USERS  = 2;
    
    public function getByLogin($username, $password)
    {
        
        $sql = "
            SELECT DISTINCT u.*  
            FROM si_users u
            LEFT JOIN si_users_users u2u
              ON u.uid = u2u.users_uid_parent
            WHERE u.uid = (SELECT uid FROM si_users WHERE alias=? AND pass=?) OR u2u.users_uid_child = (SELECT uid FROM si_users WHERE alias=? AND pass=?)  
            ORDER BY u.level DESC
            ";
     
        if(!($rawUObj = $this->_con->fetchAll($sql, array($username, sha1($password), $username, sha1($password))))){
            return null;
        }
        
        return $this->_combineUserGroups($rawUObj);
    }
    
    public function getByUid($uid, $loadGroups = false)
    {
        if(!$loadGroups){
            $sql = "SELECT * FROM si_users WHERE uid = ".intval($uid)." LIMIT 1";
            
            if(!($uobj = $this->_con->fetchRow($sql))){
                return null;
            }
            
            if($uobj->isgroup){
                return new Sicore_Models_Uobj_Group($uobj);
            }else{
                return new Sicore_Models_Uobj_User($uobj); 
            }
        }else{
             $sql = "
                    SELECT DISTINCT u.*  
                    FROM si_users u
                    LEFT JOIN si_users_users u2u
                      ON u.uid = u2u.users_uid_parent
                    WHERE u.uid = (SELECT uid FROM si_users WHERE uid = ".intval($uid).") OR u2u.users_uid_child = (SELECT uid FROM si_users WHERE uid = ".intval($uid).")  
                    ORDER BY u.level DESC
                    ";
             
             if(!($rawUObj = $this->_con->fetchAll($sql))){
                   return null;
             }
                
             return $this->_combineUserGroups($rawUObj);
        }
    }
    
    public function getByParentUid($parentUid = null, $mode = self::ALL)
    {
        $sql  = "SELECT * 
                 FROM si_users";
        $sql .= intval($parentUid) ? " WHERE uid_parent = ".intval($parentUid) : " WHERE uid_parent = 0 OR ISNULL(uid_parent)";
        $sql .= " ORDER BY alias";
        
        if(!($rawUobjs = $this->_con->fetchAll($sql))){
            return array();
        }
        
        $uobjs = array(); 
        foreach($rawUobjs as $rawUbj){ 
            if($rawUbj->isgroup && ($mode == self::ALL || $mode == self::GROUPS)){
                $uobjs[] = new Sicore_Models_Uobj_Group($rawUbj);
            }elseif($mode == self::ALL || $mode == self::USERS){
                $uobjs[] = new Sicore_Models_Uobj_User($rawUbj); 
            } 
            
        }
        
        return $uobjs;           
    }
    
    protected function _combineUserGroups($rawUObj)
    {
        
        if($rawUObj[0]->isgroup){ 
            $uObj = new Sicore_Models_Uobj_Group($rawUObj[0]);
        }else{
            $uObj = new Sicore_Models_Uobj_User($rawUObj[0]); 
        }
        
        for($i = 1; $i < count($rawUObj); $i++){     
            if(!$rawUObj[$i]->isgroup) continue;
            $group = new Sicore_Models_Uobj_Group($rawUObj[$i]);
            $uObj->addParentGroup($group);       
        }
        
        return $uObj->isactive->value() ? $uObj : null;
    }
    
    /**
     * Ändert einen Benutzr / eine Gruppe ab
     *
     * @param Sicore_Models_Uobj $uObj 
     */
    public function update(Sicore_Models_Uobj $uObj)
    {
        $data = array();
        foreach($uObj as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            if($property->field() == 'uid_parent' && !$property->value()) continue;
            $data = $property->toDatabase($data);
        }
        
        $this->_con->update('si_users', $data, "uid = ".$uObj->uid->value());        
         
        return true;
    }
    
    public function create(Sicore_Models_Uobj_Group $group, Sicore_Models_Uobj $uObj){
        $uObj->uid_parent->set($group->uid->value());
        $uObj->level->set($group->level->value() + 1);
        $data = array();
        
        $this->_con->beginTransaction();
        try{
            //si_users eintragen
            foreach($uObj as $property){
                if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
                $data = $property->toDatabase($data);
            }       
            $this->_con->insert('si_users', $data); 
            $uObj->uid->set($this->_con->lastInsertId());
            
            //user_user eintragen  
            $sql = "INSERT INTO si_users_users  
                      (users_uid_child, users_uid_parent)
                    VALUES
                      (".intval($uObj->uid->value()).", ".intval($group->uid->value()).")
                    ";
            $this->_con->exec($sql);
            
            $parentGroups = $group->getAllParentGroups();
            
            foreach($parentGroups as $parentGroup){
                $sql = "INSERT INTO si_users_users 
                          (users_uid_child, users_uid_parent)
                        VALUES
                          (".intval($uObj->uid->value()).", ".intval($parentGroup->uid->value()).")
                        "; 
                $this->_con->exec($sql);
            }
            $this->_con->commit(); 
            return $uObj->uid->value();
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            return false;
        }
    }
    
    public function remove($uid, $checkOwnUid = null, $checkRootUser = false)
    {
        if($checkOwnUid){
            $sql = "SELECT users_uid_child 
                    FROM si_users_users 
                    WHERE users_uid_parent = ".intval($uid);
            $childs = $this->_con->fetchCol($sql); 
            if(in_array($checkOwnUid, $childs) || $checkOwnUid == $uid){
                throw new Sicore_Models_Server_Db_Exception_Useringroup();
            }
        }
        if($checkRootUser){
            $sql = "SELECT users_uid_child 
                    FROM si_users_users 
                    INNER JOIN si_users 
                        ON uid = users_uid_child
                    WHERE users_uid_parent = ".intval($uid)." OR uid = ".intval($uid)." AND isadmin = 1 AND level = 0  
                    LIMIT 1";
            $isRootAdmin = $this->_con->fetchOne($sql); 
            if($isRootAdmin){
                throw new Sicore_Models_Server_Db_Exception_Rootadmin();
            }
        }
        
        $sql = "DELETE FROM
                si_users
                WHERE uid IN(
                        SELECT users_uid_child 
                        FROM si_users_users 
                        WHERE users_uid_parent = ".intval($uid)."
                      ) 
                      OR uid = ".intval($uid);
        $this->_con->exec($sql);
        
        return true; 
    }
        
    public function getChildUids($uid)
    {
        $sql = "SELECT users_uid_child 
                FROM si_users_users 
                WHERE  users_uid_parent = ".intval($uid);
        $cols = $this->_con->fetchCol($sql); 
        return $cols;
    }
    
    public function getByEmail($email)
    {
        
        $sql = "
            SELECT DISTINCT u.*  
            FROM si_users u
            LEFT JOIN si_users_users u2u
              ON u.uid = u2u.users_uid_parent
            WHERE u.uid = (SELECT uid FROM si_users WHERE email=?) OR u2u.users_uid_child = (SELECT uid FROM si_users WHERE email=?)  
            ORDER BY u.level DESC
            ";

        if(!($rawUObj = $this->_con->fetchAll($sql, array($email, $email)))){
            return null;
        }
        
        return $this->_combineUserGroups($rawUObj);
    }
    
    public function getByAlias($email)
    {
        
        $sql = "
            SELECT DISTINCT u.*  
            FROM si_users u
            LEFT JOIN si_users_users u2u
              ON u.uid = u2u.users_uid_parent
            WHERE u.uid = (SELECT uid FROM si_users WHERE alias=?) OR u2u.users_uid_child = (SELECT uid FROM si_users WHERE alias=?)  
            ORDER BY u.level DESC
            ";

        if(!($rawUObj = $this->_con->fetchAll($sql, array($email, $email)))){
            return null;
        }
        
        return $this->_combineUserGroups($rawUObj);
    }
    
    public function move(Sicore_Models_Uobj $source, Sicore_Models_Uobj $dest, $mode)
    {
        $this->_con->beginTransaction();
        try{
            $oldLevel  = $source->level->value();
            $levelDiff = $dest->level->value() - $source->level->value(); 
            switch($mode){
                case 'after':
                case 'before':
                    $destUid = $dest->uid_parent->value();  
                    $source->level->set($dest->level->value());           
                    break;
                case 'inside':
                    if($dest instanceof Sicore_Models_Uobj_User) return false;
                    $destUid = $dest->uid->value();
                    $source->level->set($dest->level->value() + 1);
                    $levelDiff++; 
                    break;
                default:
                    return false; 
            }
            $source->uid_parent->set($destUid);
             
            $data = array();
            foreach($source as $property){
                if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
                if($property instanceof Sicore_Models_Node_Property_Password) continue;
                $data = $property->toDatabase($data);
            }
            
            $sql = "DELETE FROM si_users_users 
                    WHERE users_uid_child = ".intval($source->uid->value());
            $this->_con->exec($sql);
            $sql = "INSERT INTO si_users_users
                       (users_uid_parent, users_uid_child)
                    VALUES
                       (".intval($destUid).", ".intval($source->uid->value()).")";
            $this->_con->exec($sql);
            $sql = "INSERT INTO si_users_users
                       (users_uid_parent, users_uid_child)
                    SELECT users_uid_parent, ".intval($source->uid->value())." 
                    FROM si_users_users
                    WHERE users_uid_child = ".intval($destUid);
            $this->_con->exec($sql);
            $sql = "SELECT DISTINCT users_uid_child 
                    FROM si_users_users
                    WHERE users_uid_parent = ".intval($source->uid->value()); 
            $childs = $this->_con->fetchCol($sql);
            if($childs){  
                $sql = "DELETE si_users_users AS uu 
                        FROM si_users_users AS uu 
                        INNER JOIN si_users AS u ON uu.users_uid_parent=u.uid WHERE uu.users_uid_child IN(".implode(',', $childs).") AND u.level < ".intval($oldLevel); 
                $this->_con->exec($sql);
                foreach($childs as $child){
                    $sql = "INSERT INTO si_users_users 
                               (users_uid_parent, users_uid_child)
                            SELECT users_uid_parent, ".intval($child)." 
                            FROM si_users_users
                            WHERE users_uid_child = ".intval($destUid);
                    $this->_con->exec($sql); 
                    $sql = "INSERT INTO si_users_users 
                               (users_uid_parent, users_uid_child)
                            VALUES
                               (".intval($destUid).", ".intval($child).")";
                    $this->_con->exec($sql);
                } 
                $sql = "UPDATE si_users  
                        SET level = level + ".$levelDiff."
                        WHERE uid IN (".implode(',', $childs).")"; 
                $this->_con->exec($sql);          
            } 
            $this->_con->update('si_users', $data, 'uid = '.$source->uid->value()); 
            $this->_con->commit();    
        }catch(Exception $e){
            Sicore_Models_Logger::exception($e); 
            $this->_con->rollback();
            return false;
        }
        
        return true;
    }    
    
}

