<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Ftp.php 1267 2010-03-25 08:45:27Z ms $
 */
class Sicore_Models_Server_Ftp
{
    protected $_con = null;
    
    public function __construct($server, $user, $pass, $remotedir = '', $pasv = false, $ssl = false)
    {
        
        if(!function_exists('ftp_connect')){
            throw new Sicore_Models_Server_Ftp_Exception_Extension("FTP-EXTENSION NOT LOADED"); 
        }
         
        if($ssl){
            $this->_con = @ftp_ssl_connect($server, 21, 10);
        }else{
            $this->_con = @ftp_connect($server, 21, 10);
        }
        if(!$this->_con) throw new Sicore_Models_Server_Ftp_Exception_Host("FTP-CONNECT TO SERVER '".$server."' FAILED");

        if(!@ftp_login($this->_con, $user, $pass)){
            @ftp_close($this->_con);
            throw new Sicore_Models_Server_Ftp_Exception_Login("FTP-LOGIN FOR USER '".$user."' FAILED");
        } 
        
        @ftp_pasv($pasv);
        
        if($remotedir && !@ftp_chdir($this->_con, $remotedir)){
            throw new Sicore_Models_Server_Ftp_Exception_Remotedir("FTP-REMOTEDIR '".$remotedir."' NOT EXISTS");     
        }
        
        if(@ftp_size($this->_con, 'bootstrap.php') == -1){
           throw new Sicore_Models_Server_Ftp_Exception_Fileaccess("FTP-BOOTSTRAP-FILE NOT EXISTS");  
        }
    }
        
    public function send($src, $remotePath = '.')
    {
        if(!is_dir($src)){
            ftp_put($this->_con, $remotePath.'/'.basename($src), $src, FTP_BINARY); 
            return true;
        }
        
        $iterator = new DirectoryIterator($src);
        foreach ($iterator as $fileinfo) {
            if($fileinfo->getFilename() == '.' || $fileinfo->getFilename() == '..') continue;
            if (is_dir($fileinfo->getPathname())){
                if (!ftp_nlist($this->_con, $remotePath.'/'.$fileinfo->getFilename())) {
                   ftp_mkdir($this->_con, $remotePath.'/'.$fileinfo->getFilename());
               }
               $this->send($fileinfo->getPathname(), $remotePath.'/'.$fileinfo->getFilename());
            }else{
                ftp_put($this->_con, $remotePath.'/'.$fileinfo->getFilename(), $fileinfo->getPathname(), FTP_BINARY);
            }
        }
        
        return true;
    }
    
    public function chmod($file, $perms)
    {
        $perms = 0 . $perms;
        return ftp_site($this->_con, "CHMOD $perms $file");
    }
    

    
    public function close()
    {
        @ftp_close($this->_con);
    }
}
