<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Templates.php 1304 2010-04-20 14:49:12Z ms $
 */
class Sicore_Models_Server_Http_Templates extends Sicore_Models_Server_Http
{
    public function __construct($url, $maxredirects = 0, $timeout = 30)
    {
        parent::__construct($url, $maxredirects, $timeout);
        $this->_client->setMethod(Zend_Http_Client::POST); 
    }
    
    public function getCategories($parent)
    {
        $uri = substr(Sicore_Models_Helper_FSO::joinPath($this->_url, 'templates/categories'), 1);
        
        $this->_client->setUri($uri);
        $this->_client->setParameterPost(array(
            'i' => $this->_inumber,
            'h' => sha1($this->_inumber.$this->_secret),
            'parent' => $parent
        ));
        
        $response = $this->_client->request();
        if($response->isError()){
            Sicore_Models_Logger::error('Status '.$response->getStatus().': '.$response->getMessage().' (Update-Server: '.$uri.')');
            return false;
        }
        
        return Zend_Json::decode($response->getBody());
    }
    
    public function getCategory($uid)
    {
        $uri = substr(Sicore_Models_Helper_FSO::joinPath($this->_url, 'templates/category'), 1);
        
        $this->_client->setUri($uri);
        $this->_client->setParameterPost(array(
            'i' => $this->_inumber,
            'h' => sha1($this->_inumber.$this->_secret),
            'id' => $uid
        ));
        
        $response = $this->_client->request();
        if($response->isError()){
            Sicore_Models_Logger::error('Status '.$response->getStatus().': '.$response->getMessage().' (Update-Server: '.$uri.')');
            return array();
        }
        
        return Zend_Json::decode($response->getBody());
    }
    
    public function getByFolder($parent, $relase = null)
    {
        $uri = substr(Sicore_Models_Helper_FSO::joinPath($this->_url, 'templates/list'), 1);
        
        $this->_client->setUri($uri);
        $this->_client->setParameterPost(array(
            'i'       => $this->_inumber,
            'h'       => sha1($this->_inumber.$this->_secret),
            'release' => $relase,
            'parent'  => $parent
        ));
        
        $response = $this->_client->request();
        if($response->isError()){
            Sicore_Models_Logger::error('Status '.$response->getStatus().': '.$response->getMessage().' (Update-Server: '.$uri.')');
            return array();
        }
        
        return Zend_Json::decode($response->getBody());
    }
    
    public function getByUid($uid)
    {
        $uri = substr(Sicore_Models_Helper_FSO::joinPath($this->_url, 'templates/show'), 1);
        
        $this->_client->setUri($uri);
        $this->_client->setParameterPost(array(
            'i'  => $this->_inumber,
            'h'  => sha1($this->_inumber.$this->_secret),
            'id' => $uid
        ));
        
        $response = $this->_client->request();
        if($response->isError()){
            Sicore_Models_Logger::error('Status '.$response->getStatus().': '.$response->getMessage().' (Update-Server: '.$uri.')');
            return false;
        }
        
        return Zend_Json::decode($response->getBody());  
    }
    
    public function getLicence($uid)
    {
        $uri = substr(Sicore_Models_Helper_FSO::joinPath($this->_url, 'templates/licence'), 1);
        
        $this->_client->setUri($uri);
        $this->_client->setParameterPost(array(
            'i'  => $this->_inumber,
            'h'  => sha1($this->_inumber.$this->_secret),
            'id' => $uid
        ));
        
        $response = $this->_client->request();
        if($response->isError()){
            Sicore_Models_Logger::error('Status '.$response->getStatus().': '.$response->getMessage().' (Update-Server: '.$uri.')');
            return false;
        }
        
        return $response->getBody();
    } 
    
    public function download($uid, $save = true)
    {
        $uri = substr(Sicore_Models_Helper_FSO::joinPath($this->_url, 'templates/get'), 1);
        
        $this->_client->setUri($uri);
        $this->_client->setParameterPost(array(
            'i'  => $this->_inumber,
            'h'  => sha1($this->_inumber.$this->_secret),
            'id' => $uid
        ));
        
        $response = $this->_client->request();
        if($response->isError()){
            Sicore_Models_Logger::error('Status '.$response->getStatus().': '.$response->getMessage().' (Update-Server: '.$uri.')');
            return false;
        }
        
        if(!$save){
            return $response->getBody();
        }
        
        $path    = Sicore_Models_Helper_FSO::joinPathAbs('data/tmp');
        $tmpname = tempnam($path, 'template');
        file_put_contents($tmpname, $response->getBody());
        
        return  $tmpname;
    }   
}
