<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * ben?tigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sifront_Controllers
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: PageController.php 1557 2010-10-21 16:50:11Z ms $
 */
class PageController extends Sifront_Controllers_Simploo  
{
    const REDIRECT_TIMEOUT = 10;
    
    public function initAction()
    {
        $this->_helper->contextSwitch()->addActionContext('body', 'json'); 
    }
    
    public function indexAction()
    {
        $this->dispatch('showAction');
    }
    
   
    
    public function showAction()
    {
        
        $translate = Sicore_Models_Registry::getTranslate();  
        
        $config_backend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_BACKEND);  
        
        if(!($lang = $this->getRequest()->getParam('lang'))){
            $lang = $this->config_frontend->languages->{0};
            $this->getRequest()->setParam('lang', $lang); 
        }
        $isDraft = (bool)$this->getRequest()->getParam('draft',  false);
        
        if($isDraft && !Zend_Auth::getInstance()->hasIdentity()){
            $this->_redirector = $this->_helper->getHelper('Redirector');
            $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);  
            $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path;  
            $this->_redirector->gotoUrl($path.'sicore');
            die();
        }
             
        $uid   = intval($this->getRequest()->getParam('uid'));
        $nodeModel = new Sicore_Models_Server_Db_Nodes();   
          
        if(empty($uid)){
            
            $uid = $nodeModel->getRootUid($lang, $isDraft);
        } 
                             
        $node = $nodeModel->get($uid, $lang, $isDraft);
        $this->_checkPageAccess($node); 
        
        $counter = 1;
        
        $node_redirect = null;
         
        while($node->redirect->value() || $node->autoredirect->value()){
            if($counter >= self::REDIRECT_TIMEOUT) throw new Sifront_Models_Exception_Redirecttimeout();        
            if(($uid = $node->redirect->value())){
                if(($node_redirect = $nodeModel->get($uid, $lang, $isDraft))){
                    $node = $node_redirect;
                }
                $counter++; 
                continue;
            }
            if($node->autoredirect->value()){
                switch($node->autoredirect->value()){
                    case 1:
                        if(($node_redirect = $nodeModel->getFirstSub($node->uid->value(), $lang, $isDraft))){
                            $node = $node_redirect; 
                        }    
                        break; 
                    case 2:
                        if(($node_redirect = $nodeModel->getRandomSub($node->uid->value(), $lang, $isDraft))){
                            $node = $node_redirect;
                        }       
                        break;
                }  
                break;
            }
        }
        
        $this->_checkPageAccess($node);
        if($node instanceof Sicore_Models_Node_Page){
            $this->view->page    = $node;
            
            Sicore_Models_Registry::setPage($node);
            if(!$isDraft){
                $nodeModel->doCheckDate(true);
                $nodeModel->doCheckIsActive(true); 
                $nodeModel->doCheckIsHidden(true);
                $nodeModel->doCheckParents(false);
            }else{
                $nodeModel->doCheckIsActive(false); 
                $nodeModel->doCheckIsHidden(false);
                $nodeModel->doCheckDate(false);
            }
            $cache = Sicore_Models_Registry::getCache();
            if($node->pagesize->value()){
                $p   = intval($this->getRequest()->getParam('p'));
                $key = 'content_'.$node->uid->value().'_'.$p; 
            }else{
                $key = 'content_'.$node->uid->value(); 
            }
            
            if($isDraft || !($content = $cache->load($key))){
                $content = $nodeModel->getByParentUid($node->uid->value(), array('Sicore_Models_Node_Content'), array(), $isDraft);
                
                if(($pagesize = intval($node->pagesize->value()))){
                    $currentPage = intval($this->getRequest()->getParam('p'));
                    $content = Zend_Paginator::factory($content);
                    $content->setItemCountPerPage($pagesize);
                    $content->setCurrentPageNumber($currentPage);
                }
                
                $contentUid = null;
                if(!$isDraft) $cache->save($content, $key, array('content', (string)$node->uid->value())); 
            }
            $this->view->content = $content; 
            Sicore_Models_Registry::setContent($content); 
        }else{
            
            $this->view->content = $node;
            $contentUid          = $node->uid->value();
            Sicore_Models_Registry::setContent($node); 
            
            $this->view->page    = $nodeModel->getParentPage($node, $isDraft);
             
            Sicore_Models_Registry::setPage($this->view->page );   
        }
        
        if(($this->view->page instanceof Sicore_Models_Node_Page) && ($external = $this->view->page->external->value())){  
            header('Location: '.$external);
            exit();
        }     
         
        if(($template = $this->view->page->template->value())){
            $this->view->addScriptPath("templates/frontend/$template"); 
        }
     
        if(Zend_Auth::getInstance()->hasIdentity()){
            $this->view->user = Zend_Auth::getInstance()->getIdentity();
        }else{
            $this->view->user = null;
        }
 
        $path  = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path); 
        $this->view->headScript()->appendFile($path.'lib/JQuery/jquery-1.3.2.js');
        $this->view->headScript()->appendFile($path.'lib/JQuery/superbox/jquery.superbox.js');
        if(!$this->sico->checkLicence($this->se)){
            $this->view->headScript()->appendFile($path.'application/Sicore/Javascript/simploo/se.js');  
        }
        $this->view->headLink()->appendStylesheet($path.'lib/JQuery/superbox/jquery.superbox.css');
        $this->view->headLink()->appendStylesheet($path.'templates/frontend/sibase/_css/main.css');
        
        $script = "
            $.superbox.settings = {
                boxId: 'superbox', 
                overlayOpacity: .8, 
                loadTxt: '".$translate->_('GENERAL_LOAD')."', 
                closeTxt: '".$translate->_('GENERAL_CLOSE')."', 
                prevTxt: '".$translate->_('GENERAL_PREV')." ', 
                nextTxt: ' ".$translate->_('GENERAL_NEXT')."'
            };
            $(function(){
                $.superbox();
            });
        
        ";

        $config_content = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_CONTENT);
        foreach($config_content as $key => $content){
            if($content->javascript){
                $this->view->headScript()->appendFile($this->view->layout($content->javascript)); 
            }
            if($content->stylesheet){
                $this->view->headLink()->appendStylesheet($this->view->layout($content->stylesheet)); 
            }
        }
        
        $this->view->headScript()->appendScript($script);
        $this->view->isDraft = $isDraft;
        $this->view->path    = $this->config_base->website->path;
        
        if(($this->view->user && $this->view->user->gui_showpanel->value() || $this->view->user && $this->view->user->isadmin->value()) && !$this->getRequest()->getParam('template')){
            
            $this->view->canPublish = $this->view->page->canpublish->check($this->view->user) && ($this->view->page->getSubnodesVersion() || $this->view->page->version->value());
            
            $this->view->headScript()->appendFile($path.'lib/JQuery/ui/jquery-ui-1.7.2.custom.js'); 
            $this->view->headScript()->appendFile($path.'lib/JQuery/jquery.colorwheel.js'); 
            $this->view->headScript()->appendFile($path.'lib/JQuery/uploader/flashdetect.js');  
            $this->view->headScript()->appendFile($path.'lib/JQuery/jquery.formtrack.js');   
            $this->view->headScript()->appendFile($path.'application/Sicore/Javascript/simploo/jslib.js'); 
            $this->view->headScript()->appendFile($path.'application/Sicore/Javascript/init.js'); 
  
            $this->view->headScript()->appendFile($path.'application/Sicore/Javascript/simploo/gui.js');
            $this->view->headScript()->appendFile($path.'application/Sicore/Javascript/simploo/gui/content.js');
  
            $this->view->headScript()->appendFile($path.'lib/JQuery/colorpicker/colorpicker.js');
            $this->view->headLink()->appendStylesheet($path.'templates/backend/sibase/_themes/colorpicker/css/colorpicker.css');  
            
            if(($parent = $this->view->page->getParent())){
                $parentCanEditSubpages = $parent->caneditsubpages->check($this->view->user);
                $parentCanAddSubpages  = $parent->canaddsubpages->check($this->view->user);  
            }else{
                $parentCanEditSubpages = false;
                $parentCanAddSubpages  = false;
            }
            $script = "
                if(typeof simploo == 'undefined'){
                    var simploo ={};
                }
                simploo.canPasteContent = ".intval(($this->view->page->caneditcontent->check($this->view->user) || $parentCanEditSubpages) && $isDraft).";
                simploo.canPastePages = ".intval($parentCanAddSubpages && $isDraft).";
                simploo.canPasteSubpages = ".intval($this->view->page->canaddsubpages->check($this->view->user) && $isDraft).";
                simploo.baseurl       = '".$this->config_base->website->path."';
                simploo.basescript    = '".$this->config_base->website->script."'; 
                simploo.backendlayout = '".$config_backend->layout."';
                simploo.pageName      = '".base64_encode($this->view->page->title->value())."';
                simploo.isDraft       = '".$isDraft."';
                simploo.se            = '".(Sicore_Models_Core::lc($this->se) ? $this->se : '')."';  
                simploo.page          = '".$this->view->page->uid->value()."';
                simploo.pageAdded     = '".($this->view->page->version_status->value() == 'ADDED' ? '1' : '0')."';  
                simploo.content       = '".$contentUid."'; 
                simploo.language      = '".$this->view->page->language->value()."'; 
                simploo.pages_onclickopen    = '".$config_backend->jstree->pages->onclickopen."';
                simploo.media_onclickopen    = '".$config_backend->jstree->media->onclickopen."';
                simploo.users_onclickopen    = '".$config_backend->jstree->users->onclickopen."'; 
                simploo.design_onclickopen   = '".$config_backend->jstree->design->onclickopen."'; 
                simploo.settings_onclickopen = '".$config_backend->jstree->settings->onclickopen."';
                simploo.error_title   = '".$translate->_('GENERAL_ERROR_TITLE')."';
                simploo.error_msg   = '".$translate->_('GENERAL_ERROR_MSG')."';
                simploo.panel_spacing   = ".intval($config_backend->panel->spacing).";   
                
                if(top != self) top.location.href = self.location.href;
                ";
                
             $this->view->headScript()->appendScript($script); 
                        
        }
        
        $this->view->messages = new Sicore_Models_Helper_Messages();
        if(!$isDraft && $this->config_base->app->maintenance->status){
            throw new Sifront_Models_Exception_Temporarilyunavailable(); 
        }  
        if(($this->view->user && $this->view->user->gui_showpanel->value() || $this->view->user && $this->view->user->isadmin->value()) && $this->getRequest()->getParam('template')){
            $this->view->addScriptPath("templates/frontend/".$this->getRequest()->getParam('template'));
            $script = '
                $(document).ready(function(){ 
                    $(\'a\').click(function(){ return false; });
                }); 
            ';
            $this->view->headScript()->appendScript($script);   
        }
        $this->view->business = Sicore_Models_Core::isb($this->se);
        if(($format = $this->getRequest()->getParam('format'))){
            $this->_helper->viewRenderer('pagelayout-'.$format); 
        }else{
            $this->_helper->viewRenderer('pagelayout');
        }
    }
    
    protected function _checkPageAccess(Sicore_Models_Node $node)
    {
        
        if(!($accessList = $node->access->toArray())){
            return true;
        }
 
        if(!Zend_Auth::getInstance()->hasIdentity()){
            throw new Sifront_Models_Exception_Pageprotected();     
        }
        $user = Zend_Auth::getInstance()->getIdentity();
        if($user->isadmin->value()){
            return true;  
        }     
        foreach($accessList as $role){
            if(Sicore_Models_Registry::getAcl()->hasRole($role)){
                return true;
            }
        }
  
        throw new Sifront_Models_Exception_Pageprotected(); 
    } 
}