<?php
/**********************************************************************************
* Simploo CMS Installer
*
* License GNU/LGPL - Simploo GmbH  - Februar 2010
* http://www.www.simploo.de
*
* $Id: call.php 1499 2010-08-02 10:14:27Z ms $
************************************************************************************/
ob_start(); 
set_error_handler('error_handler');
if(!isset($_REQUEST['nos'])) register_shutdown_function('shutdown');
date_default_timezone_set('Europe/Berlin');

try{
    $path = realpath(dirname(__FILE__));
    $dirname = dirname($_SERVER['SCRIPT_NAME']);
    define('APPLICATION_PATH', substr($path, 0, strlen($path) - 7));
    define('APPLICATION_DIRNAME', substr($dirname, 0, strlen($dirname) - 8));
    $steps = array(1 => new Step_Welcome(), 2 => new Step_Systemcheck(), 3 => new Step_Install(), 4 => new Step_Database(), 5 => new Step_Configuration(), 6 => new Step_Licence(), 7 => new Step_Finished(),);
    $data = array();

    if (isset($_POST['step'])) {
        if ($_POST['checked']) {
            $data = $steps[$_POST['step'] + 1]->onLoad();
            $data['step'] = $_POST['step'] + 1;
        } else {
            $data = $steps[$_POST['step']]->onSubmit();
            $data['step'] = $_POST['step'];
        }
        if (function_exists('json_encode')) {
            die(json_encode($data));
        } else {
            die('{content: \'' . PHP_VERSION_FAILED . '\', checked: 0 }');
        }
    } else {
        if (isset($_GET['step']) && intval($_GET['step']) && isset($steps[$_GET['step']])) {
            $data = $steps[$_GET['step']]->onLoad();
            $data['step'] = $_GET['step'];
            if (function_exists('json_encode')) {
                die(json_encode($data));
            } else {
                die('{content: \'' . PHP_VERSION_FAILED . '\', checked: 0}');
            }
        }
    }
}catch(Exception $e){
    $content  = '<h1>Unerwarteter Fehler während der Installation</h1><h2>Bitte senden Sie die Fehlermeldung mit Nennung Ihrers Providers oder Ihrer Systemvoraussetzungen an support@simploo.de. Vielen Dank!</h2>';
    $content .= $e->__toString();
    if($_REQUEST['type'] == 'json'){
        die('{content: \'' . addslashes($content) . '\', checked: 0 }');
    }else{
        die($content);
    } 
}
      
function shutdown(){
    
    if ($error = error_get_last()){
        if(ob_get_length()) ob_end_clean();
        error_handler($error['type'], $error['message'], $error['file'], $error['line']);
    }   
}
  
function error_handler($code, $msg, $file, $line) {
    if ( 0 == error_reporting () ) {
        return;
    }
    switch($code){
        case 8:
        case 1024:
        case 2048:
        case 8192:
        case 16384:   
            $type = 'Hinweis'; 
            break;
        case 1:
        case 4:
        case 16:
        case 64: 
        case 256: 
        case 4096: 
            $type = 'Fehler';
            break;
        case 2:
        case 32:
        case 128: 
        case 512: 
            $type = 'Warnung';
            break;
        default:
           $type = 'Unbekannter Fehler';
            break;    
    }
    $content  = '<h1>Unerwarteter Fehler während der Installation</h1><h2>Bitte senden Sie die Fehlermeldung mit Nennung Ihrers Providers oder Ihrer Systemvoraussetzungen an support@simploo.de. Vielen Dank!</h2>';      
    $content .= "<p><strong>$type ($code) in Zeile $line in Datei $file:</strong></p><p>$msg</p>";
    
    if(isset($_REQUEST['type']) && $_REQUEST['type'] == 'json'){
        die('{content: \'' . addslashes($content) . '\', checked: 0 }');
    }else{
        die($content);
    } 
}


function initFramework() {
    $include_path = '';
    $include_path.= APPLICATION_PATH . '/lib/';
    $include_path.= PATH_SEPARATOR . APPLICATION_PATH . '/application/';
    $include_path.= PATH_SEPARATOR . get_include_path();
    set_include_path($include_path);
    require_once ('Zend/Loader/Autoloader.php');
    $autoloader = Zend_Loader_Autoloader::getInstance();
    $autoloader->registerNamespace('Sicore_');
    $autoloader->registerNamespace('Sifront_');
    $autoloader->registerNamespace('Sisnippets_');
    $autoloader->registerNamespace('Simploo');
    
}



class Step_Welcome {
    var $title = "Willkommen";
    function onLoad() {
        $data = array();
        $data['content'] = '
           <h1>Willkommen</h1>
           <p>Der Installations-Assistent führt Sie durch die Installation Ihres Simploo CMS. Folgen Sie den Anweisungen und überprüfen Sie gegebenenfalls die Meldungen und Hinweise. Sollten Sie technische Fragen haben, so wenden Sie sich bitte an Ihren Provider.</p>
           <p>Sollten Sie Probleme bei der Installation haben, bietet Ihnen die Simploo GmbH auch einen kostenpflichtigen Installationsservice an. Bitte informieren Sie sich darüber unter <a href="http://www.simploo.de" target="_new">www.simploo.de</a> oder stellen eine Anfrage unter <a href="mailto:support@simploo.de">support@simploo.de</a></p>
           ';
        @chmod(APPLICATION_PATH . '/install', 0755);  
        if (!is_writable(APPLICATION_PATH . '/install')) {
            $data['content'].= '<p class="error">Der install-Ordner auf Ihrem Webserver muss während <br/>der Installation beschreibbar sein.!!!</p>';
            $data['content'].= '<p>Bitte setzen Sie die Rechte mit einem beliebigen <br/>FTP-Programm auf "777" bevor Sie mit <br/>der Installation beginnen.</p>';
            $data['checked'] = 0;
        } else {
            $data['content'].= '<p><strong>Mit einem Klick auf "Weiter" starten Sie den Assistenten,<br/>bzw. kommen zum jeweils nächsten Schritt...</strong></p>';
            $data['checked'] = 1;
        } 
        return $data;
    }
    function onSubmit() {
        return $this->onLoad();
    }
}
class Step_Systemcheck {
    var $title = "Systemüberprüfung";
    function onLoad() {
        $data = array();
        $extensions = get_loaded_extensions();
        $error = false;
        $server_version = substr($_SERVER['SERVER_SOFTWARE'], 0, strpos($_SERVER['SERVER_SOFTWARE'], ' '));
        $server_version = empty($server_version) ? 'k.A.' : $server_version;
        $server_version_status = strpos($server_version, 'Apache') !== false ? 'ja' : 'optional';
        $php_version = phpversion();
        $php_extract = explode('.', $php_version);
        if ($php_extract[0] == 5 && $php_extract[1] >= 2) {
            $php_version_status = 'ja';
        } else {
            $php_version_status = 'nein';
            $error = true;
        }
        if (function_exists('gd_info')) {
            $gdlib = gd_info();
            $gdlib_info = $gdlib['GD Version'];
            if (strpos($gdlib_info, '2.') !== false) {
                $gdlib_info_status = 'ja';
            } else {
                $gdlib_info_status = 'nein';
                $error = true;
            }
        } else {
            $gdlib_info_status = 'nein';
            $error = true;
        }
        $ftp_version = in_array('ftp', $extensions) ? 'FTP' : '--';
        $ftp_version_status = in_array('ftp', $extensions) ? 'ja' : 'optional';
        $zip_version = phpversion('zip') ? 'ZIP ' . phpversion('zip') : '--';
        $zip_version_status = $zip_version ? 'ja' : 'optional';
        $mysql_version = phpversion('pdo') ? phpversion('pdo') : '--';
        $mysql_version_status = $mysql_version ? 'ja' : 'optional';
        if (function_exists('apache_get_modules')) {
            $modules = apache_get_modules();
            $modrewrite_version = in_array('mod_rewrite', $modules) ? 'mod_rewrite' : '--';
            if (in_array('mod_rewrite', $modules)) {
                $modrewrite_version_status = 'ja';
            } else {
                $modrewrite_version_status = 'nein';
                $error = true;
            }
        } else {
            $modrewrite_version = 'k.A.';
            $modrewrite_version_status = 'optional';
        }
        $data['content'] = '
            <h1>Systemüberprüfung</h1>
            <p>Folgende Systemvoraussetzungen stehen Ihnen zur Verfügung:</p>
            <table>
            <thead>
            <tr><th>Systemvoraussetzung</th><th>Verfügbar</th><th>Status</th></tr>
            </thead>
            <tbody>
            <tr><td>Apache Webserver</td><td>' . $server_version . '</td><td class="status ' . $server_version_status . '">' . $server_version_status . '</td></tr>
            <tr><td>PHP 5.2</td><td>PHP ' . $php_version . '</td><td class="status ' . $php_version_status . '">' . $php_version_status . '</td></tr>
            <tr><td>GDLib 2</td><td>Gdlib' . $gdlib_info . '</td><td class="status ' . $gdlib_info_status . '">' . $gdlib_info_status . '</td></tr>
            <tr><td>PHP FTP-Support</td><td>' . $ftp_version . '</td><td class="status ' . $ftp_version_status . '">' . $ftp_version_status . '</td></tr>
            <tr><td>PHP ZIP-Support</td><td>' . $zip_version . '</td><td class="status ' . $zip_version_status . '">' . $zip_version_status . '</td></tr>
            <tr><td>PHP MySql PDO</td><td>MySQL ' . $mysql_version . '</td><td class="status ' . $mysql_version_status . '">' . $mysql_version_status . '</td></tr>
            <tr><td>Apache Modul mod_rewrite</td><td>' . $modrewrite_version . '</td><td class="status ' . $modrewrite_version_status . '">' . $modrewrite_version_status . '</td></tr>
            </tbody>
            </table>        
        ';
        $data['checked'] = intval(!$error); 
        return $data;
    }
    function onSubmit() {
       
    }
}
class Step_Install {
    var $title = "Installation der Daten";
    var $writableDirs = array('config/custom', 'data/cache', 'data/files', 'data/logs', 'data/tmp', 'languages/custom', 'plugins', 'templates/frontend');
    function onLoad() {
        if (!$this->folderExists() || !file_exists(APPLICATION_PATH . '/index.php') || !file_exists(APPLICATION_PATH . '/bootstrap.php')) {
            return $this->showReleases();
        } else {
            return $this->checkFolders();
        }
    }
    function onSubmit() {
        if (isset($_POST['release']) && empty($_POST['error'])) {
            return $this->installRelease($_POST['release']);
        } elseif (isset($_POST['folders']) && empty($_POST['error'])) {
            return $this->checkFolders();
        } else {
            return $this->showReleases();
        }
    }
    function showReleases() {
        $data = array();
        $releases = array();
        if ($handle = opendir(APPLICATION_PATH)) {
            while (false !== ($file = readdir($handle))) {
                if (substr($file, 0, 7) == 'simploo' && substr($file, strlen($file) - 4) == '.zip') {
                    $releases[$file] = substr($file, 0, strlen($file) - 4);
                }
            }
        }
        $isInstalled = false;
        if (is_dir(APPLICATION_PATH . '/application') && is_dir(APPLICATION_PATH . '/lib') && is_dir(APPLICATION_PATH . '/templates') && file_exists(APPLICATION_PATH . '/index.php') && file_exists(APPLICATION_PATH . '/bootstrap.php')) $isInstalled = true;
        $data['content'] = '<h1>Installation der Daten</h1>';
        if (count($releases) > 0) {
            $data['content'].= '<p>Die Installationsdateien wurden noch nicht auf Ihrem Server entpackt.</p>';
            $data['content'].= '<p><strong>Es wurde/n ' . count($releases) . " Installationspaket/e auf Ihrem Server gefunden.</strong></p>";
            $data['content'].= '<p>Bitte wählen Sie das gewünschte Installationspaket aus und klicken Sie auf "Weiter". Das Entpacken der Daten kann einige Minuten in Anspruch nehmen. Bitte unterbrechen Sie den Vorgang nicht.</p>';
            $data['content'].= '<label for="si_release">Installationspaket</label>';
            $data['content'].= '<select name="release" id="si_release">';
            foreach($releases as $zip => $name) {
                $data['content'].= '<option value="' . $zip . '">' . $name . '</option>';
            }
            $data['content'].= '</select><br class="clear"/>';
            @chmod(APPLICATION_PATH, 0755);
            if (!is_writable(APPLICATION_PATH)) {
                $extensions = get_loaded_extensions();
                if (in_array('ftp', $extensions)) {
                    $data['content'].= '
                      <label for="ftp_user">FTP-Benutzer</label>
                      <input type="text" name="ftp_user" id="ftp_user" value="" /><br class="clear"/>
                      <label for="ftp_pass">FTP-Passwort</label>
                      <input type="text" name="ftp_pass" id="ftp_pass" value="" /><br class="clear"/>
                      <label for="ftp_dir">FTP-Verzeichnis</label>
                      <input type="text" name="ftp_dir" id="ftp_dir" value="" /><br class="clear"/>
                      <p class="hint">Bitte tragen Sie Ihren FTP-Benutzer <br/>und -Passwort von Ihrem Provider <br/>ein, um das Paket auf diesem Server <br/>entpacken zu können.</p>
                      <input style="display:none;" type="hidden" name="ftp" value="1"/>
                      ';
                } else {
                    $data['content'].= '<p class="error">Ihr Webserver-Ordner ist nicht beschreibbar und Sie haben<br/> keine PHP-FTP-Erweiterung zur Verfügung.</p>';
                    $data['content'].= '<p>Entpacken Sie das Installationspaket<br/> bitte auf Ihrem Rechner und laden die<br/> entpackten Dateien mit einem beliebigen<br/> FTP-Programm auf diesen Server hoch.</p>';
                    $data['content'].= '<input style="display:none;" type="hidden" name="error" value="1"/>';
                }
            }
        } elseif (!$isInstalled) {
            $data['content'].= '<p>Das Simploo CMS Daten wurde noch nicht auf dem Server entpackt und installiert.</p>';
            $data['content'].= '<p class="error">Es wurde kein Simploo CMS auf Installationspaket Ihrem Server gefunden.</p>';
            $data['content'].= '<p>Bitte laden Sie das gewünschte Simploo CMS Installationspaket von der <a href="http://www.simploo.de/cms-software-download" target="_new">Produktwebseite</a> herunter und laden die entsprechende ZIP-Datei per FTP auf diesen Server hoch.</p>';
            $data['content'].= '<p>Alternativ können Sie das Simploo CMS Installationspaket auch manuell <br/>entpacken und per FTP auf Ihren Server laden.';
        } else {
            $data['content'].= '<p>Das Simploo CMS Daten ist bereits auf dem Server entpackt</p>';
            $data['content'].= '<p>Mit dem nächsten Schritt können Sie die entsprechenden Ordnerberechtigungen überprüfen.</p>';
            $data['content'].= '<input style="display:none;" type="hidden" name="folders" value="1"/>';
        }
        $data['checked'] = 0;
        return $data;
    }
    function installRelease($release) {
        $data = array();
        if (!empty($_POST['ftp'])) {
            return $this->installReleaseFtp($release);
        }
        $data['content'] = '<h1>Installation der Daten</h1>';
        if (!file_exists(APPLICATION_PATH . '/' . $release)) {
            $data['content'].= '<p class="error">Das ausgewählte Installationspaket wurde nicht gefunden.</p>';
        } else {
            include_once (APPLICATION_PATH . 'install/pclzip.lib.php');
            $archive = new PclZip(APPLICATION_PATH . '/' . $release);
            if (is_writable(APPLICATION_PATH) && $archive->extract(PCLZIP_OPT_PATH, APPLICATION_PATH)) {
                @chmod(APPLICATION_PATH, 0755);
                return $this->checkFolders();
            } else {
                $data['content'].= '<p class="error">Das Installationspaket konnte nicht entpackt werden.</p>';
                $data['content'].= '<p>Das Webroot-Verzeichnis benötigt für diese Aktion vorübergehend Schreibrechte.</p>';
                $data['content'].= '<p>Setzen Sie diese Schreibrechte oder laden das entpackte Installationspaket manuell per FTP auf den Server.</p>';
            }
        }
        $data['checked'] = 0;
        return $data;
    }
    function installReleaseFtp($release) {
        $data = array();
        $con = @ftp_connect('localhost', 21, 10);
        if (!@ftp_login($con, $_POST['ftp_user'], $_POST['ftp_pass'])) {
            $data['content'] = '<h1>Installation der Daten</h1>';
            $data['content'].= '<p class="error">Fehlerhafte FTP-Zugangsdaten. Die Verbindung zum Server konnte nicht aufgebaut werden..</p>';
            $data['checked'] = 0;
            return $data;
        }
        if (!empty($_POST['ftp_dir'])) {
            if (@ftp_chdir($con, './' . $_POST['ftp_dir']) != 1) {
                $data['content'] = '<h1>Installation der Daten</h1>';
                $data['content'].= '<p class="error">Das FTP-Verzeichnis wurde nicht gefunden.</p>';
                $data['checked'] = 0;
                return $data;
            }
        }
        if (@ftp_size($con, 'install.php') <= 0) {
            $data['content'] = '<h1>Installation der Daten</h1>';
            $data['content'].= '<p class="error">Der "install"-Ordner wurde nicht im angegebenen FTP-Verzeichnis gefunden.</p>';
            $data['checked'] = 0;
            return $data;
        }
        $data['content'] = '<h1>Installation der Daten</h1>';
        if (!file_exists(APPLICATION_PATH . '/install/release/index.php') || !file_exists(APPLICATION_PATH . '/install/release/bootstrap.php')) {
            include_once (APPLICATION_PATH . 'install/pclzip.lib.php');
            $archive = new PclZip(APPLICATION_PATH . '/' . $release);
            if (!@mkdir(APPLICATION_PATH . '/install/release')) {
                $data['content'].= '<p class="error">Das Installationspaket konnte nicht entpackt werden.</p>';
                $data['content'].= '<p>Das Webroot-Verzeichnis benötigt für diese Aktion vorübergehend Schreibrechte.</p>';
                $data['content'].= '<p>Setzen Sie diese Schreibrechte oder laden das entpackte Installationspaket manuell per FTP auf den Server.</p>';
                $data['checked'] = 0;
                return $data;
            }
            $archive->extract(PCLZIP_OPT_PATH, APPLICATION_PATH . '/install/release');
        }
        $contents = ftp_nlist($con, 'install/release');
        @ftp_chmod($con, 0755, './');
        foreach($contents as $content) {
            if ($content == 'install/release/.' || $content == 'install/release/..') continue;
            $dest = str_replace('install/release/', '', $content);
            if ($dest == 'install') continue;
            if (ftp_size($con, $dest) > 0) continue;
            @chmod(APPLICATION_PATH . '/' . $content, 0755);
            if (!ftp_rename($con, $content, $dest)) {
                $data['content'].= '<p class="error">Das Installationspaket konnte nicht entpackt werden.</p>';
                $data['content'].= '<p>Das Webroot-Verzeichnis benötigt für diese Aktion vorübergehend Schreibrechte.</p>';
                $data['content'].= '<p>Setzen Sie diese Schreibrechte oder laden das entpackte Installationspaket manuell per FTP auf den Server.</p>';
                $data['checked'] = 0;
                return $data;
            }
        }
        @chmod(APPLICATION_PATH . '/install/release/install/data.sql', 0755);
        @ftp_rename($con, 'install/release/install/data.sql', 'install/data.sql');
        @chmod(APPLICATION_PATH . '/install/release/install/demo.sql', 0755);
        @ftp_rename($con, 'install/release/install/demo.sql', 'install/demo.sql');
        @ftp_chmod($con, 0755, 'data/files');
        @ftp_chmod($con, 0755, 'data/logs');
        @ftp_chmod($con, 0755, 'data/tmp');
        @ftp_chmod($con, 0755, 'data/cache');
        @ftp_chmod($con, 0755, 'config/custom');
        @ftp_chmod($con, 0755, 'languages/custom');
        @ftp_chmod($con, 0755, 'templates/frontend');
        @ftp_chmod($con, 0755, './');
        return $this->checkFolders();
    }
    function checkFolders() {
        $data = array();
        $isError = false;
        $content = '';
        foreach($this->writableDirs as $dir) {
            $doExists = 'ja';
            $isWritale = 'ja';
            if (!is_dir(APPLICATION_PATH . '/' . $dir)) $doExists = 'nein';
            @chmod(APPLICATION_PATH . '/' . $dir, 0755);
            if (!is_writeable(APPLICATION_PATH . '/' . $dir)) $isWritale = 'nein';
            $content.= '
                <tr>
                <td>' . $dir . '</td>
                <td class="status ' . $doExists . '">' . $doExists . '</td>
                <td class="status ' . $isWritale . '">' . $isWritale . '</td>
                </tr>
            ';
            if ($doExists == 'nein' || $isWritale == 'nein') $isError = true;
        }
        $data['content'] = '<h1>Installation der Daten</h1>';
        if ($isError) {
            $data['content'].= '<p class="error">Die Simploo CMS Dateien wurde korrekt auf Ihrem Server entpackt. Allerdings sind einige Ordner nicht vorhanden, bzw. nicht beschreibbar.</p><p>Korrigieren Sie die Fehler mit einem beliebigen FTP-Programm.</p>';
        } else {
            $data['content'].= '<p class="ok">Die Simploo CMS Dateien wurde korrekt auf Ihrem Server entpackt und alle benötigten Ordner sind beschreibbar.</p>';
        }
        $data['content'].= '
                <table>
                <thead>
                <tr><th>Ordner</th><th>vorhanden</th><th>beschreibbar</th></tr>
                </thead>
                <tbody>
            ' . $content;
        $data['checked'] = $isError;
        return $data;
    }
    function folderExists() {
        foreach($this->writableDirs as $dir) {
            if (!is_dir(APPLICATION_PATH . '/' . $dir)) return false;
        }
        return true;
    }
}
class Step_Database {
    var $title = "Datenbankverbindung konfigurieren";
    function onLoad() {
        initFramework();
        $db_server = 'localhost';
        $db_user = '';
        $db_pass = '';
        $db_name = '';
        if (file_exists(APPLICATION_PATH . '/config/custom/base.ini.php')) {
            $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php');
            if ($config->database && $config->database->params) {
                $db_server = $config->database->params->host ? $config->database->params->host : 'localhost';
                $db_user = $config->database->params->username;
                $db_pass = $config->database->params->password;
                $db_name = $config->database->params->dbname;
            }
        }
        $data = array();
        $data['content'] = '<h1>Einrichtung der Datenbank</h1>';
        $data['content'].= '<p>Bitten geben Sie Ihre Zugangsdaten zur Datenbank ein. Die entsprechenden Daten erhalten Sie von/bei Ihrem Provider.</p>';
        $data['content'].= $this->form($db_server, $db_user, $db_pass, $db_name, true);
        $data['checked'] = 0;
        return $data;
    }
    function onSubmit() {
        $data = array();
        initFramework();
        if (file_exists(APPLICATION_PATH . '/config/custom/base.ini.php')) {
            $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
            $data = $config->toArray();
        } else {
            $data = array();
        }
        $data['database']['params']['host']     = isset($_POST['db_server']) ? $_POST['db_server'] : 'localhost';
        $data['database']['params']['username'] = isset($_POST['db_user']) ? $_POST['db_user'] : '';
        $data['database']['params']['password'] = isset($_POST['db_pass']) ? $_POST['db_pass'] : '';
        $data['database']['params']['dbname']   = isset($_POST['db_name']) ? $_POST['db_name'] : '';
        try {
            $writer = new Simploo_Config_Writer_Ini();
            $writer->write(APPLICATION_PATH . '/config/custom/base.ini.php', new Zend_Config($data, true));
        }
        catch(Zend_Config_Exception $e) {
            $data['content'] = '<h1>Einrichtung der Datenbank</h1>';
            $data['content'].= '<p class="error">Die Zugangsdaten konnten nicht gespeichert werden.</p>';
            $data['content'].= $this->form(isset($_POST['db_server']) ? $_POST['db_server'] : 'localhost', isset($_POST['db_user']) ? $_POST['db_user'] : '', isset($_POST['db_pass']) ? $_POST['db_pass'] : '', isset($_POST['db_name']) ? $_POST['db_name'] : '', isset($_POST['db_demo']) ? $_POST['db_demo'] : '');
            $data['checked'] = 0;
            return $data;
        }
        $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
        $config->database->adapter = 'PDO_MYSQL';
        try {
            $db = Zend_Db::factory($config->database);
            $db->query("SET NAMES 'utf8'");
            if (isset($_POST['import']) && $_POST['import'] == 1) $this->doImport();
            if (empty($_POST['db_name'])) {
                $databases = $db->fetchCol("SHOW DATABASES");
                $datbases2 = array();
                foreach($databases as $database) {
                    $tables = count($db->fetchCol("SHOW TABLES FROM `" . $database . "`"));
                    $datbases2[$database] = $database . ' (' . $tables . ')';
                }
                $data['content'] = '<h1>Einrichtung der Datenbank</h1>';
                $data['content'].= '<p class="ok">Die Datenbankverbindung wurde erfolgreich hergestellt.</p>';
                $data['content'].= '<p>Bitte wählen Sie eine Datenbank aus der Liste aus.</p>';
                $data['content'].= $this->form(isset($_POST['db_server']) ? $_POST['db_server'] : 'localhost', isset($_POST['db_user']) ? $_POST['db_user'] : '', isset($_POST['db_pass']) ? $_POST['db_pass'] : '', $datbases2, isset($_POST['db_demo']) ? $_POST['db_demo'] : '');
                $data['checked'] = 0;
                return $data;
            } else {
                $data['content'] = '<h1>Einrichtung der Datenbank</h1>';
                $data['content'].= '<p class="ok">Die Datenbankverbindung wurde erfolgreich hergestellt und kann nun verwendet werden.</p>';
                $tables = $db->fetchCol("SHOW TABLES FROM `" . $_POST['db_name'] . "`");
                if (count($tables) > 0 && !in_array('si_tree', $tables)) {
                    $data['content'].= '<p class="hint">Die ausgewählte Datenbank ist nicht leer. Es könnte bei der Installation der Daten zu Problemen kommen. Mit dem nächsten Schritt wird das Simploo CMS in Ihre Datenbank importiert.</p>';
                    $data['content'].= '<input style="display:none;" type="hidden" name="import" value="1" id="import" />';
                    $data['checked'] = 0;
                } elseif (count($tables) > 0 && in_array('si_tree', $tables)) {
                    $data['content'].= '<p class="ok">Das Simploo CMS wurde erfolgreich in Ihre Datenbank importiert.</p>';
                    $data['checked'] = 1;
                } else {
                    $data['content'].= '<p class="hint">Die Datenbank ist leer. Mit dem nächsten Schritt wird das Simploo CMS in Ihre Datenbank importiert.</p>';
                    $data['content'].= '<input style="display:none;" type="hidden" name="import" value="1" id="import" />';
                    $data['checked'] = 0;
                }
                $data['content'].= $this->form(isset($_POST['db_server']) ? $_POST['db_server'] : 'localhost', isset($_POST['db_user']) ? $_POST['db_user'] : '', isset($_POST['db_pass']) ? $_POST['db_pass'] : '', isset($_POST['db_name']) ? $_POST['db_name'] : '', isset($_POST['db_demo']) ? $_POST['db_demo'] : '', isset($data['checked']) ? $data['checked'] : '');
                return $data;
            }
        }
        catch(Exception $e) {
            $data['content'] = '<h1>Einrichtung der Datenbank</h1>';
            $data['content'].= '<p class="error">Die Verbindung zur Datenbank konnte nicht aufgebaut werden. Bitte übperprüfen Sie Ihre Zugangsdaten.</p>';
            $data['content'].= '<p>' . $e->getMessage() . '</p>';
            $data['content'].= $this->form(isset($_POST['db_server']) ? $_POST['db_server'] : 'localhost', isset($_POST['db_user']) ? $_POST['db_user'] : '', isset($_POST['db_pass']) ? $_POST['db_pass'] : '', isset($_POST['db_name']) ? $_POST['db_name'] : '', isset($_POST['db_demo']) ? $_POST['db_demo'] : '');
            $data['checked'] = 0;
            return $data;
        }
    }
    function doImport() {
        if (!file_exists(APPLICATION_PATH . '/install/data.sql')) return false;
        initFramework();
        try {
            $content = file_get_contents(APPLICATION_PATH . '/install/data.sql');
            $sql = explode(';', $content);
            $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
            $config->database->adapter = 'PDO_MYSQL';
            $db = Zend_Db::factory($config->database);
            $db->query("SET NAMES 'utf8'");
            foreach($sql as $query) {
                $query = str_replace("\n", "", $query);
                $query = str_replace("\r", "", $query);
                if (empty($query)) continue;;
                $db->query($query);
            }
          
            if (isset($_POST['db_demo']) && $_POST['db_demo'] == 1 && file_exists(APPLICATION_PATH . '/install/demo.sql')) {
                $content = file_get_contents(APPLICATION_PATH . '/install/demo.sql');
                $sql = explode(';', $content);
                foreach($sql as $query) {
                    $query = str_replace("\n", "", $query);
                    $query = str_replace("\r", "", $query);
                    if (empty($query)) continue;
                    $db->query($query); 
                }
            }
            return true;
        }
        catch(Exception $e) {
            echo $e->__toString();
            return false;
        }
    }
    function form($db_server = 'localhost', $db_user = '', $db_pass = '', $db_name = '', $db_demo = false, $closed = false) {
        $content = '
                <label for="db_host">DB-Server</label>
                <input type="text" name="db_server" id="db_server" value="' . $db_server . '" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                <label for="db_user">DB-Benutzer</label>
                <input type="text" name="db_user" id="db_user" value="' . $db_user . '" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                <label for="db_pass">DB-Passwort</label>
                <input type="text" name="db_pass" id="db_pass" value="' . $db_pass . '" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                <label for="db_name">DB-Name</label>';
        if (is_array($db_name)) {
            $content.= '<select name="db_name" id="db_name" ' . ($closed ? ' disabled = "disabled" ' : '') . '>';
            $content.= '<option value="">::: bitte Datenbank auswählen :::</option>';
            foreach($db_name as $key => $name) {
                if (is_integer($key)) {
                    $content.= '<option>' . $name . '</option>';
                } else {
                    $content.= '<option value="' . $key . '">' . $name . '</option>';
                }
            }
            $content.= '</select><br class="clear"/>';
        } else {
            $content.= '<input type="text" name="db_name" id="db_name" value="' . $db_name . '" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>';
        }
       
        
        $content.= '
            <p>Wollen Sie zum Einstieg gleich einige <br/>Standard-Webseiten anlegen lassen?</p>
            <label for="db_demo">Webseiten anlegen</label>
            <input style="width:auto;" type="checkbox" name="db_demo" id="db_demo" value="1" ' . ($db_demo ? 'checked="checked"' : '') . ' ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                
        ';
        return $content;
    }
}
class Step_Configuration {
    var $title = "Zugangsdaten konfigurieren";
    function onLoad($error = false) {
        initFramework();
        $sitename = '';
        $user = '';
        $admin = '';
        $path = '';
        $path = APPLICATION_DIRNAME;
        $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
        $config->database->adapter = 'PDO_MYSQL';
        $db = Zend_Db::factory($config->database);
        $db->query("SET NAMES 'utf8'");
        $sitename = $db->fetchOne("SELECT string1 FROM si_nodes WHERE uid = 1 LIMIT 1");
        $user = $db->fetchOne("SELECT alias   FROM si_users WHERE uid = 2 LIMIT 1");
        $data = array();
        $data['content'] = '<h1>Zugangsdaten konfigurieren</h1>';
        $data['content'].= '<p>Bitte vervollständigen Sie die folgende Informationen. Sollten bereits Eintragungen vorhanden sein, können diese im Normalfall übernommen werden.</p>';
        if ($error) {
            $data['content'].= '<p class="error">Sie müssen mindestens einen Seitennamen und den Administrationsbenutzer mit Passwort eingeben.</p>';
        }
        $data['content'].= $this->form($sitename, $user, $admin, $path);
        $data['checked'] = 0;
        return $data;
    }
    function onSubmit()
    {
        if(empty($_POST['sitename']) || empty($_POST['user']) || empty($_POST['pass'])){
            return $this->onLoad(true);
        }
        
        try{
            initFramework();
            $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
            $config->database->adapter = 'PDO_MYSQL';
            $db = Zend_Db::factory($config->database);
            $db->query("SET NAMES 'utf8'");
            $db->update('si_nodes',   array('string1' => $_POST['sitename']),   'uid = 1');           
            $db->update('si_folders', array('name' =>    $_POST['sitename']),   'uid = 1');
            $db->update('si_users',   array('alias' =>   $_POST['sitename']),   'uid = 1');
            $db->update('si_users',   array('alias' =>   $_POST['user'], 'pass' => sha1($_POST['pass'])), 'uid = 2');
            
            $data   = $config->toArray();
            
            $data['website']['path']   = isset($_POST['path']) ? $_POST['path'] : '';
            $data['website']['script'] = 'index.php';  
            try{
                $writer = new Simploo_Config_Writer_Ini();
                $writer->write(APPLICATION_PATH . '/config/custom/base.ini.php', new Zend_Config($data, true));
            }catch(Zend_Config_Exception $e){
                $data['content']  = '<h1>Zugangsdaten konfigurieren</h1>';
                $data['content'] .= '<p class="error">Die Zugangsdaten konnten nicht gespeichert werden.</p>';
                $data['content'] .= $this->form($_POST['sitename'], $_POST['user'], null, $_POST['path']); 
                $data['checked'] = 0;
                return $data;    
            }
             
            $data['admin'] = $_POST['user'];
            $data['pass']  = $_POST['pass'];
            
            $data['content']  = '<h1>Zugangsdaten konfigurieren</h1>';
            
            $data['content'] .= '<p class="ok">Die Zugangsdaten wurden erfolgreich gespeichert.</p>';
            $data['checked'] = 1;
            return $data;
        }catch(Exception $e){
            return $this->onLoad(true); 
        } 
    }
    function form($sitename = '', $user = '', $admin = '', $path = '', $closed = false) {
        $sitename = empty($sitename) ? $_SERVER['HTTP_HOST'] : $sitename;
        $content = '
                <h2>Allgemeine Einstellungen</h2>
                <label for="sitename">Seitenname</label>
                <input type="text" name="sitename" id="sitename" value="' . $sitename . '" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                <p style="font-size:10px;">Sollten Sie das Simploo CMS in einem Unterordner auf Ihrem Webserver<br/> (z.B. www.domain.de/simploo) hochgeladen haben, dann müssen Sie diesen Unterordner <br/>(z.B. simploo) im folgenden Feld eingeben. Ansonsten lassen Sie es bitte leer.</p> 
                <label for="path">Unterordner</label>
                <input type="text" name="path" id="path" value="' . $path . '" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                <h2>Administrations-Benutzer</h2>
                <label for="user">Benutzer</label>
                <input type="text" name="user" id="user" value="' . $user . '" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                <label for="pass">Passwort</label>
                <input type="text" name="pass" id="pass" value="" ' . ($closed ? ' disabled = "disabled" ' : '') . ' /><br class="clear"/>
                ';
        return $content;
    }
}
class Step_Licence {
    var $title = "Seriennummer/Lizenz aktivieren";
    function onLoad($error = false) {
        initFramework();
        $data = array();
        $data['content'] = '<h1>Seriennummer/Lizenz aktivieren</h1>';
        $data['content'].= '<p>Sie können zu Test- und Entwicklungszwecken die Simploo CMS Testversion auch ohne Seriennummer betreiben, bzw. können die Seriennummer später aktivieren.</p>';
        $data['content'].= '<p class="hint">Sobald Sie die Seite aber Online verfügbar machen, müssen Sie eine gültige Seriennummer (Community oder Business Edition) aktiviert haben.</p>';
        if ($error) {
            $data['content'].= '<p class="error">Sie haben einen ungültigen Aktivierungscode eingegeben.</p>';
        }
        $data['content'].= $this->form();
        $data['checked'] = 0;
        return $data;
    }
    function onSubmit() {
        initFramework();
        $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/system/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
        $data = array();
        if (empty($_POST['code'])) {
            $data['content'] = '<h1>Seriennummer/Lizenz aktivieren</h1>';
            $data['content'].= '<p class="hint">Sobald Sie die Seite Online verfügbar machen, müssen Sie eine gültige Seriennummer (Community oder Business Edition) aktiviert haben.</p>';
            $data['content'].= '
                <label for="path">Release</label>
                <input type="text" name="release" id="release" value="' . $config->app->release . '" disabled = "disabled" /><br class="clear"/>
                <label for="path">Seriennummer</label>
                <input type="text" name="serial" id="serial" value="--" disabled = "disabled" /><br class="clear"/> 
                <label for="path">Version</label>
                <input type="text" name="version" id="version" value="Developer Edition" disabled = "disabled" /><br class="clear"/>     
            ';
            $data['checked'] = 1;
            return $data;
        } else {
            Sicore_Models_Core::$config = $config;
            if (Sicore_Models_Core::ac($_POST['code']) <= 0) {
                return $this->onLoad(true);
            }
            $translate = new Zend_Translate('array', APPLICATION_PATH . '/languages/system/de.php', 'de');
            $codes = Sicore_Models_Core::ae($_POST['code']);
            if (file_exists(APPLICATION_PATH . '/config/custom/base.ini.php')) {
                $config_cust = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
                $data = $config_cust->toArray();
            } else {
                $data = array();
            }
            $data['app']['number']      = $codes[0];
            $data['app']['serial']      = $codes[1];
            $data['customer']['secret'] = $codes[2];
            try {
                $writer = new Simploo_Config_Writer_Ini();
                $writer->write(APPLICATION_PATH . '/config/custom/base.ini.php', new Zend_Config($data, true));
            }
            catch(Zend_Config_Exception $e) {
                die("test");
                return $this->onLoad(true);
            }
            $data['content'] = '<h1>Seriennummer/Lizenz aktivieren</h1>';
            $data['content'].= '<p class="ok">Ihre Seriennummer/Lizenz wurde erfolgreich aktiviert.</p>';
            $data['content'].= '
                <label for="path">Release</label>
                <input type="text" name="release" id="release" value="' . $config->app->release . '" disabled = "disabled" /><br class="clear"/>
                <label for="path">Seriennummer</label>
                <input type="text" name="serial" id="serial" value="' . $codes[1] . '" disabled = "disabled" /><br class="clear"/>
                <label for="path">Version</label>
                <input type="text" name="version" id="version" value="' . $translate->_('GENERAL_E' . Sicore_Models_Core::le($codes[1])) . '" disabled = "disabled" /><br class="clear"/>     
            '; 
            $data['checked'] = 1;
            return $data;
        }
    }
    function form() {
        $content = '
                <h2>Seriennummerjetzt aktivieren</h2>
                <p>Kopieren Sie Ihren Aktivierungscode in das folgenden Textfeld<br/>und klicken Sie auf "Weiter"...</p>
                <label for="sitename">Aktivierungscode</label>
                <textarea name="code" id="code" value=""></textarea><br class="clear"/>
                ';
        $content .= '<ul style="margin-left:25px;"><li><a target="lizenz" href="http://www.simploo.de/cms-kostenlos-edition/lizenz-beantragen">kostenlose Lizenz für<br/> Community Edition beantragen</a></li>';
        $content .= '<li><a target="lizenz" href="http://www.simploo.de/cms-firmen-edition/lizenz-kaufen">Lizenz für Business Edition kaufen</a></li></ul>';
            
        return $content;
    }
}
class Step_Finished {
    var $title = "Installation erfolgreich abgeschlossen";
    function onLoad($error = false) {
        initFramework();
        $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/base.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
        $path = Sicore_Models_Helper_FSO::joinPath($config->website->path);
        $path = $config->website->script ? $path.$config->website->script.'/' : $path; 
        $data = array();
        $data['content'] = '<h1>Herzlichen Glückwunsch</h1>';
        $data['content'].= '<p class="ok">Sie haben das Simploo CMS erfolgreich auf Ihrem Server gespeichert.</p>';
        @Sicore_Models_Helper_FSO::deleteDir(APPLICATION_PATH . 'install');
        
        if(file_exists(APPLICATION_PATH.'/install/call.php')){
            $data['content'].= '<p class="error">BITTE LÖSCHEN SIE AUS SICHERHEITSGRÜNDEN DEN "install"-ORDNER AUF IHREM SERVER!</p>';
        }
        $data['content'].= '
            <h2>Ihre Simploo CMS Webseite</h2>
            <p><a href="http://' . $_SERVER['HTTP_HOST'] . $path . '">' . $_SERVER['HTTP_HOST'] . $path . '</a></p>
            <h2>Ihre Simploo CMS Administrationsbereich</h2>
            <p><a href="http://' . $_SERVER['HTTP_HOST'] . $path . 'sicore">' . $_SERVER['HTTP_HOST'] . $path . 'sicore</a></p>
            <label for="sitename">Benutzer</label>
            <input type="text" name="name" id="name" value="' . $_POST['admin'] . '" disabled = "disabled" /><br class="clear"/>
            <label for="sitename">Passwort</label>
            <input type="text" name="pass" id="pass" value="' . $_POST['pass'] . '" disabled = "disabled" /><br class="clear"/> 
            ';
        $data['content'] .= '<a href="javascript: self.print();">Seite ausdrucken</a>';
        
        if(file_exists(APPLICATION_PATH.'/install.php')){
            $installData = "<?php header('Location: " . $path . "');?>";
            if (!@file_put_contents(APPLICATION_PATH . 'install.php', $installData)) {
                $data['content'].= '<p class="hint">Bitte löschen Sie dringend die Datei<br/> "install.php" auf Ihrem Server!</p>';
            }
        }
        $data['checked'] = 1;
        $data['finished'] = 1;
        return $data;
    }
}
ob_end_flush(); 


?>
