<?php
 
/**
 * Zend Framework
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @category   Zend
 * @package    Zend_Validate
 * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id: PasswordExists.php 1294 2010-03-31 17:48:59Z ms $
 */  
  
  
/**
 * @see Zend_Validate_Db_Abstract
 */  
require_once 'Zend/Validate/Db/Abstract.php';  
  
/**
 * Confirms a record exists in a table.
 * 
 * @category   Zend
 * @package    Zend_Validate
 * @uses       Zend_Validate_Db_Abstract
 * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */  
class Simploo_Validate_Db_PasswordExists extends Zend_Validate_Db_Abstract  
{ 
    const PASSWORD_NOTCORRECT   = 'recordFound'; 
    
    protected $_messageTemplates = array(self::PASSWORD_NOTCORRECT => 'Incorrect Password'); 
    
    public function isValid($value) 
    { 
        
        $valid = true; 
        $this->_setValue($value); 
         
        $result = $this->_query($value); 
        if (!$result) { 
            $valid = false; 
            $this->_error(self::PASSWORD_NOTCORRECT); 
        } 
         
        return $valid; 
    } 
    
    protected function _query($value) 
    { 
        /**
         * Check for an adapter being defined. if not, fetch the default adapter.
         */ 
           
        if($this->_adapter === null) {
            $this->_adapter = Zend_Db_Table_Abstract::getDefaultAdapter();
        }
         
        /**
         * Build select object
         */ 
        $select = new Zend_Db_Select($this->_adapter);
        $select->from($this->_table)
               ->columns($this->_field)
               ->where($this->_adapter->quoteIdentifier($this->_field).' = ?', sha1($value)); 
        if ($this->_exclude !== null) { 
            if (is_array($this->_exclude)) { 
                $select->where($this->_adapter->quoteIdentifier($this->_exclude['field']).' = ?', $this->_exclude['value']); 
            } else { 
                $select->where($this->_exclude); 
            } 
        } 
        $select->limit(1);       
        /**
         * Run query
         */ 
        
        $result = $this->_adapter->fetchRow($select, array(), Zend_Db::FETCH_ASSOC);  
        return $result; 
    } 
}
